@@ -27,6 +27,7 @@ from xarray.coding.cftime_offsets import (
 from xarray.coding.cftimeindex import CFTimeIndex
 from xarray.core.resample import DataArrayResample
 
+from xclim.core.formatting import update_xclim_history
 from xclim.core.utils import DayOfYearStr, _calc_perc
 
 # cftime and datetime classes to use for each calendar name
@@ -340,7 +341,7 @@ def _convert_datetime(
     datetime: Union[pydt.datetime, cftime.datetime],
     new_doy: Optional[Union[float, int]] = None,
     calendar: str = "default",
-) -> Union[cftime.datetime, pydt.datetime, np.int]:
+) -> Union[cftime.datetime, pydt.datetime, float]:
     """Convert a datetime object to another calendar.
 
     Nanosecond information are lost as cftime.datetime doesn't support them.
@@ -439,6 +440,7 @@ def days_in_year(year: int, calendar: str = "default") -> int:
     )
 
 
+@update_xclim_history
 def percentile_doy(
     arr: xr.DataArray,
     window: int = 5,
@@ -464,6 +466,7 @@ def percentile_doy(
       The percentiles indexed by the day of the year.
       For calendars with 366 days, percentiles of doys 1-365 are interpolated to the 1-366 range.
     """
+
     # Ensure arr sampling frequency is daily or coarser
     # but cowardly escape the non-inferrable case.
     if compare_offsets(xr.infer_freq(arr.time) or "D", "<", "D"):
@@ -502,7 +505,14 @@ def percentile_doy(
         p = adjust_doy_calendar(p.sel(dayofyear=(p.dayofyear < 366)), arr)
 
     p.attrs.update(arr.attrs.copy())
-    return p
+
+    # Saving percentile attributes
+    n = len(arr.time)
+    p.attrs["climatology_bounds"] = (
+        arr.time[0 :: n - 1].dt.strftime("%Y-%m-%d").values.tolist()
+    )
+    p.attrs["window"] = window
+    return p.rename("per")
 
 
 def compare_offsets(freqA: str, op: str, freqB: str):  # noqa
@@ -530,8 +540,8 @@ def compare_offsets(freqA: str, op: str, freqB: str):  # noqa
     from xclim.indices.generic import get_op
 
     # Get multiplicator and base frequency
-    t_a, b_a, _ = parse_offset(freqA)
-    t_b, b_b, _ = parse_offset(freqB)
+    t_a, b_a, _, _ = parse_offset(freqA)
+    t_b, b_b, _, _ = parse_offset(freqB)
 
     if b_a == b_b:
         # Same base freq, compare mulitplicator only.
@@ -547,12 +557,12 @@ def compare_offsets(freqA: str, op: str, freqB: str):  # noqa
     return get_op(op)(t_a, t_b)
 
 
-def parse_offset(freq: str) -> Tuple[str, str, Optional[str]]:
+def parse_offset(freq: str) -> Tuple[str, str, Optional[str], Optional[str]]:
     """Parse an offset string.
 
-    Returns: multiplicator, offset base, anchor (or None)
+    Returns: multiplicator, offset base, start stamp (or None), anchor (or None)
     """
-    patt = r"(\d*)(\w)S?(?:-(\w{2,3}))?"
+    patt = r"(\d*)(\w)(S)?(?:-(\w{2,3}))?"
     return re.search(patt, freq.replace("Y", "A")).groups()
 
 
@@ -786,13 +796,15 @@ def time_bnds(group, freq):
 
     Examples
     --------
-    >>> import xarray as xr
+    >>> from xarray import cftime_range
     >>> from xclim.core.calendar import time_bnds
-    >>> index = xr.cftime_range(start='2000-01-01', periods=3, freq='2QS', calendar='360_day')
-    >>> time_bnds(index, '2Q')
-    ((cftime.Datetime360Day(2000, 1, 1, 0, 0, 0, 0), cftime.Datetime360Day(2000, 3, 30, 23, 59, 59, 999999)),
-    (cftime.Datetime360Day(2000, 7, 1, 0, 0, 0, 0), cftime.Datetime360Day(2000, 9, 30, 23, 59, 59, 999999)),
-    (cftime.Datetime360Day(2001, 1, 1, 0, 0, 0, 0), cftime.Datetime360Day(2001, 3, 30, 23, 59, 59, 999999)))
+    >>> index = cftime_range(start='2000-01-01', periods=3, freq='2QS', calendar='360_day')
+    >>> out = time_bnds(index, '2Q')
+    >>> for bnds in out:
+    ...     print(bnds[0].strftime("%Y-%m-%dT%H:%M:%S"), ' -', bnds[1].strftime("%Y-%m-%dT%H:%M:%S"))
+    2000-01-01T00:00:00  - 2000-03-30T23:59:59
+    2000-07-01T00:00:00  - 2000-09-30T23:59:59
+    2001-01-01T00:00:00  - 2001-03-30T23:59:59
     """
     if isinstance(group, CFTimeIndex):
         cfindex = group
@@ -899,8 +911,9 @@ def doy_to_days_since(
 
     Examples
     --------
+    >>> from xarray import DataArray
     >>> time = date_range('2020-07-01', '2021-07-01', freq='AS-JUL')
-    >>> da = xr.DataArray([190, 2], dims=('time',), coords={'time': time})  # July 8th 2020 and Jan 2nd 2022
+    >>> da = DataArray([190, 2], dims=('time',), coords={'time': time})  # July 8th 2020 and Jan 2nd 2022
     >>> doy_to_days_since(da, start='10-02').values  # Convert to days since Oct. 2nd, of the data's year.
     array([-86, 92])
     """
@@ -956,8 +969,9 @@ def days_since_to_doy(
 
     Examples
     --------
+    >>> from xarray import DataArray
     >>> time = date_range('2020-07-01', '2021-07-01', freq='AS-JUL')
-    >>> da = xr.DataArray(
+    >>> da = DataArray(
             [-86, 92], dims=('time',), coords={'time': time}, attrs={'units': 'days since 10-02'}
         )
     >>> days_since_to_doy(da).values
