@@ -5,13 +5,15 @@ Formatting utilities for indicators
 ===================================
 """
 import datetime as dt
+import itertools
 import re
 import string
 from ast import literal_eval
 from fnmatch import fnmatch
-from typing import Dict, Mapping, Optional, Sequence, Union
+from typing import Callable, Dict, Mapping, Optional, Sequence, Union
 
 import xarray as xr
+from boltons.funcutils import wraps
 
 from .utils import InputKind
 
@@ -34,9 +36,11 @@ class AttrFormatter(string.Formatter):
         mapping : Mapping[str, Sequence[str]]
             A mapping from values to their possible variations.
         modifiers : Sequence[str]
-            The list of modifiers, must be the as long as the longest value of `mapping`.
+            The list of modifiers, must be the as long as the longest value of `mapping`. Cannot include reserved modifier 'r'.
         """
         super().__init__()
+        if "r" in modifiers:
+            raise ValueError("Modifier 'r' is reserved for default raw formatting.")
         self.modifiers = modifiers
         self.mapping = mapping
 
@@ -44,8 +48,8 @@ class AttrFormatter(string.Formatter):
         """Format a value given a formatting spec.
 
         If `format_spec` is in this Formatter's modifiers, the corresponding variation
-        of value is given. If `format_spec` is not specified but `value` is in the
-        mapping, the first variation is returned.
+        of value is given. If `format_spec` is 'r' (raw), the value is returned unmodified.
+        If `format_spec` is not specified but `value` is in the mapping, the first variation is returned.
 
         Examples
         --------
@@ -54,9 +58,9 @@ class AttrFormatter(string.Formatter):
         In french, the genre of the noun changes the adjective (cat = chat is masculine,
         and goose = oie is feminine) so we initialize the formatter as:
 
-        >>> fmt = AttrFormatter({'nice': ['beau', 'belle'], 'evil' : ['méchant', 'méchante']}, ['m', 'f'])
-        >>> fmt.format("Le chien est {adj1:m}, l'oie est {adj2:f}", adj1='nice', adj2='evil')
-        "Le chien est beau, l'oie est méchante"
+        >>> fmt = AttrFormatter({'nice': ['beau', 'belle'], 'evil' : ['méchant', 'méchante'], 'smart': ['intelligent', 'intelligente']}, ['m', 'f'])
+        >>> fmt.format("Le chien est {adj1:m}, l'oie est {adj2:f}, le gecko est {adj3:r}", adj1='nice', adj2='evil', adj3='smart')
+        "Le chien est beau, l'oie est méchante, le gecko est smart"
 
         The base values may be given using unix shell-like patterns:
 
@@ -74,6 +78,8 @@ class AttrFormatter(string.Formatter):
             raise ValueError(
                 f"No known mapping for string '{value}' with modifier '{format_spec}'"
             )
+        elif format_spec == "r":
+            return super().format_field(value, "")
         return super().format_field(value, format_spec)
 
     def _match_value(self, value):
@@ -87,10 +93,12 @@ class AttrFormatter(string.Formatter):
 # Tag mappings between keyword arguments and long-form text.
 default_formatter = AttrFormatter(
     {
+        # Arguments to "freq"
         "YS": ["annual", "years"],
         "AS-*": ["annual", "years"],
         "MS": ["monthly", "months"],
         "QS-*": ["seasonal", "seasons"],
+        # Arguments to "indexer"
         "DJF": ["winter"],
         "MAM": ["spring"],
         "JJA": ["summer"],
@@ -108,6 +116,12 @@ default_formatter = AttrFormatter(
         "m10": ["october"],
         "m11": ["november"],
         "m12": ["december"],
+        # Arguments to "op / reducer"
+        "mean": ["average"],
+        "max": ["maximal", "maximum"],
+        "min": ["minimal", "minimum"],
+        "sum": ["total", "sum"],
+        "std": ["standard deviation"],
     },
     ["adj", "noun"],
 )
@@ -309,6 +323,84 @@ def update_history(
     return merged_history
 
 
+def update_xclim_history(func):
+    """Decorator that auto-generates and fills the history attribute.
+
+    The history is generated from the signature of the function and added to the first output.
+    """
+
+    @wraps(func)
+    def _call_and_add_history(*args, **kwargs):
+        """Call the function and then generate and add the history attr."""
+        outs = func(*args, **kwargs)
+
+        if isinstance(outs, tuple):
+            out = outs[0]
+        else:
+            out = outs
+
+        if not isinstance(out, (xr.DataArray, xr.Dataset)):
+            raise TypeError(
+                f"Decorated `update_xclim_history` received a non-xarray output from {func.__name__}."
+            )
+
+        da_list = [arg for arg in args if isinstance(arg, xr.DataArray)]
+        da_dict = {
+            name: arg for name, arg in kwargs.items() if isinstance(arg, xr.DataArray)
+        }
+
+        attr = update_history(
+            gen_call_string(func.__name__, *args, **kwargs),
+            *da_list,
+            new_name=out.name,
+            **da_dict,
+        )
+        out.attrs["history"] = attr
+        return outs
+
+    return _call_and_add_history
+
+
+def gen_call_string(funcname: str, *args, **kwargs):
+    """Generate a signature string for use in the history attribute.
+
+    DataArrays and Dataset are replaced with their name, floats, ints and strings are
+    printed directly, all other objects have their type printed between < >.
+
+    Arguments given through *args are printed positionnally and those given through
+    **kwargs are printed prefixed by their name.
+
+    Parameters
+    ----------
+    funcname : str
+      Name of the function
+    *args, **kwargs
+      Arguments given to the function.
+
+    Example
+    -------
+    >>> A = xr.DataArray([1], dims=('x',), name='A')
+    >>> gen_call_string("func", A, b=2.0, c="3", d=[4, 5, 6])
+    "func(A, b=2.0, c='3', d=<list>)"
+    """
+    elements = []
+    chain = itertools.chain(zip([None] * len(args), args), kwargs.items())
+    for name, val in chain:
+        if isinstance(val, xr.DataArray):
+            rep = val.name or "<array>"
+        elif isinstance(val, (int, float, str, bool)):
+            rep = repr(val)
+        else:
+            rep = f"<{type(val).__name__}>"
+
+        if name is not None:
+            rep = f"{name}={rep}"
+
+        elements.append(rep)
+
+    return f"{funcname}({', '.join(elements)})"
+
+
 def prefix_attrs(source, keys, prefix):
     """Rename some of the keys of a dictionary by adding a prefix.
 
