@@ -31,6 +31,7 @@ Indicator-defining yaml files are structured in the following way:
     indices:
       <identifier>:
         base: <base indicator class>  # Defaults to module-wide base class or "Daily".
+                                      # If the name startswith a '.', the base class is taken from the current module (thus an indicator declared _above_)
         realm: <realm>  # Defaults to the module-wide realm or "atmos"
         reference: <references>
         references: <references>  # Plural or singular accepted (for harmonizing clix-meta and xclim)
@@ -38,13 +39,7 @@ Indicator-defining yaml files are structured in the following way:
         notes: <notes>
         title: <title>
         abstract: <abstract>
-        period:  # If given, both "allowed" and "default" must also be given.
-          allowed:  # A list of allowed periods (resampling frequencies)
-            annual:  # Translates to "A" (includes "Y")
-            seasonal:  # Translates to "Q"
-            monthly:  # Translates to "M"
-            weekly:  # Translates to "W"
-          default: annual  #  Translates to "YS", "QS-DEC", "MS" or "W-SUN". See xclim.core.units.FREQ_NAMES.
+        period: annual  # Becomes the default value for `freq`. Translates to "YS", "QS-DEC", "MS" or "W-SUN". See xclim.core.units.FREQ_NAMES.
         output:
           var_name: <var_name>  # Defaults to "identifier",
           standard_name: <standard_name>
@@ -57,9 +52,11 @@ Indicator-defining yaml files are structured in the following way:
             ...
 
         index_function:
-          name: <function name>  # Refering to a function in xclim.indices.generic or xclim.indices
+          name: <function name>  # Refering to a function in the passed indices module, xclim.indices.generic or xclim.indices
+          # When using Indicator.from_dict, the "name" field can also be a function (instead of a string)
           parameters:  # See below for details on that section.
-            <param name>  # Refering to a parameter of the function above.
+            <param name>: <param data>  # Simplest case when only injecting is needed.
+            <param name>:  # Most complex case where we want to change parameters metadata. Also retrocompatible with clix-meta.
               kind: <param kind>  # Optional, one of quantity, operator or reducer
               data: <param data>
               units: <param units>
@@ -99,7 +96,6 @@ the mapping of `data.input` simply links a variable name from the function in `d
 to one of those official variables.
 
 """
-import logging
 import re
 import warnings
 import weakref
@@ -119,23 +115,33 @@ from yaml import safe_load
 from .. import indices
 from . import datachecks
 from .calendar import parse_offset
-from .cfchecks import generate_cfcheck
+from .cfchecks import cfcheck_from_name
 from .formatting import (
     AttrFormatter,
     default_formatter,
+    gen_call_string,
     generate_indicator_docstring,
     merge_attributes,
     parse_cell_methods,
     parse_doc,
     update_history,
 )
-from .locales import TRANSLATABLE_ATTRS, get_local_attrs, get_local_formatter
-from .options import MISSING_METHODS, MISSING_OPTIONS, OPTIONS
+from .locales import (
+    TRANSLATABLE_ATTRS,
+    get_local_attrs,
+    get_local_formatter,
+    load_locale,
+    read_locale_file,
+)
+from .options import METADATA_LOCALES, MISSING_METHODS, MISSING_OPTIONS, OPTIONS
 from .units import FREQ_NAMES, convert_units_to, declare_units, units
 from .utils import (
+    VARIABLES,
+    InputKind,
     MissingVariableError,
     infer_kind_from_parameter,
-    variables,
+    load_module,
+    raise_warn_or_log,
     wrapped_partial,
 )
 
@@ -258,11 +264,6 @@ class Indicator(IndicatorRegistrar):
     or parse all available instances.
 
     """
-
-    #: Number of input DataArray variables. Should be updated by subclasses if needed.
-    #: This number sets which inputs are passed to the tests.
-    nvar = 1
-
     # Allowed metadata attributes on the output variables
     _cf_names = [
         "var_name",
@@ -357,6 +358,10 @@ class Indicator(IndicatorRegistrar):
         # We dump whatever the base class had and take what was parsed from the current compute function.
         kwds["parameters"] = params
 
+        # By default skip missing values handling if there is no resampling.
+        if "freq" not in params:
+            kwds["missing"] = "skip"
+
         # Parse kwds to organize cf_attrs
         # Must be done after parsing var_name
         # And before converting callables to staticmethods
@@ -428,10 +433,6 @@ class Indicator(IndicatorRegistrar):
         data: dict,
         identifier: str,
         module: Optional[str] = None,
-        realm: Optional[str] = None,
-        keywords: Optional[str] = None,
-        references: Optional[str] = None,
-        notes: Optional[str] = None,
     ):
         """Create an indicator subclass and instance from a dictionary of parameters.
 
@@ -444,11 +445,6 @@ class Indicator(IndicatorRegistrar):
         module : str
           The module name of the indicator. This is meant to be used only if the indicator
           is part of a dynamically generated submodule, to override the module of the base class.
-        realm: str, optional
-        keywords: str, optional
-        references str, optional
-        notes: str, optional
-          Other indicator attributes to fill in for missing values in the individual definition.
         """
         # Make cell methods. YAML will generate a list-of-dict structure, put it back in a space-divided string
         if data.get("output", {}).get("cell_methods") is not None:
@@ -460,34 +456,32 @@ class Indicator(IndicatorRegistrar):
         if "input" in data:
             # Override input metadata
             input_units = {}
-            nvar = len(data["input"])
             for varname, name in data["input"].items():
                 # Indicator's new will put the name of the variable as its default,
                 # we override this with the real variable name.
-                # Also take the dimensionaliy and description from the yaml of official variables.
+                # Also take the canonical units and description from the yaml of official variables.
                 # Description overrides the one parsed from the generic compute docstring
-                # Dimensionality goes into the declare_units wrapper.
+                # Canonical units go into the declare_units wrapper.
                 params[varname] = {
                     "default": name,
-                    "description": variables[name]["description"],
+                    "description": VARIABLES[name]["description"],
                 }
-                input_units[varname] = variables[name]["dimensionality"]
-
-            cfcheck = generate_cfcheck(*[varname for varname in data["input"].values()])
+                input_units[varname] = VARIABLES[name]["canonical_units"]
         else:
-            nvar = None
-            cfcheck = None
             input_units = None
 
         metadata_placeholders = {}
         if "index_function" in data:
             # Generate compute function
             # data.index_function.name refers to a function in xclim.indices.generic or xclim.indices (in this order of priority).
+            # It can also directly be a function.
             # data.index_function.parameters is a list of injected arguments.
             funcname = data["index_function"].get("name")
             if funcname is None:
                 # No index function given, reuse the one from the base class.
                 compute = cls.compute
+            elif callable(funcname):
+                compute = funcname
             else:
                 compute = getattr(
                     indices.generic, funcname, getattr(indices, funcname, None)
@@ -500,8 +494,11 @@ class Indicator(IndicatorRegistrar):
             injected_params = {}
             # In clix-meta, when there are no parameters, the key is still there with a None value.
             for name, param in (data["index_function"].get("parameters") or {}).items():
+                if not isinstance(param, dict):
+                    # Simplest case for injecting, passing a value directly.
+                    value = param
                 # Handle clix-meta cases
-                if param.get("kind") == "quantity" and isinstance(
+                elif param.get("kind") == "quantity" and isinstance(
                     param["data"], (str, int, float)
                 ):
                     # A string with units, but not a placeholder (where data is a dict)
@@ -517,8 +514,14 @@ class Indicator(IndicatorRegistrar):
                     # User-chosen parameter. placeholder.
                     # It should be a string, this is a bug from clix-meta.
                     value = list(value.keys())[0]
+                    # Get default from parent class if possible.
+                    default = (
+                        getattr(cls, "parameters", {})
+                        .get(name, {})
+                        .get("default", None)
+                    )
                     params[name] = {
-                        "default": param.get("default"),
+                        "default": param.get("default", default),
                         "description": param.get(
                             "description", param.get("standard_name", name)
                         ),
@@ -542,22 +545,31 @@ class Indicator(IndicatorRegistrar):
             compute = None
 
         # Allowed resampling frequencies
+        allowed_periods = None
         if "period" in data:
-            params["freq"] = {"default": FREQ_NAMES[data["period"]["default"]][1]}
-            allowed_periods = []
-            for period_name in data["period"]["allowed"]:
-                allowed_periods.append(FREQ_NAMES[period_name][0])
-        else:
-            allowed_periods = None
+            if isinstance(data["period"], str):
+                deffreq = data["period"]
+            elif "default" in data["period"]:
+                # old-clix-meta version where multiple allowed values can be listed.
+                # Kept in xclim.
+                deffreq = data["period"]["default"]
+            else:
+                # clix-meta when a special season specification exists : xclim doesn't support it.
+                deffreq = list(data["period"].keys())[0]
+            params["freq"] = {"default": FREQ_NAMES[deffreq][1]}
+            if "allowed" in data["period"]:
+                allowed_periods = []
+                for period_name in data["period"]["allowed"]:
+                    allowed_periods.append(FREQ_NAMES[period_name][0])
 
         kwargs = dict(
             # General
             identifier=identifier,
             module=module,
-            realm=data.get("realm", realm),
-            keywords=data.get("keywords", keywords),
-            references=data.get("references", data.get("reference", references)),
-            notes=data.get("notes", notes),
+            realm=data.get("realm"),
+            keywords=data.get("keywords"),
+            references=data.get("references", data.get("reference")),
+            notes=data.get("notes"),
             # Indicator-specific metadata
             title=data.get("title"),
             abstract=data.get("abstract"),
@@ -571,10 +583,8 @@ class Indicator(IndicatorRegistrar):
             cell_methods=cell_methods,
             # Input data, override defaults given in generic compute's signature.
             parameters=params or None,  # None if an empty dict
-            nvar=nvar,
             compute=compute,
             # Checks
-            cfcheck=cfcheck,
             allowed_periods=allowed_periods,
         )
 
@@ -585,7 +595,7 @@ class Indicator(IndicatorRegistrar):
 
         # Remove kwargs passed as "None", they will be taken from the base class instead.
         # For most parameters it would be ok to pass a None anyway (we figure that out in __new__),
-        # but some (like nvar) would not like that.
+        # but some would not like that.
         return cls(**{k: v for k, v in kwargs.items() if v is not None})
 
     def __init__(self, **kwds):
@@ -614,27 +624,23 @@ class Indicator(IndicatorRegistrar):
         # For convenience
         n_outs = len(self.cf_attrs)
 
-        # Bind call arguments to `compute` arguments and set defaults.
-        ba = self._sig.bind(*args, **kwds)
-        ba.apply_defaults()
-
-        # Assign inputs passed as strings from ds.
-        self._assign_named_args(ba)
-
-        # Assume the first arguments are always the DataArrays.
-        # Only the first nvar inputs are checked (data + cf checks)
-        das = OrderedDict()
-        for name in self._parameters[: self.nvar]:
-            das[name] = ba.arguments.pop(name)
+        # Put the variables in `das`, parse them according to the annotations
+        # das : OrderedDict of variables (required + non-None optionals)
+        # params : OrderedDict of parameters INCLUDING unpacked kwargs
+        # all_params: OrderedDict of parameters with PACKED kwargs <- this is needed by _update_attrs and _mask because of `indexer`.
+        #                  AND includes injected arguments <- this is needed by update_attrs and missing (when "freq" is injected)
+        das, params, all_params = self._parse_variables_from_call(args, kwds)
 
         # Metadata attributes from templates
         var_id = None
         var_attrs = []
         for attrs in self.cf_attrs:
             if n_outs > 1:
-                var_id = f"{self._registry_id}.{attrs['var_name']}"
+                var_id = attrs["var_name"]
             var_attrs.append(
-                self._update_attrs(ba, das, attrs, names=self._cf_names, var_id=var_id)
+                self._update_attrs(
+                    all_params.copy(), das, attrs, names=self._cf_names, var_id=var_id
+                )
             )
 
         # Pre-computation validation checks on DataArray arguments
@@ -644,17 +650,19 @@ class Indicator(IndicatorRegistrar):
         # Check if the period is allowed:
         if (
             self.allowed_periods is not None
-            and "freq" in kwds
-            and parse_offset(kwds["freq"])[1] not in self.allowed_periods
+            and "freq" in all_params
+            and parse_offset(all_params["freq"])[1] not in self.allowed_periods
         ):
             raise ValueError(
-                f"Resampling frequency {kwds['freq']} is not allowed for indicator {self.identifier} (needs something equivalent to one of {self.allowed_periods})."
+                f"Resampling frequency {all_params['freq']} is not allowed for indicator {self.identifier} (needs something equivalent to one of {self.allowed_periods})."
             )
 
         # Compute the indicator values, ignoring NaNs and missing values.
-        outs = self.compute(**das, **ba.kwargs)
+        outs = self.compute(**das, **params)
+
         if isinstance(outs, DataArray):
             outs = [outs]
+
         if len(outs) != n_outs:
             raise ValueError(
                 f"Indicator {self.identifier} was wrongly defined. Expected {n_outs} outputs, got {len(outs)}."
@@ -672,10 +680,11 @@ class Indicator(IndicatorRegistrar):
             out.attrs.update(attrs)
             out.name = var_name
 
-        # Mask results that do not meet criteria defined by the `missing` method.
-        # This means all variables must have the same dimensions...
-        mask = self._mask(*das.values(), **ba.arguments)
-        outs = [out.where(~mask) for out in outs]
+        if self.missing != "skip":
+            # Mask results that do not meet criteria defined by the `missing` method.
+            # This means all outputs must have the same dimensions as the broadcasted inputs (excluding time)
+            mask = self._mask(*das.values(), **all_params)
+            outs = [out.where(~mask) for out in outs]
 
         # Return a single DataArray in case of single output, otherwise a tuple
         if n_outs == 1:
@@ -686,8 +695,13 @@ class Indicator(IndicatorRegistrar):
         """Assign inputs passed as strings from ds."""
         ds = ba.arguments.pop("ds")
         for name, param in self._sig.parameters.items():
-            if param.annotation is Union[str, DataArray] and isinstance(
-                ba.arguments[name], str
+            if (
+                self.parameters[name]["kind"]
+                in (
+                    InputKind.VARIABLE,
+                    InputKind.OPTIONAL_VARIABLE,
+                )
+                and isinstance(ba.arguments[name], str)
             ):
                 if ds is not None:
                     try:
@@ -701,6 +715,37 @@ class Indicator(IndicatorRegistrar):
                         f"Passing variable names as string requires giving the `ds` dataset (got {name}='{ba.arguments[name]}')"
                     )
 
+    def _parse_variables_from_call(self, args, kwds):
+        """Extract variable and optional variables from call arguments."""
+        # Bind call arguments to `compute` arguments and set defaults.
+        ba = self._sig.bind(*args, **kwds)
+        ba.apply_defaults()
+
+        # Assign inputs passed as strings from ds.
+        self._assign_named_args(ba)
+
+        das = OrderedDict()
+        for name, param in self.parameters.items():
+            kind = param["kind"]
+            # If a variable pop the arg
+            if kind in (InputKind.VARIABLE, InputKind.OPTIONAL_VARIABLE):
+                data = ba.arguments.pop(name)
+                # If a non-optional variable OR None, store the arg
+                if kind == InputKind.VARIABLE or data is not None:
+                    das[name] = data
+
+        # Remove **kwargs from bind object and put all those params in "kwargs" to be passed to compute.
+        params = ba.arguments.copy()
+        for param in self._sig.parameters.values():
+            if param.kind == param.VAR_KEYWORD:
+                kwargs = params.pop(param.name)
+                params.update(**kwargs)
+
+        # Add injected kwargs to the all_params
+        all_params = ba.arguments
+        all_params.update(getattr(self._indcompute, "_injected", {}))
+        return das, params, all_params
+
     def _bind_call(self, func, **das):
         """Call function using `__call__` `DataArray` arguments.
 
@@ -733,23 +778,50 @@ class Indicator(IndicatorRegistrar):
             return func(*ba.args, **ba.kwargs)
 
     @classmethod
-    def _update_attrs(cls, ba, das, attrs, var_id=None, names=None):
+    def _get_translated_metadata(
+        cls, locale, var_id=None, names=None, append_locale_name=True
+    ):
+        """Get raw translated metadata for the curent indicator and a given locale.
+
+        All available translated metadata from the current indicator and those it is based on are merged,
+        with highest priority to the current one.
+        """
+        var_id = var_id or ""
+        if var_id:
+            var_id = "." + var_id
+
+        family_tree = []
+        cl = cls
+        while hasattr(cl, "_registry_id"):
+            family_tree.append(cl._registry_id + var_id)
+            # The indicator mechanism always has single inheritance.
+            cl = cl.__bases__[0]
+
+        return get_local_attrs(
+            family_tree,
+            locale,
+            names=names,
+            append_locale_name=append_locale_name,
+        )
+
+    @classmethod
+    def _update_attrs(cls, args, das, attrs, var_id=None, names=None):
         """Format attributes with the run-time values of `compute` call parameters.
 
-        Cell methods and xclim_history attributes are updated, adding to existing values. The language of the string is
+        Cell methods and history attributes are updated, adding to existing values. The language of the string is
         taken from the `OPTIONS` configuration dictionary.
 
         Parameters
         ----------
-        das: tuple
-          Input arrays.
-        ba: bound argument object
+        args: Mapping[str, Any]
           Keyword arguments of the `compute` call.
+        das: Mapping[str, DataArray]
+          Input arrays.
         attrs : Mapping[str, str]
           The attributes to format and update.
         var_id : str
           The identifier to use when requesting the attributes translations.
-          Defaults to the class name (for the translations) or the `identifier` field of the class (for the xclim_history attribute).
+          Defaults to the class name (for the translations) or the `identifier` field of the class (for the history attribute).
           If given, the identifier will be converted to uppercase to get the translation attributes.
           This is meant for multi-outputs indicators.
         names : Sequence[str]
@@ -758,39 +830,21 @@ class Indicator(IndicatorRegistrar):
         Returns
         -------
         dict
-          Attributes with {} expressions replaced by call argument values. With updated `cell_methods` and `xclim_history`.
+          Attributes with {} expressions replaced by call argument values. With updated `cell_methods` and `history`.
           `cell_methods` is not added is `names` is given and those not contain `cell_methods`.
         """
-        args = ba.arguments
         out = cls._format(attrs, args)
-        for locale in OPTIONS["metadata_locales"]:
+        for locale in OPTIONS[METADATA_LOCALES]:
             out.update(
                 cls._format(
-                    get_local_attrs(
-                        var_id or cls._registry_id,
-                        locale,
-                        names=names or list(attrs.keys()),
-                        append_locale_name=True,
+                    cls._get_translated_metadata(
+                        locale, var_id=var_id, names=names or list(attrs.keys())
                     ),
                     args=args,
                     formatter=get_local_formatter(locale),
                 )
             )
 
-        # Generate a signature string for the history attribute
-        # We remove annotations, replace default float/int/str by values
-        # and replace others by type
-        callstr = []
-        for (k, v) in das.items():
-            callstr.append(f"{k}=<array>")
-        for (k, v) in ba.arguments.items():
-            if isinstance(v, (float, int, str)):
-                callstr.append(f"{k}={v!r}")  # repr so strings have ' '
-            else:
-                callstr.append(
-                    f"{k}={type(v)}"
-                )  # don't take chance of having unprintable values
-
         # Get history and cell method attributes from source data
         attrs = defaultdict(str)
         if names is None or "cell_methods" in names:
@@ -800,8 +854,13 @@ class Indicator(IndicatorRegistrar):
             if "cell_methods" in out:
                 attrs["cell_methods"] += " " + out.pop("cell_methods")
 
-        attrs["xclim_history"] = update_history(
-            f"{var_id or cls._registry_id}({', '.join(callstr)})",
+        # Use of OrderedDict to ensure inputs (das) get listed before parameters (args).
+        # In the history attr, call signature will be all keywords
+        # and might be in a different order than the real function (but order doesn't really matter with keywords).
+        kwargs = OrderedDict(**das)
+        kwargs.update(**args)
+        attrs["history"] = update_history(
+            gen_call_string(cls._registry_id, **kwargs),
             new_name=out.get("var_name"),
             **das,
         )
@@ -819,8 +878,9 @@ class Indicator(IndicatorRegistrar):
                 UserWarning,
             )
 
+    @classmethod
     def translate_attrs(
-        self, locale: Union[str, Sequence[str]], fill_missing: bool = True
+        cls, locale: Union[str, Sequence[str]], fill_missing: bool = True
     ):
         """Return a dictionary of unformated translated translatable attributes.
 
@@ -835,10 +895,10 @@ class Indicator(IndicatorRegistrar):
             If True (default fill the missing attributes by their english values.
         """
 
-        def _translate(var_id, var_attrs, names):
-            attrs = get_local_attrs(
-                var_id,
+        def _translate(var_attrs, names, var_id=None):
+            attrs = cls._get_translated_metadata(
                 locale,
+                var_id=var_id,
                 names=names,
                 append_locale_name=False,
             )
@@ -849,19 +909,24 @@ class Indicator(IndicatorRegistrar):
             return attrs
 
         # Translate global attrs
-        attrid = self._registry_id
         attrs = _translate(
-            attrid,
-            self.__dict__,
+            cls.__dict__,
             # Translate only translatable attrs that are not variable attrs
-            set(TRANSLATABLE_ATTRS).difference(set(self._cf_names)),
+            set(TRANSLATABLE_ATTRS).difference(set(cls._cf_names)),
         )
         # Translate variable attrs
         attrs["outputs"] = []
-        for var_attrs in self.cf_attrs:  # Translate for each variable
-            if len(self.cf_attrs) > 1:
-                attrid = f"{self.registry_id}.{var_attrs['var_name']}"
-            attrs["outputs"].append(_translate(attrid, var_attrs, TRANSLATABLE_ATTRS))
+        var_id = None
+        for var_attrs in cls.cf_attrs:  # Translate for each variable
+            if len(cls.cf_attrs) > 1:
+                var_id = var_attrs["var_name"]
+            attrs["outputs"].append(
+                _translate(
+                    var_attrs,
+                    set(TRANSLATABLE_ATTRS).intersection(cls._cf_names),
+                    var_id=var_id,
+                )
+            )
         return attrs
 
     def json(self, args=None):
@@ -917,8 +982,7 @@ class Indicator(IndicatorRegistrar):
         # Use defaults
         if args is None:
             args = {k: v["default"] for k, v in cls.parameters.items()}
-
-        args.update(getattr(cls._indcompute, "_injected", {}))
+            args.update(getattr(cls._indcompute, "_injected", {}))
 
         out = {}
         for key, val in attrs.items():
@@ -963,9 +1027,12 @@ class Indicator(IndicatorRegistrar):
 
         options = self.missing_options or OPTIONS[MISSING_OPTIONS].get(self.missing, {})
 
-        # We flag periods according to the missing method.
-        miss = (self._missing(da, freq, self.freq, options, indexer) for da in args)
-
+        # We flag periods according to the missing method. skip variables without a time coordinate.
+        miss = (
+            self._missing(da, freq, self.freq, options, indexer)
+            for da in args
+            if "time" in da.coords
+        )
         return reduce(np.logical_or, miss)
 
     # The following static methods are meant to be replaced to define custom indicators.
@@ -981,9 +1048,18 @@ class Indicator(IndicatorRegistrar):
     def cfcheck(**das):
         """Compare metadata attributes to CF-Convention standards.
 
+        Default cfchecks use the specifications in `xclim.core.utils.VARIABLES`,
+        assuming the indicator's inputs are using the CMIP6/xclim variable names correctly.
+        Variables absent from these default specs are silently ignored.
+
         When subclassing this method, use functions decorated using `xclim.core.options.cfcheck`.
         """
-        return True
+        for varname, vardata in das.items():
+            try:
+                cfcheck_from_name(varname, vardata)
+            except KeyError:
+                # Silently ignore unknown variables.
+                pass
 
     @staticmethod
     def datacheck(**das):
@@ -996,13 +1072,7 @@ class Indicator(IndicatorRegistrar):
          - assert no precipitation is negative
          - assert no temperature has the same value 5 days in a row
         """
-        return True
-
-
-class Indicator2D(Indicator):
-    """Indicator using two dimensions."""
-
-    nvar = 2
+        pass
 
 
 class Daily(Indicator):
@@ -1013,13 +1083,8 @@ class Daily(Indicator):
     @staticmethod
     def datacheck(**das):  # noqa
         for key, da in das.items():
-            datachecks.check_daily(da)
-
-
-class Daily2D(Daily):
-    """Indicator using two dimensions at daily frequency."""
-
-    nvar = 2
+            if "time" in da.coords and da.time.ndim == 1 and len(da.time) > 3:
+                datachecks.check_daily(da)
 
 
 class Hourly(Indicator):
@@ -1064,22 +1129,33 @@ def _parse_indice(indice: Callable, passed=None, **new_kwargs):
     def _upd_param(param):
         # Required DataArray arguments receive their own name as new default
         #         + the Union[str, DataArray] annotation
-        # Parameters with no default receive None
         if param.kind in [param.VAR_KEYWORD, param.VAR_POSITIONAL]:
             return param
 
-        if param.annotation is DataArray:
-            annot = Union[str, DataArray]
-        else:
-            annot = param.annotation
+        xckind = infer_kind_from_parameter(param)
 
         default = passed.get(param.name, {}).get("default", param.default)
+        if xckind == InputKind.OPTIONAL_VARIABLE and (
+            default is _empty or isinstance(default, str)
+        ):
+            # Was wrapped with suggested={param: _empty} OR somehow a variable name was injected (ex: through yaml)
+            # It becomes a non-optional variable
+            xckind = InputKind.VARIABLE
         if default is _empty:
-            if param.annotation is DataArray:
+            if xckind == InputKind.VARIABLE:
                 default = param.name
             else:
+                # Parameters with no default receive None
+                # Because we can't have no-default args _after_ default args and we just set the default on the variables (which are the first args)
                 default = None
 
+        # Python dont need no switch case
+        annots = {
+            InputKind.VARIABLE: Union[str, DataArray],
+            InputKind.OPTIONAL_VARIABLE: Optional[Union[str, DataArray]],
+        }
+        annot = annots.get(xckind, param.annotation)
+
         return Parameter(
             param.name,
             # We keep the kind, except we replace POSITIONAL_ONLY by POSITONAL_OR_KEYWORD
@@ -1137,12 +1213,25 @@ def _parse_indice(indice: Callable, passed=None, **new_kwargs):
     return indice_wrapper, parsed, params
 
 
+def add_iter_indicators(module):
+    if not hasattr(module, "iter_indicators"):
+
+        def iter_indicators():
+            for indname, ind in module.__dict__.items():
+                if isinstance(ind, Indicator):
+                    yield indname, ind
+
+        iter_indicators.__doc__ = f"Iterated over the (name, indicator) pairs in the {module.__name__} indicator module."
+
+        module.__dict__["iter_indicators"] = iter_indicators
+
+
 def build_indicator_module(
     name: str,
     objs: Mapping[str, Indicator],
     doc: Optional[str] = None,
 ) -> ModuleType:
-    """Create a module from imported objects.
+    """Create or update a module from imported objects.
 
     The module is inserted as a submodule of `xclim.indicators`.
 
@@ -1178,6 +1267,7 @@ def build_indicator_module(
         indicators.__dict__[name] = out
 
     out.__dict__.update(objs)
+    add_iter_indicators(out)
     return out
 
 
@@ -1186,6 +1276,8 @@ def build_indicator_module_from_yaml(
     name: Optional[str] = None,
     base: Type[Indicator] = Daily,
     doc: Optional[str] = None,
+    indices: Optional[Union[Mapping[str, Callable], ModuleType]] = None,
+    translations: Optional[Mapping[str, dict]] = None,
     mode: str = "raise",
     realm: Optional[str] = None,
     keywords: Optional[str] = None,
@@ -1194,20 +1286,27 @@ def build_indicator_module_from_yaml(
 ) -> ModuleType:
     """Build or extend an indicator module from a YAML file.
 
-    The module is inserted as a submodule of `xclim.indicators`.
+    The module is inserted as a submodule of `xclim.indicators`. When given only a base filename (no 'yml' extesion), this
+    tries to find custom indices in a module of the same name (*.py) and translations in json files (*.<lang>.json), see Notes.
 
     Parameters
     ----------
     filename: PathLike
-      Path to a YAML file.
+      Path to a YAML file or to the stem of all module files. See Notes for behaviour when passing a basename only.
     name: str, optional
-      The name of the new or existing module, defaults to the name of the file.
+      The name of the new or existing module, defaults to the basename of the file.
       (e.g: `atmos.yml` -> `atmos`)
     base: Indicator subclass
       The Indicator subclass from which the new indicators are based. Superseeded by
       the class given in the yaml file or in individual indicator definitions (see submodule's doc).
     doc : str, optional
       The docstring of the new submodule. Defaults to a very minimal header with the submodule's name.
+    indices : Mapping of callables or module, optional
+      A mapping or module of indice functions. When creating the indicator, the name in the `index_function` field is
+      first sought here, then in xclim.indices.generic and finally in xclim.indices.
+    translations  : Mapping of dicts, optional
+      Translated metadata for the new indicators. Keys of the mapping must be 2-char language tags.
+      See Notes and :ref:`Internationalization` for more details.
     mode: {'raise', 'warn', 'ignore'}
       How to deal with broken indice definitions.
     realm: str, optional
@@ -1225,13 +1324,33 @@ def build_indicator_module_from_yaml(
     ModuleType
       A submodule of `xclim.indicators`.
 
+    Notes
+    -----
+    When the given `filename` has no suffix (usually '.yaml' or '.yml'), the function will try to load
+    custom indice definitions from a file with the same name but with a `.py` extension. Similarly,
+    it will try to load translations in `*.<lang>.json` files, where `<lang>` is the IETF language tag.
+
+    For example. a set of custom indicators could be fully described by the following files:
+
+        - `example.yml` : defining the indicator's metadata.
+        - `example.py` : defining a few indice functions.
+        - `example.fr.json` : French translations
+        - `example.tlh.json` : Klingon translations.
+
     See also
     --------
     The doc of :py:mod:`xclim.core.indicator` and of :py:func:`build_module`.
     """
-    # Read YAML file
     filepath = Path(filename)
-    with filepath.open() as f:
+
+    if not filepath.suffix:
+        # A stem was passed, try to load files
+        ymlpath = filepath.with_suffix(".yml")
+    else:
+        ymlpath = filepath
+
+    # Read YAML file
+    with ymlpath.open() as f:
         yml = safe_load(f)
 
     # Load values from top-level in yml.
@@ -1240,36 +1359,87 @@ def build_indicator_module_from_yaml(
     default_base = registry.get(yml.get("base"), base)
     doc = doc or yml.get("doc")
 
+    # When given as a stem, we try to load indices and translations
+    if not filepath.suffix:
+        if indices is None:
+            try:
+                indices = load_module(filepath.with_suffix(".py"))
+            except ModuleNotFoundError:
+                pass
+
+        if translations is None:
+            translations = {}
+            for locfile in filepath.parent.glob(filepath.stem + ".*.json"):
+                locale = locfile.suffixes[0][1:]
+                translations[locale] = read_locale_file(locfile, module=module_name)
+
     # Module-wide default values for some attributes
     defkwargs = {
-        # We can override the module of indicators in their init (weird but cool)
-        # This way, submodule indicators are prefixed with the module name in the registry.
-        "module": module_name,
         # Other default argument, only given in case the indicator definition does not give them.
         "realm": realm or yml.get("realm"),
         "keywords": keywords or yml.get("keywords"),
         "references": references or yml.get("references"),
         "notes": notes or yml.get("notes"),
     }
+
     # Parse the indicators:
     mapping = {}
     for identifier, data in yml["indices"].items():
-        # clix-meta has illegal characters in the identifiers.
-        clean_id = identifier.replace("{", "").replace("}", "")
         try:
+            clean_id, data = _cleanup_indicator_dict(
+                identifier, data, indices, defkwargs
+            )
+
             if "base" in data:
-                base = registry[data["base"].upper()]
+                if data["base"].startswith("."):
+                    # A point means the base has been declared above.
+                    base = registry[module_name + data["base"].upper()]
+                else:
+                    base = registry[data["base"].upper()]
             else:
                 base = default_base
-            mapping[clean_id] = base.from_dict(data, clean_id, **defkwargs)
+
+            mapping[clean_id] = base.from_dict(
+                data, identifier=clean_id, module=module_name
+            )
+
         except Exception as err:
-            msg = f"Constructing {identifier} failed with {err!r}"
-            if mode == "ignore":
-                logging.info(msg)
-            elif mode == "warn":
-                warnings.warn(msg)
-            else:  # mode == "raise"
-                raise ValueError(msg) from err
+            raise_warn_or_log(
+                err, mode, msg=f"Constructing {identifier} failed with {err!r}"
+            )
 
     # Construct module
-    return build_indicator_module(module_name, objs=mapping, doc=doc)
+    mod = build_indicator_module(module_name, objs=mapping, doc=doc)
+
+    # If there are translations, load them
+    if translations:
+        for locale, locdict in translations.items():
+            load_locale(locdict, locale)
+
+    return mod
+
+
+def _cleanup_indicator_dict(identifier, data, indices, defaults):
+    # Assign indice as func.
+    # If data.index_function.name refers to a function in `indices`, replace that field by the function.
+    indice_name = data.get("index_function", {}).get("name", None)
+    if indice_name is not None and indices is not None:
+        indice_func = getattr(indices, indice_name, None)
+        if indice_func is None and hasattr(indices, "__getitem__"):
+            try:
+                indice_func = indices[indice_name]
+            except KeyError:
+                pass
+
+        if indice_func is not None:
+            data["index_function"]["name"] = indice_func
+
+    # clix-meta has illegal characters in the identifiers.
+    clean_id = identifier.replace("{", "").replace("}", "")
+
+    # Workaround for clix-meta (we name it references, they name it reference)
+    data.setdefault("references", data.get("reference"))
+    for k, v in defaults.items():
+        data.setdefault(k, v)
+
+    return clean_id, data
