@@ -1,15 +1,20 @@
 # -*- coding: utf-8 -*-
 """Temperature indicator definitions."""
 
+from inspect import _empty  # noqa
+
 from xclim import indices
 from xclim.core import cfchecks
-from xclim.core.indicator import Daily, Daily2D
-from xclim.core.units import check_units
+from xclim.core.indicator import Daily
 from xclim.core.utils import wrapped_partial
 
 __all__ = [
+    "tn_days_above",
     "tn_days_below",
+    "tg_days_above",
+    "tg_days_below",
     "tx_days_above",
+    "tx_days_below",
     "tx_tn_days_above",
     "heat_wave_frequency",
     "heat_wave_max_length",
@@ -39,7 +44,11 @@ __all__ = [
     "cold_spell_duration_index",
     "cold_spell_days",
     "cold_spell_frequency",
+    "cool_night_index",
     "daily_freezethaw_cycles",
+    "freezethaw_spell_frequency",
+    "freezethaw_spell_max_length",
+    "freezethaw_spell_mean_length",
     "cooling_degree_days",
     "heating_degree_days",
     "growing_degree_days",
@@ -59,59 +68,29 @@ __all__ = [
     "warm_spell_duration_index",
     "maximum_consecutive_warm_days",
     "fire_season",
+    "corn_heat_units",
+    "huglin_index",
+    "biologically_effective_degree_days",
+    "latitude_temperature_index",
 ]
 
 
-# TODO: Should we reference the standard vocabulary we're using ?
-# E.g. http://vocab.nerc.ac.uk/collection/P07/current/BHMHISG2/
+# We need to declare the class here so that the `atmos` realm is automatically detected.
+class Temp(Daily):
+    """Indicators involving temperature."""
 
 
-class Tas(Daily):
-    """Class for univariate indices using mean daily temperature as the input."""
+tn_days_above = Temp(
+    identifier="tn_days_above",
+    units="days",
+    standard_name="number_of_days_with_air_temperature_above_threshold",
+    long_name="Number of days with Tmin > {thresh}",
+    description="{freq} number of days where daily minimum temperature exceeds {thresh}.",
+    cell_methods="time: minimum within days time: sum over days",
+    compute=indices.tn_days_above,
+)
 
-    @staticmethod
-    def cfcheck(tas):
-        cfchecks.check_valid(tas, "cell_methods", "*time: mean within days*")
-        cfchecks.check_valid(tas, "standard_name", "air_temperature")
-
-
-class Tasmin(Daily):
-    """Class for univariate indices using min daily temperature as the input."""
-
-    @staticmethod
-    def cfcheck(tasmin):
-        cfchecks.check_valid(tasmin, "cell_methods", "*time: minimum within days*")
-        cfchecks.check_valid(tasmin, "standard_name", "air_temperature")
-
-
-class Tasmax(Daily):
-    """Class for univariate indices using max daily temperature as the input."""
-
-    @staticmethod
-    def cfcheck(tasmax):
-        cfchecks.check_valid(tasmax, "cell_methods", "*time: maximum within days*")
-        cfchecks.check_valid(tasmax, "standard_name", "air_temperature")
-
-
-class Tasx(Daily):
-    """Class for univariate indices using mean daily temperature as the input."""
-
-    @staticmethod
-    def cfcheck(tas):
-        cfchecks.check_valid(tas, "standard_name", "air_temperature")
-
-
-class TasminTasmax(Daily2D):
-    @staticmethod
-    def cfcheck(tasmin, tasmax):
-        for da in (tasmin, tasmax):
-            cfchecks.check_valid(da, "standard_name", "air_temperature")
-        cfchecks.check_valid(tasmin, "cell_methods", "*time: minimum within days*")
-        cfchecks.check_valid(tasmax, "cell_methods", "*time: maximum within days*")
-        check_units(tasmax, tasmin.attrs["units"])
-
-
-tn_days_below = Tasmin(
+tn_days_below = Temp(
     identifier="tn_days_below",
     units="days",
     standard_name="number_of_days_with_air_temperature_below_threshold",
@@ -121,7 +100,27 @@ tn_days_below = Tasmin(
     compute=indices.tn_days_below,
 )
 
-tx_days_above = Tasmax(
+tg_days_above = Temp(
+    identifier="tg_days_above",
+    units="days",
+    standard_name="number_of_days_with_air_temperature_above_threshold",
+    long_name="Number of days with Tavg > {thresh}",
+    description="{freq} number of days where daily mean temperature exceeds {thresh}.",
+    cell_methods="time: mean within days time: sum over days",
+    compute=indices.tg_days_above,
+)
+
+tg_days_below = Temp(
+    identifier="tg_days_below",
+    units="days",
+    standard_name="number_of_days_with_air_temperature_below_threshold",
+    long_name="Number of days with Tavg < {thresh}",
+    description="{freq} number of days where daily mean temperature is below {thresh}.",
+    cell_methods="time: mean within days time: sum over days",
+    compute=indices.tg_days_below,
+)
+
+tx_days_above = Temp(
     identifier="tx_days_above",
     units="days",
     standard_name="number_of_days_with_air_temperature_above_threshold",
@@ -131,7 +130,17 @@ tx_days_above = Tasmax(
     compute=indices.tx_days_above,
 )
 
-tx_tn_days_above = TasminTasmax(
+tx_days_below = Temp(
+    identifier="tx_days_below",
+    units="days",
+    standard_name="number_of_days_with_air_temperature_below_threshold",
+    long_name="Number of days with Tmax < {thresh}",
+    description="{freq} number of days where daily max temperature is below {thresh}.",
+    cell_methods="time: max within days time: sum over days",
+    compute=indices.tx_days_below,
+)
+
+tx_tn_days_above = Temp(
     identifier="tx_tn_days_above",
     units="days",
     standard_name="number_of_days_with_air_temperature_above_threshold",
@@ -142,7 +151,8 @@ tx_tn_days_above = TasminTasmax(
     compute=indices.tx_tn_days_above,
 )
 
-heat_wave_frequency = TasminTasmax(
+
+heat_wave_frequency = Temp(
     identifier="heat_wave_frequency",
     units="",
     standard_name="heat_wave_events",
@@ -158,7 +168,7 @@ heat_wave_frequency = TasminTasmax(
     compute=indices.heat_wave_frequency,
 )
 
-heat_wave_max_length = TasminTasmax(
+heat_wave_max_length = Temp(
     identifier="heat_wave_max_length",
     units="days",
     standard_name="spell_length_of_days_with_air_temperature_above_threshold",
@@ -174,7 +184,7 @@ heat_wave_max_length = TasminTasmax(
     compute=indices.heat_wave_max_length,
 )
 
-heat_wave_total_length = TasminTasmax(
+heat_wave_total_length = Temp(
     identifier="heat_wave_total_length",
     units="days",
     standard_name="spell_length_of_days_with_air_temperature_above_threshold",
@@ -191,7 +201,7 @@ heat_wave_total_length = TasminTasmax(
 )
 
 
-heat_wave_index = Tasmax(
+heat_wave_index = Temp(
     identifier="heat_wave_index",
     units="days",
     standard_name="heat_wave_index",
@@ -203,7 +213,7 @@ heat_wave_index = Tasmax(
 )
 
 
-hot_spell_frequency = Tasmax(
+hot_spell_frequency = Temp(
     identifier="hot_spell_frequency",
     units="",
     standard_name="hot_spell_events",
@@ -216,7 +226,7 @@ hot_spell_frequency = Tasmax(
     compute=indices.hot_spell_frequency,
 )
 
-hot_spell_max_length = Tasmax(
+hot_spell_max_length = Temp(
     identifier="hot_spell_max_length",
     units="days",
     standard_name="spell_length_of_days_with_air_temperature_above_threshold",
@@ -229,7 +239,7 @@ hot_spell_max_length = Tasmax(
     compute=indices.hot_spell_max_length,
 )
 
-tg_mean = Tas(
+tg_mean = Temp(
     identifier="tg_mean",
     units="K",
     standard_name="air_temperature",
@@ -239,7 +249,7 @@ tg_mean = Tas(
     compute=indices.tg_mean,
 )
 
-tg_max = Tas(
+tg_max = Temp(
     identifier="tg_max",
     units="K",
     standard_name="air_temperature",
@@ -249,7 +259,7 @@ tg_max = Tas(
     compute=indices.tg_max,
 )
 
-tg_min = Tas(
+tg_min = Temp(
     identifier="tg_min",
     units="K",
     standard_name="air_temperature",
@@ -259,7 +269,7 @@ tg_min = Tas(
     compute=indices.tg_min,
 )
 
-tx_mean = Tasmax(
+tx_mean = Temp(
     identifier="tx_mean",
     units="K",
     standard_name="air_temperature",
@@ -269,7 +279,7 @@ tx_mean = Tasmax(
     compute=indices.tx_mean,
 )
 
-tx_max = Tasmax(
+tx_max = Temp(
     identifier="tx_max",
     units="K",
     standard_name="air_temperature",
@@ -279,7 +289,7 @@ tx_max = Tasmax(
     compute=indices.tx_max,
 )
 
-tx_min = Tasmax(
+tx_min = Temp(
     identifier="tx_min",
     units="K",
     standard_name="air_temperature",
@@ -289,7 +299,7 @@ tx_min = Tasmax(
     compute=indices.tx_min,
 )
 
-tn_mean = Tasmin(
+tn_mean = Temp(
     identifier="tn_mean",
     units="K",
     standard_name="air_temperature",
@@ -299,7 +309,7 @@ tn_mean = Tasmin(
     compute=indices.tn_mean,
 )
 
-tn_max = Tasmin(
+tn_max = Temp(
     identifier="tn_max",
     units="K",
     standard_name="air_temperature",
@@ -309,7 +319,7 @@ tn_max = Tasmin(
     compute=indices.tn_max,
 )
 
-tn_min = Tasmin(
+tn_min = Temp(
     identifier="tn_min",
     units="K",
     standard_name="air_temperature",
@@ -319,7 +329,7 @@ tn_min = Tasmin(
     compute=indices.tn_min,
 )
 
-daily_temperature_range = TasminTasmax(
+daily_temperature_range = Temp(
     title="Mean of daily temperature range.",
     identifier="dtr",
     units="K",
@@ -330,7 +340,7 @@ daily_temperature_range = TasminTasmax(
     compute=wrapped_partial(indices.daily_temperature_range, op="mean"),
 )
 
-max_daily_temperature_range = TasminTasmax(
+max_daily_temperature_range = Temp(
     title="Maximum of daily temperature range.",
     identifier="dtrmax",
     units="K",
@@ -341,7 +351,7 @@ max_daily_temperature_range = TasminTasmax(
     compute=wrapped_partial(indices.daily_temperature_range, op="max"),
 )
 
-daily_temperature_range_variability = TasminTasmax(
+daily_temperature_range_variability = Temp(
     identifier="dtrvar",
     units="K",
     standard_name="air_temperature",
@@ -355,7 +365,7 @@ daily_temperature_range_variability = TasminTasmax(
     compute=indices.daily_temperature_range_variability,
 )
 
-extreme_temperature_range = TasminTasmax(
+extreme_temperature_range = Temp(
     identifier="etr",
     units="K",
     standard_name="air_temperature",
@@ -365,7 +375,7 @@ extreme_temperature_range = TasminTasmax(
     compute=indices.extreme_temperature_range,
 )
 
-cold_spell_duration_index = Tasmin(
+cold_spell_duration_index = Temp(
     identifier="cold_spell_duration_index",
     var_name="csdi_{window}",
     units="days",
@@ -379,43 +389,110 @@ cold_spell_duration_index = Tasmin(
     compute=indices.cold_spell_duration_index,
 )
 
-cold_spell_days = Tas(
+cold_spell_days = Temp(
     identifier="cold_spell_days",
     units="days",
     standard_name="cold_spell_days",
     long_name="Number of days part of a cold spell",
     description="{freq} number of days that are part of a cold spell, defined as {window} "
     "or more consecutive days with mean daily "
-    "temperature below  {thresh}.",
+    "temperature below {thresh}.",
     cell_methods="",
     compute=indices.cold_spell_days,
 )
 
-cold_spell_frequency = Tas(
+cold_spell_frequency = Temp(
     identifier="cold_spell_frequency",
     units="",
     standard_name="cold_spell_frequency",
     long_name="Number of cold spell events",
     description="{freq} number cold spell events, defined as {window} "
     "or more consecutive days with mean daily "
-    "temperature below  {thresh}.",
+    "temperature below {thresh}.",
     cell_methods="",
     compute=indices.cold_spell_frequency,
 )
 
+cool_night_index = Temp(
+    identifier="cool_night_index",
+    units="degC",
+    long_name="cool night index",
+    description="Mean minimum temperature for September (northern hemisphere) or March (southern hemisphere).",
+    cell_methods="time: min within days time: mean over days",
+    abstract="A night coolness variable which takes into account the mean minimum night temperatures during the "
+    "month when ripening usually occurs beyond the ripening period.",
+    allowed_periods=["A"],
+    compute=indices.cool_night_index,
+)
 
-daily_freezethaw_cycles = TasminTasmax(
+daily_freezethaw_cycles = Temp(
     identifier="dlyfrzthw",
     units="days",
-    standard_name="daily_freezethaw_cycles",
     long_name="daily freezethaw cycles",
     description="{freq} number of days with a diurnal freeze-thaw cycle "
     ": Tmax > {thresh_tasmax} and Tmin <= {thresh_tasmin}.",
     cell_methods="",
-    compute=indices.daily_freezethaw_cycles,
+    compute=wrapped_partial(
+        indices.multiday_temperature_swing,
+        op="sum",
+        window=1,
+        suggested=dict(thresh_tasmax="0 degC", thresh_tasmin="0 degC"),
+    ),
 )
 
-cooling_degree_days = Tas(
+
+freezethaw_spell_frequency = Temp(
+    identifier="freezethaw_spell_frequency",
+    title="Frequency of freeze-thaw spells",
+    units="days",
+    long_name="{freq} number of freeze-thaw spells.",
+    description="{freq} number of freeze-thaw spells"
+    ": Tmax > {thresh_tasmax} and Tmin <= {thresh_tasmin} "
+    "for at least {window} consecutive day(s).",
+    cell_methods="",
+    compute=wrapped_partial(
+        indices.multiday_temperature_swing,
+        op="count",
+        suggested=dict(thresh_tasmax="0 degC", thresh_tasmin="0 degC"),
+    ),
+)
+
+
+freezethaw_spell_mean_length = Temp(
+    identifier="freezethaw_spell_mean_length",
+    title="Averge length of freeze-thaw spells.",
+    units="days",
+    long_name="{freq} average length of freeze-thaw spells.",
+    description="{freq} average length of freeze-thaw spells"
+    ": Tmax > {thresh_tasmax} and Tmin <= {thresh_tasmin} "
+    "for at least {window} consecutive day(s).",
+    cell_methods="",
+    compute=wrapped_partial(
+        indices.multiday_temperature_swing,
+        op="mean",
+        suggested=dict(thresh_tasmax="0 degC", thresh_tasmin="0 degC"),
+    ),
+)
+
+
+freezethaw_spell_max_length = Temp(
+    identifier="freezethaw_spell_max_length",
+    title="Maximal length of freeze-thaw spells.",
+    units="days",
+    long_name="{freq} maximal length of freeze-thaw spells.",
+    description="{freq} maximal length of freeze-thaw spells"
+    ": Tmax > {thresh_tasmax} and Tmin <= {thresh_tasmin} "
+    "for at least {window} consecutive day(s).",
+    cell_methods="",
+    compute=wrapped_partial(
+        indices.multiday_temperature_swing,
+        op="max",
+        suggested=dict(thresh_tasmax="0 degC", thresh_tasmin="0 degC"),
+    ),
+)
+
+
+cooling_degree_days = Temp(
     identifier="cooling_degree_days",
     units="K days",
     standard_name="integral_of_air_temperature_excess_wrt_time",
@@ -425,7 +502,7 @@ cooling_degree_days = Tas(
     compute=indices.cooling_degree_days,
 )
 
-heating_degree_days = Tas(
+heating_degree_days = Temp(
     identifier="heating_degree_days",
     units="K days",
     standard_name="integral_of_air_temperature_deficit_wrt_time",
@@ -435,7 +512,7 @@ heating_degree_days = Tas(
     compute=indices.heating_degree_days,
 )
 
-growing_degree_days = Tas(
+growing_degree_days = Temp(
     identifier="growing_degree_days",
     units="K days",
     standard_name="integral_of_air_temperature_excess_wrt_time",
@@ -445,7 +522,7 @@ growing_degree_days = Tas(
     compute=indices.growing_degree_days,
 )
 
-freshet_start = Tas(
+freshet_start = Temp(
     identifier="freshet_start",
     units="",
     standard_name="day_of_year",
@@ -455,7 +532,7 @@ freshet_start = Tas(
     compute=indices.freshet_start,
 )
 
-frost_days = Tasmin(
+frost_days = Temp(
     identifier="frost_days",
     units="days",
     standard_name="days_with_air_temperature_below_threshold",
@@ -465,7 +542,7 @@ frost_days = Tasmin(
     compute=indices.frost_days,
 )
 
-frost_season_length = Tasmin(
+frost_season_length = Temp(
     identifier="frost_season_length",
     units="days",
     standard_name="days_with_air_temperature_below_threshold",
@@ -478,7 +555,7 @@ frost_season_length = Tasmin(
     compute=wrapped_partial(indices.frost_season_length, thresh="0 degC"),
 )
 
-last_spring_frost = Tasmin(
+last_spring_frost = Temp(
     identifier="last_spring_frost",
     units="",
     standard_name="day_of_year",
@@ -488,7 +565,7 @@ last_spring_frost = Tasmin(
     compute=indices.last_spring_frost,
 )
 
-first_day_below = Tasmin(
+first_day_below = Temp(
     identifier="first_day_below",
     units="",
     standard_name="day_of_year",
@@ -497,7 +574,7 @@ first_day_below = Tasmin(
     compute=indices.first_day_below,
 )
 
-first_day_above = Tasmin(
+first_day_above = Temp(
     identifier="first_day_above",
     units="",
     standard_name="day_of_year",
@@ -507,7 +584,7 @@ first_day_above = Tasmin(
 )
 
 
-ice_days = Tasmax(
+ice_days = Temp(
     identifier="ice_days",
     standard_name="days_with_air_temperature_below_threshold",
     units="days",
@@ -517,7 +594,7 @@ ice_days = Tasmax(
     compute=indices.ice_days,
 )
 
-consecutive_frost_days = Tasmin(
+consecutive_frost_days = Temp(
     identifier="consecutive_frost_days",
     units="days",
     standard_name="spell_length_of_days_with_air_temperature_below_threshold",
@@ -528,7 +605,7 @@ consecutive_frost_days = Tasmin(
     compute=indices.maximum_consecutive_frost_days,
 )
 
-maximum_consecutive_frost_free_days = Tasmin(
+maximum_consecutive_frost_free_days = Temp(
     identifier="consecutive_frost_free_days",
     units="days",
     standard_name="spell_length_of_days_with_air_temperature_above_threshold",
@@ -539,7 +616,7 @@ maximum_consecutive_frost_free_days = Tasmin(
     compute=indices.maximum_consecutive_frost_free_days,
 )
 
-growing_season_length = Tas(
+growing_season_length = Temp(
     identifier="growing_season_length",
     units="days",
     standard_name="growing_season_length",
@@ -552,7 +629,7 @@ growing_season_length = Tas(
     compute=indices.growing_season_length,
 )
 
-growing_season_end = Tas(
+growing_season_end = Temp(
     identifier="growing_season_end",
     units="",
     standard_name="day_of_year",
@@ -564,7 +641,7 @@ growing_season_end = Tas(
     compute=indices.growing_season_end,
 )
 
-tropical_nights = Tasmin(
+tropical_nights = Temp(
     identifier="tropical_nights",
     units="days",
     standard_name="number_of_days_with_air_temperature_above_threshold",
@@ -572,10 +649,10 @@ tropical_nights = Tasmin(
     description="{freq} number of Tropical Nights : defined as days with minimum daily temperature"
     " above {thresh}",
     cell_methods="time: minimum within days time: sum over days",
-    compute=indices.tropical_nights,
+    compute=wrapped_partial(indices.tn_days_above, suggested=dict(thresh="20 degC")),
 )
 
-tg90p = Tas(
+tg90p = Temp(
     identifier="tg90p",
     units="days",
     standard_name="days_with_air_temperature_above_threshold",
@@ -587,7 +664,7 @@ tg90p = Tas(
     compute=indices.tg90p,
 )
 
-tg10p = Tas(
+tg10p = Temp(
     identifier="tg10p",
     units="days",
     standard_name="days_with_air_temperature_below_threshold",
@@ -599,7 +676,7 @@ tg10p = Tas(
     compute=indices.tg10p,
 )
 
-tx90p = Tasmax(
+tx90p = Temp(
     identifier="tx90p",
     units="days",
     standard_name="days_with_air_temperature_above_threshold",
@@ -611,7 +688,7 @@ tx90p = Tasmax(
     compute=indices.tx90p,
 )
 
-tx10p = Tasmax(
+tx10p = Temp(
     identifier="tx10p",
     units="days",
     standard_name="days_with_air_temperature_below_threshold",
@@ -623,7 +700,7 @@ tx10p = Tasmax(
     compute=indices.tx10p,
 )
 
-tn90p = Tasmin(
+tn90p = Temp(
     identifier="tn90p",
     units="days",
     standard_name="days_with_air_temperature_above_threshold",
@@ -635,7 +712,7 @@ tn90p = Tasmin(
     compute=indices.tn90p,
 )
 
-tn10p = Tasmin(
+tn10p = Temp(
     identifier="tn10p",
     units="days",
     standard_name="days_with_air_temperature_below_threshold",
@@ -648,7 +725,7 @@ tn10p = Tasmin(
 )
 
 
-degree_days_exceedance_date = Tas(
+degree_days_exceedance_date = Temp(
     identifier="degree_days_exceedance_date",
     units="",
     standard_name="day_of_year",
@@ -660,7 +737,7 @@ degree_days_exceedance_date = Tas(
 )
 
 
-warm_spell_duration_index = Tasmax(
+warm_spell_duration_index = Temp(
     identifier="warm_spell_duration_index",
     description="{freq} total number of days within spells of at least {window} days"
     " with tmax above the 90th daily percentile.",
@@ -671,7 +748,7 @@ warm_spell_duration_index = Tasmax(
 )
 
 
-maximum_consecutive_warm_days = Tasmax(
+maximum_consecutive_warm_days = Temp(
     identifier="maximum_consecutive_warm_days",
     description="{freq} longest spell of consecutive days with Tmax above {thresh}.",
     units="days",
@@ -681,9 +758,117 @@ maximum_consecutive_warm_days = Tasmax(
 )
 
 
-fire_season = Tasx(
+class FireSeasonBase(Daily):
+    """Special Indicator class for FireSeason that accepts any tas[min/max] and optional snd."""
+
+    @staticmethod
+    def cfcheck(tas, snd=None):
+        cfchecks.check_valid(tas, "standard_name", "air_temperature")
+        cfchecks.cfcheck_from_name("snd", snd)
+
+
+fire_season = FireSeasonBase(
     identifier="fire_season",
     description="Fire season mask, computed with method {method}.",
     units="",
     compute=indices.fire_season,
 )
+
+
+corn_heat_units = Temp(
+    identifier="corn_heat_units",
+    units="",
+    long_name="Corn heat units (Tmin > {thresh_tasmin} and Tmax > {thresh_tasmax}).",
+    description="Temperature-based index used to estimate the development of corn crops. "
+    "Corn growth occurs when the minimum and maximum daily temperature both exceeds "
+    "specific thresholds : Tmin > {thresh_tasmin} and Tmax > {thresh_tasmax}.",
+    var_name="chu",
+    cell_methods="",
+    missing="skip",
+    compute=indices.corn_heat_units,
+)
+
+huglin_index = Temp(
+    identifier="huglin_index",
+    units="",
+    long_name="Huglin heliothermal index (Summation of ((Tmin + Tmax)/2 - {thresh_tasmin}) * Latitude-based day-length"
+    "coefficient (`k`), for days between {start_date} and {end_date}).",
+    description="Heat-summation index for agroclimatic suitability estimation, developed specifically for viticulture. "
+    "Considers daily Tmin and Tmax with a base of {thresh_tasmin}, typically between 1 April and 30 September. "
+    "Integrates a day-length coefficient calculation for higher latitudes.",
+    cell_methods="",
+    comment="Metric originally published in Huglin (1978). Day-length coefficient based on Hall & Jones (2010)",
+    var_name="hi",
+    compute=wrapped_partial(
+        indices.huglin_index,
+        method="jones",
+    ),
+)
+
+
+biologically_effective_degree_days = Temp(
+    identifier="biologically_effective_degree_days",
+    units="K days",
+    long_name="Biologically effective degree days computed with {method} formula (Summation of min((max((Tmin + Tmax)/2"
+    " - {thresh_tasmin}, 0) * k) + TR_adg, 9°C), for days between {start_date} and {end_date}).",
+    description="Heat-summation index for agroclimatic suitability estimation, developed specifically for viticulture. "
+    "Considers daily Tmin and Tmax with a base of {thresh_tasmin} between 1 April and 31 October, with a maximum daily "
+    "value for degree days (typically 9°C). It also integrates a modification coefficient for latitudes "
+    "between 40°N and 50°N as well as swings in daily temperature range.",
+    cell_methods="",
+    comment="Original formula published in Gladstones, 1992.",
+    var_name="bedd",
+    compute=wrapped_partial(
+        indices.biologically_effective_degree_days,
+        method="gladstones",
+        suggested=dict(
+            thresh_tasmin="10 degC",
+            low_dtr="10 degC",
+            high_dtr="13 degC",
+            max_daily_degree_days="9 degC",
+            start_date="04-01",
+            end_date="11-01",
+            lat=_empty,
+        ),
+    ),
+)
+
+effective_growing_degree_days = Temp(
+    identifier="effective_growing_degree_days",
+    units="K days",
+    long_name="Effective growing degree days computed with {method} formula (Summation of max((Tmin + Tmax)/2 "
+    "- {thresh}, 0), for days between between dynamically-determined start and end dates).",
+    description="Heat-summation index for agroclimatic suitability estimation."
+    "Considers daily Tmin and Tmax with a base of {thresh} between dynamically-determined growing season start"
+    "and end dates. The 'bootsma' method uses a 10-day average temperature above {thresh} to identify a start date, "
+    "while the 'qian' method uses a weighted mean average above {thresh} over 5 days to determine start date. "
+    "The end date of the growing season is the date of first fall frost (Tmin < 0 degC).",
+    cell_methods="",
+    comment="Original formula published in Bootsma et al. 2005.",
+    var_name="egdd",
+    compute=wrapped_partial(
+        indices.effective_growing_degree_days,
+        suggested=dict(
+            thresh="5 degC",
+            method="bootsma",
+            after_date="07-01",
+        ),
+    ),
+)
+
+
+latitude_temperature_index = Temp(
+    identifier="latitude_temperature_index",
+    units="",
+    long_name="Latitude-temperature index",
+    description="A climate indice based on mean temperature of the warmest month and a latitude-based coefficient to "
+    "account for longer day-length favouring growing conditions. Developed specifically for viticulture. Mean temperature of warmest "
+    "month * ({lat_factor} - latitude).",
+    cell_methods="",
+    allowed_periods=["A"],
+    comment="Indice originally published in Jackson, D. I., & Cherry, N. J. (1988)",
+    var_name="lti",
+    compute=wrapped_partial(
+        indices.latitude_temperature_index, suggested=dict(lat_factor=60, lat=_empty)
+    ),
+)
