@@ -3,27 +3,19 @@
 Sea ice indicators
 ------------------
 """
-import abc
-
 from xclim import indices
-from xclim.core import cfchecks
-from xclim.core.indicator import Indicator2D
+from xclim.core.indicator import Indicator
 
 __all__ = ["sea_ice_area", "sea_ice_extent"]
 
 
-class SicArea(Indicator2D):
+class SiconcAreacello(Indicator):
     """Class for indicators having sea ice concentration and grid cell area inputs."""
 
     missing = "skip"
 
-    @staticmethod
-    def cfcheck(sic, area):
-        cfchecks.check_valid(sic, "standard_name", "sea_ice_area_fraction")
-        cfchecks.check_valid(area, "standard_name", "cell_area")
 
-
-sea_ice_extent = SicArea(
+sea_ice_extent = SiconcAreacello(
     identifier="sea_ice_extent",
     units="m2",
     standard_name="sea_ice_extent",
@@ -34,7 +26,7 @@ sea_ice_extent = SicArea(
 )
 
 
-sea_ice_area = SicArea(
+sea_ice_area = SiconcAreacello(
     identifier="sea_ice_area",
     units="m2",
     standard_name="sea_ice_area",
