@@ -1,25 +1,30 @@
 # noqa: D100
+import warnings
 from typing import Optional
 
 import numpy as np
 import xarray
+from xarray.core.dataarray import DataArray
 
+from xclim.core.bootstrapping import percentile_bootstrap
 from xclim.core.calendar import resample_doy
 from xclim.core.units import (
     convert_units_to,
     declare_units,
+    infer_sampling_units,
     pint2cfunits,
     rate2amount,
     str2pint,
     to_agg_units,
 )
+from xclim.core.utils import ValidationError
 
 from . import run_length as rl
 from ._conversion import rain_approximation, snowfall_approximation
 from .generic import select_resample_op, threshold_count
 
 # Frequencies : YS: year start, QS-DEC: seasons starting in december, MS: month start
-# See http://pandas.pydata.org/pandas-docs/stable/timeseries.html#offset-aliases
+# See https://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html
 
 # -------------------------------------------------- #
 # ATTENTION: ASSUME ALL INDICES WRONG UNTIL TESTED ! #
@@ -29,7 +34,11 @@ __all__ = [
     "blowing_snow",
     "cold_spell_duration_index",
     "cold_and_dry_days",
+    "warm_and_dry_days",
+    "warm_and_wet_days",
+    "cold_and_wet_days",
     "daily_freezethaw_cycles",
+    "multiday_temperature_swing",
     "daily_temperature_range",
     "daily_temperature_range_variability",
     "days_over_precip_thresh",
@@ -55,8 +64,13 @@ __all__ = [
 
 
 @declare_units(tasmin="[temperature]", tn10="[temperature]")
+@percentile_bootstrap
 def cold_spell_duration_index(
-    tasmin: xarray.DataArray, tn10: xarray.DataArray, window: int = 6, freq: str = "YS"
+    tasmin: xarray.DataArray,
+    tn10: xarray.DataArray,
+    window: int = 6,
+    freq: str = "YS",
+    bootstrap: bool = False,  # noqa
 ) -> xarray.DataArray:
     r"""Cold spell duration index.
 
@@ -73,6 +87,8 @@ def cold_spell_duration_index(
       Minimum number of days with temperature below threshold to qualify as a cold spell.
     freq : str
       Resampling frequency.
+    bootstrap : bool
+      Flag to run bootstrapping. Used by percentile_bootstrap decorator.
 
     Returns
     -------
@@ -119,8 +135,18 @@ def cold_spell_duration_index(
     return to_agg_units(out, tasmin, "count")
 
 
+@declare_units(
+    tas="[temperature]",
+    tas_25="[temperature]",
+    pr="[precipitation]",
+    pr_25="[precipitation]",
+)
 def cold_and_dry_days(
-    tas: xarray.DataArray, tgin25, pr, wet25, freq: str = "YS"
+    tas: xarray.DataArray,
+    tas_25: xarray.DataArray,
+    pr: xarray.DataArray,
+    pr_25: xarray.DataArray,
+    freq: str = "YS",
 ) -> xarray.DataArray:
     r"""Cold and dry days.
 
@@ -129,13 +155,16 @@ def cold_and_dry_days(
     Parameters
     ----------
     tas : xarray.DataArray
-      Mean daily temperature values,
-    tgin25 : xarray.DataArray
+      Mean daily temperature values
+    tas_25 : xarray.DataArray
       First quartile of daily mean temperature computed by month.
     pr : xarray.DataArray
       Daily precipitation.
-    wet25 : xarray.DataArray
+    pr_25 : xarray.DataArray
       First quartile of daily total precipitation computed by month.
+      Warning:
+        Before computing the percentiles, all the precipitation below 1mm must be filtered out !
+        Otherwise the percentiles will include non wet days.
     freq : str
       Resampling frequency.
 
@@ -146,6 +175,8 @@ def cold_and_dry_days(
 
     Notes
     -----
+    Bootstrapping is not available for quartiles because it would make no significant difference to bootstrap percentiles so far from the extremes.
+
     Formula to be written [cold_dry_days]_.
 
     References
@@ -153,27 +184,212 @@ def cold_and_dry_days(
     .. [cold_dry_days] Beniston, M. (2009). Trends in joint quantiles of temperature and precipitation in Europe
         since 1901 and projected for 2100. Geophysical Research Letters, 36(7). https://doi.org/10.1029/2008GL037119
     """
-    raise NotImplementedError
-    # There is an issue with the 1 mm threshold. It makes no sense to assume a day with < 1mm is not dry.
-    #
-    # c1 = tas < convert_units_to(tgin25, tas)
-    # c2 = (pr > convert_units_to('1 mm', pr)) * (pr < convert_units_to(wet25, pr))
+    tas_25 = convert_units_to(tas_25, tas)
+    thresh = resample_doy(tas_25, tas)
+    tg25 = tas < thresh
 
-    # c = (c1 * c2) * 1
-    # return c.resample(time=freq).sum(dim='time')
+    pr_25 = convert_units_to(pr_25, pr)
+    thresh = resample_doy(pr_25, pr)
+    pr25 = pr < thresh
+
+    cold_and_dry = np.logical_and(tg25, pr25).resample(time=freq).sum(dim="time")
+    return to_agg_units(cold_and_dry, tas, "count")
+
+
+@declare_units(
+    tas="[temperature]",
+    tas_75="[temperature]",
+    pr="[precipitation]",
+    pr_25="[precipitation]",
+)
+def warm_and_dry_days(
+    tas: xarray.DataArray,
+    tas_75: xarray.DataArray,
+    pr: xarray.DataArray,
+    pr_25: xarray.DataArray,
+    freq: str = "YS",
+) -> xarray.DataArray:
+    r"""Warm and dry days.
+
+    Returns the total number of days where "warm" and "Dry" conditions coincide.
+
+    Parameters
+    ----------
+    tas : xarray.DataArray
+      Mean daily temperature values
+    tas_75 : xarray.DataArray
+      Third quartile of daily mean temperature computed by month.
+    pr : xarray.DataArray
+      Daily precipitation.
+    pr_25 : xarray.DataArray
+      First quartile of daily total precipitation computed by month.
+      Warning:
+        Before computing the percentiles, all the precipitation below 1mm must be filtered out !
+        Otherwise the percentiles will include non wet days.
+    freq : str
+      Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray,
+      The total number of days where warm and dry conditions coincide.
+
+    Notes
+    -----
+    Bootstrapping is not available for quartiles because it would make no significant difference to bootstrap percentiles so far from the extremes.
+
+    Formula to be written [warm_dry_days]_.
+
+    References
+    ----------
+    .. [warm_dry_days] Beniston, M. (2009). Trends in joint quantiles of temperature and precipitation in Europe
+        since 1901 and projected for 2100. Geophysical Research Letters, 36(7). https://doi.org/10.1029/2008GL037119
+    """
+    tas_75 = convert_units_to(tas_75, tas)
+    thresh = resample_doy(tas_75, tas)
+    tg75 = tas > thresh
+
+    pr_25 = convert_units_to(pr_25, pr)
+    thresh = resample_doy(pr_25, pr)
+    pr25 = pr < thresh
+
+    warm_and_dry = np.logical_and(tg75, pr25).resample(time=freq).sum(dim="time")
+    return to_agg_units(warm_and_dry, tas, "count")
+
+
+@declare_units(
+    tas="[temperature]",
+    tas_75="[temperature]",
+    pr="[precipitation]",
+    pr_75="[precipitation]",
+)
+def warm_and_wet_days(
+    tas: xarray.DataArray,
+    tas_75: xarray.DataArray,
+    pr: xarray.DataArray,
+    pr_75: xarray.DataArray,
+    freq: str = "YS",
+) -> xarray.DataArray:
+    r"""Warm and wet days.
+
+    Returns the total number of days where "warm" and "wet" conditions coincide.
+
+    Parameters
+    ----------
+    tas : xarray.DataArray
+      Mean daily temperature values
+    tas_75 : xarray.DataArray
+      Third quartile of daily mean temperature computed by month.
+    pr : xarray.DataArray
+      Daily precipitation.
+    pr_75 : xarray.DataArray
+      Third quartile of daily total precipitation computed by month.
+      Warning:
+        Before computing the percentiles, all the precipitation below 1mm must be filtered out !
+        Otherwise the percentiles will include non wet days.
+    freq : str
+      Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray,
+      The total number of days where warm and wet conditions coincide.
+
+    Notes
+    -----
+    Bootstrapping is not available for quartiles because it would make no significant difference to bootstrap percentiles so far from the extremes.
+
+    Formula to be written [warm_wet_days]_.
+
+    References
+    ----------
+    .. [warm_wet_days] Beniston, M. (2009). Trends in joint quantiles of temperature and precipitation in Europe
+        since 1901 and projected for 2100. Geophysical Research Letters, 36(7). https://doi.org/10.1029/2008GL037119
+    """
+    tas_75 = convert_units_to(tas_75, tas)
+    thresh = resample_doy(tas_75, tas)
+    tg75 = tas > thresh
+
+    pr_75 = convert_units_to(pr_75, pr)
+    thresh = resample_doy(pr_75, pr)
+    pr75 = pr > thresh
+
+    warm_and_wet = np.logical_and(tg75, pr75).resample(time=freq).sum(dim="time")
+    return to_agg_units(warm_and_wet, tas, "count")
+
+
+@declare_units(
+    tas="[temperature]",
+    tas_25="[temperature]",
+    pr="[precipitation]",
+    pr_75="[precipitation]",
+)
+def cold_and_wet_days(
+    tas: xarray.DataArray,
+    tas_25: xarray.DataArray,
+    pr: xarray.DataArray,
+    pr_75: xarray.DataArray,
+    freq: str = "YS",
+) -> xarray.DataArray:
+    r"""cold and wet days.
+
+    Returns the total number of days where "cold" and "wet" conditions coincide.
+
+    Parameters
+    ----------
+    tas : xarray.DataArray
+      Mean daily temperature values
+    tas_25 : xarray.DataArray
+      First quartile of daily mean temperature computed by month.
+    pr : xarray.DataArray
+      Daily precipitation.
+    pr_75 : xarray.DataArray
+      Third quartile of daily total precipitation computed by month.
+      Warning:
+        Before computing the percentiles, all the precipitation below 1mm must be filtered out !
+        Otherwise the percentiles will include non wet days.
+    freq : str
+      Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray,
+      The total number of days where cold and wet conditions coincide.
+
+    Notes
+    -----
+    Bootstrapping is not available for quartiles because it would make no significant difference to bootstrap percentiles so far from the extremes.
+
+    Formula to be written [cold_wet_days]_.
+
+    References
+    ----------
+    .. [cold_wet_days] Beniston, M. (2009). Trends in joint quantiles of temperature and precipitation in Europe
+        since 1901 and projected for 2100. Geophysical Research Letters, 36(7). https://doi.org/10.1029/2008GL037119
+    """
+    tas_25 = convert_units_to(tas_25, tas)
+    thresh = resample_doy(tas_25, tas)
+    tg25 = tas < thresh
+
+    pr_75 = convert_units_to(pr_75, pr)
+    thresh = resample_doy(pr_75, pr)
+    pr75 = pr > thresh
+
+    cold_and_wet = np.logical_and(tg25, pr75).resample(time=freq).sum(dim="time")
+    return to_agg_units(cold_and_wet, tas, "count")
 
 
 @declare_units(
-    tasmax="[temperature]",
     tasmin="[temperature]",
-    thresh_tasmax="[temperature]",
+    tasmax="[temperature]",
     thresh_tasmin="[temperature]",
+    thresh_tasmax="[temperature]",
 )
 def daily_freezethaw_cycles(
     tasmin: xarray.DataArray,
     tasmax: xarray.DataArray,
-    thresh_tasmax: str = "0 degC",
     thresh_tasmin: str = "0 degC",
+    thresh_tasmax: str = "0 degC",
     freq: str = "YS",
 ) -> xarray.DataArray:  # noqa: D401
     r"""Number of days with a diurnal freeze-thaw cycle.
@@ -186,10 +402,10 @@ def daily_freezethaw_cycles(
       Minimum daily temperature.
     tasmax : xarray.DataArray
       Maximum daily temperature.
-    thresh_tasmax : str
-      The temperature threshold needed to trigger a thaw event.
     thresh_tasmin : str
       The temperature threshold needed to trigger a freeze event.
+    thresh_tasmax : str
+      The temperature threshold needed to trigger a thaw event.
     freq : str
       Resampling frequency.
 
@@ -209,12 +425,99 @@ def daily_freezethaw_cycles(
         \sum_{i \in \phi} [ TX_{i} > 0℃ ] [ TN_{i} <  0℃ ]
 
     where :math:`[P]` is 1 if :math:`P` is true, and 0 if false.
+
+    Warnings
+    --------
+    The `daily_freezethaw_cycles` indice is being deprecated in favour of `multiday_temperature_swing` with
+    `thresh_tasmax='0 degC, thresh_tasmin='0 degC', window=1, op='sum'` by default. The indicator reflects this change.
+    This indice will be removed in a future version of xclim.
+    """
+    warnings.warn(
+        "The `daily_freezethaw_cycles` indice is being deprecated in favour of `multiday_temperature_swing` with "
+        "`thresh_tasmax='0 degC, thresh_tasmin='0 degC', window=1, op='sum'` by default. "
+        "This indice will be removed in `xclim>=0.28.0`. Please update your scripts accordingly.",
+        UserWarning,
+        stacklevel=3,
+    )
+
+    return multiday_temperature_swing(
+        tasmin=tasmin,
+        tasmax=tasmax,
+        thresh_tasmin=thresh_tasmin,
+        thresh_tasmax=thresh_tasmax,
+        window=1,
+        op="sum",
+        freq=freq,
+    )
+
+
+@declare_units(
+    tasmin="[temperature]",
+    tasmax="[temperature]",
+    thresh_tasmin="[temperature]",
+    thresh_tasmax="[temperature]",
+)
+def multiday_temperature_swing(
+    tasmin: xarray.DataArray,
+    tasmax: xarray.DataArray,
+    thresh_tasmin: str = "0 degC",
+    thresh_tasmax: str = "0 degC",
+    window: int = 1,
+    op: str = "mean",
+    freq: str = "YS",
+) -> xarray.DataArray:  # noqa: D401
+    r"""Statistics of consecutive diurnal temperature swing events.
+
+    A diurnal swing of max and min temperature event is when Tmax > thresh_tasmax and Tmin <= thresh_tasmin. This indice
+    finds all days that constitute these events and computes statistics over the length and frequency of these events.
+
+    Parameters
+    ----------
+    tasmin : xarray.DataArray
+      Minimum daily temperature.
+    tasmax : xarray.DataArray
+      Maximum daily temperature.
+    thresh_tasmin : str
+      The temperature threshold needed to trigger a freeze event.
+    thresh_tasmax : str
+      The temperature threshold needed to trigger a thaw event.
+    window : int
+      The minimal length of spells to be included in the statistics.
+    op : {'mean', 'sum', 'max', 'min', 'std', 'count'}
+      The statistical operation to use when reducing the list of spell lengths.
+    freq : str
+      Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray, [time]
+      {freq} {op} length of diurnal temperature cycles exceeding thresholds.
+
+    Notes
+    -----
+    Let :math:`TX_{i}` be the maximum temperature at day :math:`i` and :math:`TN_{i}` be
+    the daily minimum temperature at day :math:`i`. Then freeze thaw spells during a given
+    period are consecutive days where:
+
+    .. math::
+
+        TX_{i} > 0℃ \land TN_{i} <  0℃
+
+    This indice returns a given statistic of the found lengths, optionally dropping those shorter than the `window`
+    argument. For example, `window=1` and `op='sum'` returns the same value as :py:func:`daily_freezethaw_cycles`.
     """
     thaw_threshold = convert_units_to(thresh_tasmax, tasmax)
     freeze_threshold = convert_units_to(thresh_tasmin, tasmin)
 
-    ft = (tasmin <= freeze_threshold) * (tasmax > thaw_threshold) * 1
-    out = ft.resample(time=freq).sum(dim="time")
+    ft = (tasmin <= freeze_threshold) * (tasmax > thaw_threshold)
+    if op == "count":
+        out = ft.resample(time=freq).map(
+            rl.windowed_run_events, window=window, dim="time"
+        )
+    else:
+        out = ft.resample(time=freq).map(
+            rl.rle_statistics, reducer=op, window=window, dim="time"
+        )
     return to_agg_units(out, tasmin, "count")
 
 
@@ -242,7 +545,7 @@ def daily_temperature_range(
 
     Returns
     -------
-    xarray.DataArray, [same as tasmin]
+    xarray.DataArray, [same units as tasmin]
       The average variation in daily temperature range for the given time period.
 
     Notes
@@ -283,7 +586,7 @@ def daily_temperature_range_variability(
 
     Returns
     -------
-    xarray.DataArray, [same as tasmin]
+    xarray.DataArray, [same units as tasmin]
       The average day-to-day variation in daily temperature range for the given time period.
 
     Notes
@@ -323,7 +626,7 @@ def extreme_temperature_range(
 
     Returns
     -------
-    xarray.DataArray, [same as tasmin]
+    xarray.DataArray, [same units as tasmin]
       Extreme intra-period temperature range for the given time period.
 
     Notes
@@ -383,7 +686,7 @@ def heat_wave_frequency(
     Returns
     -------
     xarray.DataArray, [dimensionless]
-      Number of heatwave at the wanted frequency
+      Number of heatwave at the requested frequency.
 
     Notes
     -----
@@ -452,7 +755,7 @@ def heat_wave_max_length(
     Returns
     -------
     xarray.DataArray, [time]
-      Maximum length of heatwave at the wanted frequency
+      Maximum length of heatwave at the requested frequency.
 
     Notes
     -----
@@ -519,7 +822,7 @@ def heat_wave_total_length(
     Returns
     -------
     xarray.DataArray, [time]
-      Total length of heatwave at the wanted frequency
+      Total length of heatwave at the requested frequency.
 
     Notes
     -----
@@ -567,7 +870,7 @@ def liquid_precip_ratio(
     Returns
     -------
     xarray.DataArray, [dimensionless]
-      Ratio of rainfall to total precipitation
+      Ratio of rainfall to total precipitation.
 
     Notes
     -----
@@ -684,7 +987,7 @@ def rain_on_frozen_ground_days(
     Returns
     -------
     xarray.DataArray, [time]
-      The number of rain on frozen ground events per period
+      The number of rain on frozen ground events per period.
 
     Notes
     -----
@@ -702,7 +1005,6 @@ def rain_on_frozen_ground_days(
         TG_{i} ≤ 0℃
 
     is true for continuous periods where :math:`i ≥ 7`
-
     """
     t = convert_units_to(thresh, pr)
     frz = convert_units_to("0 C", tas)
@@ -771,11 +1073,13 @@ def high_precip_low_temp(
 
 
 @declare_units(pr="[precipitation]", per="[precipitation]", thresh="[precipitation]")
+@percentile_bootstrap
 def days_over_precip_thresh(
     pr: xarray.DataArray,
     per: xarray.DataArray,
     thresh: str = "1 mm/day",
     freq: str = "YS",
+    bootstrap: bool = False,
 ) -> xarray.DataArray:  # noqa: D401
     r"""Number of wet days with daily precipitation over a given percentile.
 
@@ -792,11 +1096,13 @@ def days_over_precip_thresh(
        Precipitation value over which a day is considered wet.
     freq : str
       Resampling frequency.
+    bootstrap : bool
+      Flag to run bootstrapping. Used by percentile_bootstrap decorator.
 
     Returns
     -------
     xarray.DataArray, [time]
-      Count of days with daily precipitation above the given percentile [days]
+      Count of days with daily precipitation above the given percentile [days].
 
     Examples
     --------
@@ -819,11 +1125,13 @@ def days_over_precip_thresh(
 
 
 @declare_units(pr="[precipitation]", per="[precipitation]", thresh="[precipitation]")
+@percentile_bootstrap
 def fraction_over_precip_thresh(
     pr: xarray.DataArray,
     per: xarray.DataArray,
     thresh: str = "1 mm/day",
     freq: str = "YS",
+    bootstrap: bool = False,
 ) -> xarray.DataArray:
     r"""Fraction of precipitation due to wet days with daily precipitation over a given percentile.
 
@@ -840,6 +1148,8 @@ def fraction_over_precip_thresh(
        Precipitation value over which a day is considered wet.
     freq : str
       Resampling frequency.
+    bootstrap : bool
+      Flag to run bootstrapping. Used by percentile_bootstrap decorator.
 
     Returns
     -------
@@ -867,8 +1177,12 @@ def fraction_over_precip_thresh(
 
 
 @declare_units(tas="[temperature]", t90="[temperature]")
+@percentile_bootstrap
 def tg90p(
-    tas: xarray.DataArray, t90: xarray.DataArray, freq: str = "YS"
+    tas: xarray.DataArray,
+    t90: xarray.DataArray,
+    freq: str = "YS",
+    bootstrap: bool = False,
 ) -> xarray.DataArray:  # noqa: D401
     r"""Number of days with daily mean temperature over the 90th percentile.
 
@@ -882,11 +1196,13 @@ def tg90p(
       90th percentile of daily mean temperature.
     freq : str
       Resampling frequency.
+    bootstrap : bool
+      Flag to run bootstrapping. Used by percentile_bootstrap decorator.
 
     Returns
     -------
     xarray.DataArray, [time]
-      Count of days with daily mean temperature below the 10th percentile [days]
+      Count of days with daily mean temperature below the 10th percentile [days].
 
     Notes
     -----
@@ -911,8 +1227,12 @@ def tg90p(
 
 
 @declare_units(tas="[temperature]", t10="[temperature]")
+@percentile_bootstrap
 def tg10p(
-    tas: xarray.DataArray, t10: xarray.DataArray, freq: str = "YS"
+    tas: xarray.DataArray,
+    t10: xarray.DataArray,
+    freq: str = "YS",
+    bootstrap: bool = False,
 ) -> xarray.DataArray:  # noqa: D401
     r"""Number of days with daily mean temperature below the 10th percentile.
 
@@ -926,11 +1246,13 @@ def tg10p(
       10th percentile of daily mean temperature.
     freq : str
       Resampling frequency.
+    bootstrap : bool
+      Flag to run bootstrapping. Used by percentile_bootstrap decorator.
 
     Returns
     -------
     xarray.DataArray, [time]
-      Count of days with daily mean temperature below the 10th percentile [days]
+      Count of days with daily mean temperature below the 10th percentile [days].
 
     Notes
     -----
@@ -955,8 +1277,12 @@ def tg10p(
 
 
 @declare_units(tasmin="[temperature]", t90="[temperature]")
+@percentile_bootstrap
 def tn90p(
-    tasmin: xarray.DataArray, t90: xarray.DataArray, freq: str = "YS"
+    tasmin: xarray.DataArray,
+    t90: xarray.DataArray,
+    freq: str = "YS",
+    bootstrap: bool = False,
 ) -> xarray.DataArray:  # noqa: D401
     r"""Number of days with daily minimum temperature over the 90th percentile.
 
@@ -970,11 +1296,13 @@ def tn90p(
       90th percentile of daily minimum temperature.
     freq : str
       Resampling frequency.
+    bootstrap : bool
+      Flag to run bootstrapping. Used by percentile_bootstrap decorator.
 
     Returns
     -------
     xarray.DataArray, [time]
-      Count of days with daily minimum temperature below the 10th percentile [days]
+      Count of days with daily minimum temperature below the 10th percentile [days].
 
     Notes
     -----
@@ -999,8 +1327,12 @@ def tn90p(
 
 
 @declare_units(tasmin="[temperature]", t10="[temperature]")
+@percentile_bootstrap
 def tn10p(
-    tasmin: xarray.DataArray, t10: xarray.DataArray, freq: str = "YS"
+    tasmin: xarray.DataArray,
+    t10: xarray.DataArray,
+    freq: str = "YS",
+    bootstrap: bool = False,
 ) -> xarray.DataArray:  # noqa: D401
     r"""Number of days with daily minimum temperature below the 10th percentile.
 
@@ -1014,11 +1346,13 @@ def tn10p(
       10th percentile of daily minimum temperature.
     freq : str
       Resampling frequency.
+    bootstrap : bool
+      Flag to run bootstrapping. Used by percentile_bootstrap decorator.
 
     Returns
     -------
     xarray.DataArray, [time]
-      Count of days with daily minimum temperature below the 10th percentile [days]
+      Count of days with daily minimum temperature below the 10th percentile [days].
 
     Notes
     -----
@@ -1042,9 +1376,13 @@ def tn10p(
     return to_agg_units(out, tasmin, "count")
 
 
-@declare_units(tasmax="[temperature]", t90="[temperature]")
+@declare_units(tasmax="[temperature]")
+@percentile_bootstrap
 def tx90p(
-    tasmax: xarray.DataArray, t90: xarray.DataArray, freq: str = "YS"
+    tasmax: xarray.DataArray,
+    t90: xarray.DataArray,
+    freq: str = "YS",
+    bootstrap: bool = False,
 ) -> xarray.DataArray:  # noqa: D401
     r"""Number of days with daily maximum temperature over the 90th percentile.
 
@@ -1058,11 +1396,13 @@ def tx90p(
       90th percentile of daily maximum temperature.
     freq : str
       Resampling frequency.
+    bootstrap : bool
+      Flag to run bootstrapping. Used by percentile_bootstrap decorator.
 
     Returns
     -------
     xarray.DataArray, [time]
-      Count of days with daily maximum temperature below the 10th percentile [days]
+      Count of days with daily maximum temperature below the 10th percentile [days].
 
     Notes
     -----
@@ -1087,8 +1427,12 @@ def tx90p(
 
 
 @declare_units(tasmax="[temperature]", t10="[temperature]")
+@percentile_bootstrap
 def tx10p(
-    tasmax: xarray.DataArray, t10: xarray.DataArray, freq: str = "YS"
+    tasmax: xarray.DataArray,
+    t10: xarray.DataArray,
+    freq: str = "YS",
+    bootstrap: bool = False,
 ) -> xarray.DataArray:  # noqa: D401
     r"""Number of days with daily maximum temperature below the 10th percentile.
 
@@ -1102,11 +1446,13 @@ def tx10p(
       10th percentile of daily maximum temperature.
     freq : str
       Resampling frequency.
+    bootstrap : bool
+      Flag to run bootstrapping. Used by percentile_bootstrap decorator.
 
     Returns
     -------
     xarray.DataArray, [time]
-      Count of days with daily maximum temperature below the 10th percentile [days]
+      Count of days with daily maximum temperature below the 10th percentile [days].
 
     Notes
     -----
@@ -1163,9 +1509,7 @@ def tx_tn_days_above(
     Returns
     -------
     xarray.DataArray, [time]
-      the number of days with tasmin > thresh_tasmin and
-      tasmax > thresh_tasamax per period
-
+      the number of days with tasmin > thresh_tasmin and tasmax > thresh_tasmax per period.
 
     Notes
     -----
@@ -1193,8 +1537,13 @@ def tx_tn_days_above(
 
 
 @declare_units(tasmax="[temperature]", tx90="[temperature]")
+@percentile_bootstrap
 def warm_spell_duration_index(
-    tasmax: xarray.DataArray, tx90: xarray.DataArray, window: int = 6, freq: str = "YS"
+    tasmax: xarray.DataArray,
+    tx90: xarray.DataArray,
+    window: int = 6,
+    freq: str = "YS",
+    bootstrap: bool = False,
 ) -> xarray.DataArray:
     r"""Warm spell duration index.
 
@@ -1212,11 +1561,13 @@ def warm_spell_duration_index(
       Minimum number of days with temperature above threshold to qualify as a warm spell.
     freq : str
       Resampling frequency.
+    bootstrap : bool
+      Flag to run bootstrapping. Used by percentile_bootstrap decorator.
 
     Returns
     -------
     xarray.DataArray, [time]
-      Warm spell duration index
+      Warm spell duration index.
 
     Examples
     --------
@@ -1241,7 +1592,7 @@ def warm_spell_duration_index(
     # Create time series out of doy values.
     thresh = resample_doy(thresh, tasmax)
 
-    above = tasmax > thresh
+    above: DataArray = tasmax > thresh
 
     out = above.resample(time=freq).map(
         rl.windowed_run_count, window=window, dim="time"
@@ -1276,7 +1627,7 @@ def winter_rain_ratio(
     Returns
     -------
     xarray.DataArray
-      Ratio of rainfall to total precipitation during winter months (DJF)
+      Ratio of rainfall to total precipitation during winter months (DJF).
     """
     ratio = liquid_precip_ratio(pr, prsn, tas, freq=freq)
     winter = ratio.indexes["time"].month == 12
