@@ -1,4 +1,5 @@
 # noqa: D100
+import warnings
 from typing import Optional
 
 import numpy as np
@@ -25,6 +26,7 @@ from .generic import domain_count, threshold_count
 # -------------------------------------------------- #
 
 __all__ = [
+    "calm_days",
     "cold_spell_days",
     "cold_spell_frequency",
     "daily_pr_intensity",
@@ -48,8 +50,13 @@ __all__ = [
     "hot_spell_frequency",
     "hot_spell_max_length",
     "snow_cover_duration",
+    "tn_days_above",
     "tn_days_below",
+    "tg_days_above",
+    "tg_days_below",
     "tx_days_above",
+    "tx_days_below",
+    "tropical_nights",
     "warm_day_frequency",
     "warm_night_frequency",
     "wetdays",
@@ -62,17 +69,54 @@ __all__ = [
     "maximum_consecutive_wet_days",
     "sea_ice_area",
     "sea_ice_extent",
-    "tropical_nights",
+    "windy_days",
 ]
 
 
+@declare_units(sfcWind="[speed]", thresh="[speed]")
+def calm_days(
+    sfcWind: xarray.DataArray, thresh: str = "2 m s-1", freq: str = "MS"
+) -> xarray.DataArray:
+    r"""Calm days.
+
+    The number of days with average near-surface wind speed below threshold.
+
+    Parameters
+    ----------
+    sfcWind : xarray.DataArray
+      Daily windspeed.
+    thresh : str
+      Threshold average near-surface wind speed on which to base evaluation.
+    freq : str
+      Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray, [time]
+      Number of days with average near-surface wind speed below threshold.
+
+    Notes
+    -----
+    Let :math:`WS_{ij}` be the windspeed at day :math:`i` of period :math:`j`. Then
+    counted is the number of days where:
+
+    .. math::
+
+        WS_{ij} < Threshold [m s-1]
+    """
+    thresh = convert_units_to(thresh, sfcWind)
+    out = threshold_count(sfcWind, "<", thresh, freq)
+    out = to_agg_units(out, sfcWind, "count")
+    return out
+
+
 @declare_units(tas="[temperature]", thresh="[temperature]")
 def cold_spell_days(
     tas: xarray.DataArray,
     thresh: str = "-10 degC",
     window: int = 5,
     freq: str = "AS-JUL",
-):
+) -> xarray.DataArray:
     r"""Cold spell days.
 
     The number of days that are part of cold spell events, defined as a sequence of consecutive days with mean daily
@@ -120,7 +164,7 @@ def cold_spell_frequency(
     thresh: str = "-10 degC",
     window: int = 5,
     freq: str = "AS-JUL",
-):
+) -> xarray.DataArray:
     r"""Cold spell frequency.
 
     The number of cold spell events, defined as a sequence of consecutive days with mean daily
@@ -183,7 +227,8 @@ def continuous_snow_cover_end(
 
     References
     ----------
-    Chaumont D., Mailhot A., Diaconescu E.P., Fournier É., Logan T. 2017: Élaboration du portrait bioclimatique futur du Nunavik – Tome II. [Rapport présenté au Ministère de la forêt, de la faune et des parcs], Ouranos.
+    Chaumont D., Mailhot A., Diaconescu E.P., Fournier É., Logan T. 2017: Élaboration du portrait bioclimatique futur
+    du Nunavik – Tome II. [Rapport présenté au Ministère de la forêt, de la faune et des parcs], Ouranos.
     """
     thresh = convert_units_to(thresh, snd)
     cond = snd >= thresh
@@ -225,7 +270,8 @@ def continuous_snow_cover_start(
 
     References
     ----------
-    Chaumont D., Mailhot A., Diaconescu E.P., Fournier É., Logan T. 2017: Élaboration du portrait bioclimatique futur du Nunavik – Tome II. [Rapport présenté au Ministère de la forêt, de la faune et des parcs], Ouranos.
+    Chaumont D., Mailhot A., Diaconescu E.P., Fournier É., Logan T. 2017: Élaboration du portrait bioclimatique futur
+    du Nunavik – Tome II. [Rapport présenté au Ministère de la forêt, de la faune et des parcs], Ouranos.
     """
     thresh = convert_units_to(thresh, snd)
     cond = snd >= thresh
@@ -247,7 +293,7 @@ def continuous_snow_cover_start(
 @declare_units(pr="[precipitation]", thresh="[precipitation]")
 def daily_pr_intensity(
     pr: xarray.DataArray, thresh: str = "1 mm/day", freq: str = "YS"
-):
+) -> xarray.DataArray:
     r"""Average daily precipitation intensity.
 
     Return the average precipitation over wet days.
@@ -307,7 +353,9 @@ def daily_pr_intensity(
 
 
 @declare_units(pr="[precipitation]", thresh="[precipitation]")
-def dry_days(pr: xarray.DataArray, thresh: str = "0.2 mm/d", freq: str = "YS"):
+def dry_days(
+    pr: xarray.DataArray, thresh: str = "0.2 mm/d", freq: str = "YS"
+) -> xarray.DataArray:
     r"""Dry days.
 
     The number of days with daily precipitation below threshold.
@@ -344,7 +392,7 @@ def dry_days(pr: xarray.DataArray, thresh: str = "0.2 mm/d", freq: str = "YS"):
 @declare_units(pr="[precipitation]", thresh="[precipitation]")
 def maximum_consecutive_wet_days(
     pr: xarray.DataArray, thresh: str = "1 mm/day", freq: str = "YS"
-):
+) -> xarray.DataArray:
     r"""Consecutive wet days.
 
     Returns the maximum number of consecutive wet days.
@@ -389,7 +437,7 @@ def maximum_consecutive_wet_days(
 @declare_units(tas="[temperature]", thresh="[temperature]")
 def cooling_degree_days(
     tas: xarray.DataArray, thresh: str = "18 degC", freq: str = "YS"
-):
+) -> xarray.DataArray:
     r"""Cooling degree days.
 
     Sum of degree days above the temperature threshold at which spaces are cooled.
@@ -429,7 +477,7 @@ def cooling_degree_days(
 @declare_units(tas="[temperature]", thresh="[temperature]")
 def freshet_start(
     tas: xarray.DataArray, thresh: str = "0 degC", window: int = 5, freq: str = "YS"
-):
+) -> xarray.DataArray:
     r"""First day consistently exceeding threshold temperature.
 
     Returns first day of period where a temperature threshold is exceeded
@@ -474,7 +522,7 @@ def freshet_start(
 @declare_units(tas="[temperature]", thresh="[temperature]")
 def growing_degree_days(
     tas: xarray.DataArray, thresh: str = "4.0 degC", freq: str = "YS"
-):
+) -> xarray.DataArray:
     r"""Growing degree-days over threshold temperature value.
 
     The sum of degree-days over the threshold temperature.
@@ -514,7 +562,7 @@ def growing_season_end(
     mid_date: DayOfYearStr = "07-01",
     window: int = 5,
     freq: str = "YS",
-):
+) -> xarray.DataArray:
     r"""End of the growing season.
 
     Day of the year of the start of a sequence of days with a temperature consistently
@@ -561,7 +609,7 @@ def growing_season_length(
     window: int = 6,
     mid_date: DayOfYearStr = "07-01",
     freq: str = "YS",
-):
+) -> xarray.DataArray:
     r"""Growing season length.
 
     The number of days between the first occurrence of at least six consecutive days
@@ -635,7 +683,7 @@ def frost_season_length(
     mid_date: Optional[DayOfYearStr] = "01-01",
     thresh: str = "0.0 degC",
     freq: str = "AS-JUL",
-):
+) -> xarray.DataArray:
     r"""Frost season length.
 
     The number of days between the first occurrence of at least N (def: 5) consecutive days
@@ -708,7 +756,7 @@ def last_spring_frost(
     before_date: DayOfYearStr = "07-01",
     window: int = 1,
     freq: str = "YS",
-):
+) -> xarray.DataArray:
     r"""Last day of temperatures inferior to a threshold temperature.
 
     Returns last day of period where a temperature is inferior to a threshold
@@ -731,7 +779,7 @@ def last_spring_frost(
     -------
     xarray.DataArray, [dimensionless]
       Day of the year when temperature is inferior to a threshold over a given number of days for the first time.
-      If there is no such day, return np.nan.
+      If there is no such day, returns np.nan.
     """
     thresh = convert_units_to(thresh, tas)
     cond = tas < thresh
@@ -754,7 +802,7 @@ def first_day_below(
     after_date: DayOfYearStr = "07-01",
     window: int = 1,
     freq: str = "YS",
-):
+) -> xarray.DataArray:
     r"""First day of temperatures inferior to a threshold temperature.
 
     Returns first day of period where a temperature is inferior to a threshold
@@ -779,7 +827,7 @@ def first_day_below(
     -------
     xarray.DataArray, [dimensionless]
       Day of the year when minimum temperature is inferior to a threshold over a given number of days for the first time.
-      If there is no such day, return np.nan.
+      If there is no such day, returns np.nan.
     """
     thresh = convert_units_to(thresh, tasmin)
     cond = tasmin < thresh
@@ -802,7 +850,7 @@ def first_day_above(
     after_date: DayOfYearStr = "01-01",
     window: int = 1,
     freq: str = "YS",
-):
+) -> xarray.DataArray:
     r"""First day of temperatures superior to a threshold temperature.
 
     Returns first day of period where a temperature is superior to a threshold
@@ -827,7 +875,7 @@ def first_day_above(
     -------
     xarray.DataArray, [dimensionless]
       Day of the year when minimum temperature is superior to a threshold over a given number of days for the first time.
-      If there is no such day, return np.nan.
+      If there is no such day, returns np.nan.
     """
     thresh = convert_units_to(thresh, tasmin)
     cond = tasmin > thresh
@@ -848,7 +896,7 @@ def first_snowfall(
     prsn: xarray.DataArray,
     thresh: str = "0.5 mm/day",
     freq: str = "AS-JUL",
-):
+) -> xarray.DataArray:
     r"""First day with solid precipitation above a threshold.
 
     Returns the first day of a period where the solid precipitation exceeds a threshold.
@@ -866,8 +914,8 @@ def first_snowfall(
     Returns
     -------
     xarray.DataArray, [dimensionless]
-      First day of the year when the solid precipitation  is superior to a threshold,
-      If there is no such day, return np.nan.
+      First day of the year when the solid precipitation is superior to a threshold.
+      If there is no such day, returns np.nan.
 
     References
     ----------
@@ -892,7 +940,7 @@ def last_snowfall(
     prsn: xarray.DataArray,
     thresh: str = "0.5 mm/day",
     freq: str = "AS-JUL",
-):
+) -> xarray.DataArray:
     r"""Last day with solid precipitation above a threshold.
 
     Returns the last day of a period where the solid precipitation exceeds a threshold.
@@ -910,8 +958,8 @@ def last_snowfall(
     Returns
     -------
     xarray.DataArray, [dimensionless]
-      Last day of the year when the solid precipitation is superior to a threshold,
-      If there is no such day, return np.nan.
+      Last day of the year when the solid precipitation is superior to a threshold.
+      If there is no such day, returns np.nan.
 
     References
     ----------
@@ -937,7 +985,7 @@ def days_with_snow(
     low: str = "0 kg m-2 s-1",
     high: str = "1E6 kg m-2 s-1",
     freq: str = "AS-JUL",
-):
+) -> xarray.DataArray:
     r"""Days with snow.
 
     Return the number of days where snowfall is within low and high thresholds.
@@ -976,7 +1024,7 @@ def heat_wave_index(
     thresh: str = "25.0 degC",
     window: int = 5,
     freq: str = "YS",
-):
+) -> xarray.DataArray:
     """Heat wave index.
 
     Number of days that are part of a heatwave, defined as five or more consecutive days over 25℃.
@@ -1008,7 +1056,7 @@ def heat_wave_index(
 @declare_units(tas="[temperature]", thresh="[temperature]")
 def heating_degree_days(
     tas: xarray.DataArray, thresh: str = "17.0 degC", freq: str = "YS"
-):
+) -> xarray.DataArray:
     r"""Heating degree days.
 
     Sum of degree days below the temperature threshold at which spaces are heated.
@@ -1071,7 +1119,7 @@ def hot_spell_max_length(
     Returns
     -------
     xarray.DataArray, [time]
-      Maximum length of continuous hot days at the wanted frequency
+      Maximum length of continuous hot days at the wanted frequency.
 
     Notes
     -----
@@ -1184,11 +1232,47 @@ def snow_cover_duration(
     return to_agg_units(out, snd, "count")
 
 
+@declare_units(tasmin="[temperature]", thresh="[temperature]")
+def tn_days_above(
+    tasmin: xarray.DataArray, thresh: str = "20.0 degC", freq: str = "YS"
+):  # noqa: D401
+    """Number of days with tasmin above a threshold (number of tropical nights).
+
+    Number of days where daily minimum temperature exceeds a threshold.
+
+    Parameters
+    ----------
+    tasmin : xarray.DataArray
+      Minimum daily temperature.
+    thresh : str
+      Threshold temperature on which to base evaluation.
+    freq : str
+      Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray, [time]
+      Number of days where tasmin > threshold.
+
+    Notes
+    -----
+    Let :math:`TN_{ij}` be the daily minimum temperature at day :math:`i` of period :math:`j`. Then
+    counted is the number of days where:
+
+    .. math::
+
+        TN_{ij} > Threshold [℃]
+    """
+    thresh = convert_units_to(thresh, tasmin)
+    f = threshold_count(tasmin, ">", thresh, freq)
+    return to_agg_units(f, tasmin, "count")
+
+
 @declare_units(tasmin="[temperature]", thresh="[temperature]")
 def tn_days_below(
     tasmin: xarray.DataArray, thresh: str = "-10.0 degC", freq: str = "YS"
-):  # noqa: D401
-    """Number of days with tmin below a threshold.
+) -> xarray.DataArray:  # noqa: D401
+    """Number of days with tasmin below a threshold.
 
     Number of days where daily minimum temperature is below a threshold.
 
@@ -1204,7 +1288,7 @@ def tn_days_below(
     Returns
     -------
     xarray.DataArray, [time]
-      Number of days Tmin < threshold.
+      Number of days where tasmin < threshold.
 
     Notes
     -----
@@ -1213,20 +1297,92 @@ def tn_days_below(
 
     .. math::
 
-        TX_{ij} < Threshold [℃]
+        TN_{ij} < Threshold [℃]
     """
     thresh = convert_units_to(thresh, tasmin)
     f1 = threshold_count(tasmin, "<", thresh, freq)
     return to_agg_units(f1, tasmin, "count")
 
 
+@declare_units(tas="[temperature]", thresh="[temperature]")
+def tg_days_above(
+    tas: xarray.DataArray, thresh: str = "10.0 degC", freq: str = "YS"
+):  # noqa: D401
+    """Number of days with tas above a threshold.
+
+    Number of days where daily mean temperature exceeds a threshold.
+
+    Parameters
+    ----------
+    tas : xarray.DataArray
+      Mean daily temperature.
+    thresh : str
+      Threshold temperature on which to base evaluation.
+    freq : str
+      Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray, [time]
+      Number of days where tas > threshold.
+
+    Notes
+    -----
+    Let :math:`TG_{ij}` be the daily mean temperature at day :math:`i` of period :math:`j`. Then
+    counted is the number of days where:
+
+    .. math::
+
+        TG_{ij} > Threshold [℃]
+    """
+    thresh = convert_units_to(thresh, tas)
+    f = threshold_count(tas, ">", thresh, freq)
+    return to_agg_units(f, tas, "count")
+
+
+@declare_units(tas="[temperature]", thresh="[temperature]")
+def tg_days_below(
+    tas: xarray.DataArray, thresh: str = "10.0 degC", freq: str = "YS"
+):  # noqa: D401
+    """Number of days with tas below a threshold.
+
+    Number of days where daily mean temperature is below a threshold.
+
+    Parameters
+    ----------
+    tas : xarray.DataArray
+      Mean daily temperature.
+    thresh : str
+      Threshold temperature on which to base evaluation.
+    freq : str
+      Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray, [time]
+      Number of days where tas < threshold.
+
+    Notes
+    -----
+    Let :math:`TG_{ij}` be the daily mean temperature at day :math:`i` of period :math:`j`. Then
+    counted is the number of days where:
+
+    .. math::
+
+        TG_{ij} < Threshold [℃]
+    """
+    thresh = convert_units_to(thresh, tas)
+    f1 = threshold_count(tas, "<", thresh, freq)
+    return to_agg_units(f1, tas, "count")
+
+
 @declare_units(tasmax="[temperature]", thresh="[temperature]")
 def tx_days_above(
     tasmax: xarray.DataArray, thresh: str = "25.0 degC", freq: str = "YS"
-):  # noqa: D401
-    """Number of summer days.
+) -> xarray.DataArray:  # noqa: D401
+    """Number of days with tasmax above a threshold (number of summer days).
 
-    Number of days where daily maximum temperature exceed a threshold.
+    Number of days where daily maximum temperature exceeds a threshold.
 
     Parameters
     ----------
@@ -1240,7 +1396,7 @@ def tx_days_above(
     Returns
     -------
     xarray.DataArray, [time]
-      Number of summer days.
+      Number of days where tasmax > threshold (number of summer days).
 
     Notes
     -----
@@ -1256,10 +1412,46 @@ def tx_days_above(
     return to_agg_units(f, tasmax, "count")
 
 
+@declare_units(tasmax="[temperature]", thresh="[temperature]")
+def tx_days_below(
+    tasmax: xarray.DataArray, thresh: str = "25.0 degC", freq: str = "YS"
+):  # noqa: D401
+    """Number of days with tmax below a threshold.
+
+    Number of days where daily maximum temperature is below a threshold.
+
+    Parameters
+    ----------
+    tasmax : xarray.DataArray
+      Maximum daily temperature.
+    thresh : str
+      Threshold temperature on which to base evaluation.
+    freq : str
+      Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray, [time]
+      Number of days where tasmin < threshold.
+
+    Notes
+    -----
+    Let :math:`TN_{ij}` be the daily minimum temperature at day :math:`i` of period :math:`j`. Then
+    counted is the number of days where:
+
+    .. math::
+
+        TN_{ij} < Threshold [℃]
+    """
+    thresh = convert_units_to(thresh, tasmax)
+    f1 = threshold_count(tasmax, "<", thresh, freq)
+    return to_agg_units(f1, tasmax, "count")
+
+
 @declare_units(tasmax="[temperature]", thresh="[temperature]")
 def warm_day_frequency(
     tasmax: xarray.DataArray, thresh: str = "30 degC", freq: str = "YS"
-):
+) -> xarray.DataArray:
     """Frequency of extreme warm days.
 
     Return the number of days with tasmax > thresh per period
@@ -1276,7 +1468,7 @@ def warm_day_frequency(
     Returns
     -------
     xarray.DataArray, [time]
-      Number of days exceeding threshold.
+      Number of days with tasmax > threshold per period.
 
     Notes
     -----
@@ -1296,7 +1488,7 @@ def warm_day_frequency(
 @declare_units(tasmin="[temperature]", thresh="[temperature]")
 def warm_night_frequency(
     tasmin: xarray.DataArray, thresh: str = "22 degC", freq: str = "YS"
-):
+) -> xarray.DataArray:
     """Frequency of extreme warm nights.
 
     Return the number of days with tasmin > thresh per period
@@ -1313,7 +1505,7 @@ def warm_night_frequency(
     Returns
     -------
     xarray.DataArray, [time]
-      The number of days with tasmin > thresh per period
+      Number of days with tasmin > threshold per period.
     """
     thresh = convert_units_to(thresh, tasmin)
     events = threshold_count(tasmin, ">", thresh, freq)
@@ -1321,7 +1513,9 @@ def warm_night_frequency(
 
 
 @declare_units(pr="[precipitation]", thresh="[precipitation]")
-def wetdays(pr: xarray.DataArray, thresh: str = "1.0 mm/day", freq: str = "YS"):
+def wetdays(
+    pr: xarray.DataArray, thresh: str = "1.0 mm/day", freq: str = "YS"
+) -> xarray.DataArray:
     """Wet days.
 
     Return the total number of days during period with precipitation over threshold.
@@ -1338,7 +1532,7 @@ def wetdays(pr: xarray.DataArray, thresh: str = "1.0 mm/day", freq: str = "YS"):
     Returns
     -------
     xarray.DataArray, [time]
-      The number of wet days for each period [day]
+      The number of wet days for each period [day].
 
     Examples
     --------
@@ -1360,7 +1554,7 @@ def maximum_consecutive_frost_days(
     tasmin: xarray.DataArray,
     thresh: str = "0.0 degC",
     freq: str = "AS-JUL",
-):
+) -> xarray.DataArray:
     r"""Maximum number of consecutive frost days (Tn < 0℃).
 
     The maximum number of consecutive days within the period where the
@@ -1379,7 +1573,7 @@ def maximum_consecutive_frost_days(
     Returns
     -------
     xarray.DataArray, [time]
-      The maximum number of consecutive frost days.
+      The maximum number of consecutive frost days (tasmin < threshold per period).
 
     Notes
     -----
@@ -1404,7 +1598,7 @@ def maximum_consecutive_frost_days(
 @declare_units(pr="[precipitation]", thresh="[precipitation]")
 def maximum_consecutive_dry_days(
     pr: xarray.DataArray, thresh: str = "1 mm/day", freq: str = "YS"
-):
+) -> xarray.DataArray:
     r"""Maximum number of consecutive dry days.
 
     Return the maximum number of consecutive days within the period where precipitation
@@ -1422,7 +1616,7 @@ def maximum_consecutive_dry_days(
     Returns
     -------
     xarray.DataArray, [time]
-      The maximum number of consecutive dry days.
+      The maximum number of consecutive dry days (precipitation < threshold per period).
 
     Notes
     -----
@@ -1447,7 +1641,7 @@ def maximum_consecutive_dry_days(
 @declare_units(tasmin="[temperature]", thresh="[temperature]")
 def maximum_consecutive_frost_free_days(
     tasmin: xarray.DataArray, thresh: str = "0 degC", freq: str = "YS"
-):
+) -> xarray.DataArray:
     r"""Maximum number of consecutive frost free days (Tn > 0℃).
 
     Return the maximum number of consecutive days within the period where the
@@ -1465,7 +1659,7 @@ def maximum_consecutive_frost_free_days(
     Returns
     -------
     xarray.DataArray, [time]
-      The maximum number of consecutive frost free days.
+      The maximum number of consecutive frost free days (tasmin > threshold per period).
 
     Notes
     -----
@@ -1490,8 +1684,8 @@ def maximum_consecutive_frost_free_days(
 @declare_units(tasmax="[temperature]", thresh="[temperature]")
 def maximum_consecutive_tx_days(
     tasmax: xarray.DataArray, thresh: str = "25 degC", freq: str = "YS"
-):
-    r"""Maximum number of consecutive summer days.
+) -> xarray.DataArray:
+    r"""Maximum number of consecutive days with tasmax above a threshold (summer days).
 
     Return the maximum number of consecutive days within the period where
     the maximum temperature is above a certain threshold.
@@ -1508,7 +1702,7 @@ def maximum_consecutive_tx_days(
     Returns
     -------
     xarray.DataArray, [time]
-      The maximum number of consecutive summer days.
+      The maximum number of days with tasmax > thresh per periods (summer days).
 
     Notes
     -----
@@ -1530,8 +1724,10 @@ def maximum_consecutive_tx_days(
     return to_agg_units(out, tasmax, "count")
 
 
-@declare_units(sic="[]", area="[area]", thresh="[]")
-def sea_ice_area(sic: xarray.DataArray, area: xarray.DataArray, thresh: str = "15 pct"):
+@declare_units(siconc="[]", areacello="[area]", thresh="[]")
+def sea_ice_area(
+    siconc: xarray.DataArray, areacello: xarray.DataArray, thresh: str = "15 pct"
+) -> xarray.DataArray:
     """Total sea ice area.
 
     Sea ice area measures the total sea ice covered area where sea ice concentration is above a threshold,
@@ -1539,17 +1735,17 @@ def sea_ice_area(sic: xarray.DataArray, area: xarray.DataArray, thresh: str = "1
 
     Parameters
     ----------
-    sic : xarray.DataArray
-      Sea ice concentration [0, 1].
-    area : xarray.DataArray
-      Grid cell area.
+    siconc : xarray.DataArray
+      Sea ice concentration (area fraction).
+    areacello : xarray.DataArray
+      Grid cell area (usually over the ocean).
     thresh : str
       Minimum sea ice concentration for a grid cell to contribute to the sea ice extent.
 
     Returns
     -------
     xarray.DataArray, [length]^2
-        Sea ice area
+      Sea ice area.
 
     Notes
     -----
@@ -1561,17 +1757,17 @@ def sea_ice_area(sic: xarray.DataArray, area: xarray.DataArray, thresh: str = "1
     <https://nsidc.org/arcticseaicenews/faq/#area_extent>`_
 
     """
-    t = convert_units_to(thresh, sic)
-    factor = convert_units_to("100 pct", sic)
-    out = xarray.dot(sic.where(sic >= t, 0), area) / factor
-    out.attrs["units"] = area.units
+    t = convert_units_to(thresh, siconc)
+    factor = convert_units_to("100 pct", siconc)
+    out = xarray.dot(siconc.where(siconc >= t, 0), areacello) / factor
+    out.attrs["units"] = areacello.units
     return out
 
 
-@declare_units(sic="[]", area="[area]", thresh="[]")
+@declare_units(siconc="[]", areacello="[area]", thresh="[]")
 def sea_ice_extent(
-    sic: xarray.DataArray, area: xarray.DataArray, thresh: str = "15 pct"
-):
+    siconc: xarray.DataArray, areacello: xarray.DataArray, thresh: str = "15 pct"
+) -> xarray.DataArray:
     """Total sea ice extent.
 
     Sea ice extent measures the *ice-covered* area, where a region is considered ice-covered if its sea ice
@@ -1579,9 +1775,9 @@ def sea_ice_extent(
 
     Parameters
     ----------
-    sic : xarray.DataArray
-      Sea ice concentration [0, 1].
-    area : xarray.DataArray
+    siconc : xarray.DataArray
+      Sea ice concentration (area fraction).
+    areacello : xarray.DataArray
       Grid cell area.
     thresh : str
       Minimum sea ice concentration for a grid cell to contribute to the sea ice extent.
@@ -1589,7 +1785,7 @@ def sea_ice_extent(
     Returns
     -------
     xarray.DataArray, [length]^2
-        Sea ice extent
+      Sea ice extent.
 
     Notes
     -----
@@ -1600,9 +1796,46 @@ def sea_ice_extent(
     `What is the difference between sea ice area and extent
     <https://nsidc.org/arcticseaicenews/faq/#area_extent>`_
     """
-    t = convert_units_to(thresh, sic)
-    out = xarray.dot(sic >= t, area)
-    out.attrs["units"] = area.units
+    t = convert_units_to(thresh, siconc)
+    out = xarray.dot(siconc >= t, areacello)
+    out.attrs["units"] = areacello.units
+    return out
+
+
+@declare_units(sfcWind="[speed]", thresh="[speed]")
+def windy_days(
+    sfcWind: xarray.DataArray, thresh: str = "10.8 m s-1", freq: str = "MS"
+) -> xarray.DataArray:
+    r"""Windy days.
+
+    The number of days with average near-surface wind speed above threshold.
+
+    Parameters
+    ----------
+    sfcWind : xarray.DataArray
+      Daily average near-surface wind speed.
+    thresh : str
+      Threshold average near-surface wind speed on which to base evaluation.
+    freq : str
+      Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray, [time]
+      Number of days with average near-surface wind speed above threshold.
+
+    Notes
+    -----
+    Let :math:`WS_{ij}` be the windspeed at day :math:`i` of period :math:`j`. Then
+    counted is the number of days where:
+
+    .. math::
+
+        WS_{ij} >= Threshold [m s-1]
+    """
+    thresh = convert_units_to(thresh, sfcWind)
+    out = threshold_count(sfcWind, ">=", thresh, freq)
+    out = to_agg_units(out, sfcWind, "count")
     return out
 
 
@@ -1611,7 +1844,7 @@ def tropical_nights(
     tasmin: xarray.DataArray,
     thresh: str = "20.0 degC",
     freq: str = "YS",
-):
+) -> xarray.DataArray:
     """Tropical nights.
 
     The number of days with minimum daily temperature above threshold.
@@ -1638,10 +1871,20 @@ def tropical_nights(
     .. math::
 
         TN_{ij} > Threshold [℃]
+
+    Warnings
+    --------
+    The `tropical_nights` indice is being deprecated in favour of `tn_days_above` with `thresh="20 degC"` by default.
+    The indicator reflects this change. This indice will be removed in a future version of xclim.
     """
-    thresh = convert_units_to(thresh, tasmin)
-    out = threshold_count(tasmin, ">", thresh, freq)
-    return to_agg_units(out, tasmin, "count")
+    warnings.warn(
+        "The `tropical_nights` indice is being deprecated in favour of `tn_days_above` with `thresh='20 degC'`. "
+        "This indice will be removed in `xclim>=0.28.0`. Please update your scripts accordingly.",
+        UserWarning,
+        stacklevel=3,
+    )
+
+    return tn_days_above(tasmin, thresh=thresh, freq=freq)
 
 
 @declare_units(tas="[temperature]", thresh="[temperature]", sum_thresh="K days")
@@ -1652,7 +1895,7 @@ def degree_days_exceedance_date(
     op: str = ">",
     after_date: DayOfYearStr = None,
     freq: str = "YS",
-):
+) -> xarray.DataArray:
     r"""Degree days exceedance date.
 
     Day of year when the sum of degree days exceeds a threshold. Degree days are
@@ -1678,7 +1921,7 @@ def degree_days_exceedance_date(
     Returns
     -------
     xarray.DataArray, [dimensionless]
-      Degree days exceedance date
+      Degree-days exceedance date.
 
     Notes
     -----
@@ -1706,6 +1949,8 @@ def degree_days_exceedance_date(
         c = thresh - tas
     elif op in [">", ">=", "gt", "ge"]:
         c = tas - thresh
+    else:
+        raise NotImplementedError(f"op: '{op}'.")
 
     def _exceedance_date(grp):
         strt_idx = rl.index_of_date(grp.time, after_date, max_idxs=1, default=0)
@@ -1726,9 +1971,10 @@ def degree_days_exceedance_date(
 
 
 @declare_units(snd="[length]", thresh="[length]")
-def winter_storm(snd: xarray.DataArray, thresh: str = "25 cm", freq="AS-JUL"):
-    """
-    Days with snowfall over threshold.
+def winter_storm(
+    snd: xarray.DataArray, thresh: str = "25 cm", freq: str = "AS-JUL"
+) -> xarray.DataArray:
+    """Days with snowfall over threshold.
 
     Number of days with snowfall accumulation greater or equal to threshold.
 
