@@ -6,27 +6,53 @@ Generic indices submodule
 
 Helper functions for common generic actions done in the computation of indices.
 """
-from typing import Union
+from typing import Optional, Union
 
 import numpy as np
+import xarray
 import xarray as xr
 
-from xclim.core.calendar import get_calendar
-from xclim.core.units import convert_units_to, pint2cfunits, str2pint, to_agg_units
+from xclim.core.calendar import (
+    convert_calendar,
+    days_in_year,
+    doy_to_days_since,
+    get_calendar,
+)
+from xclim.core.units import (
+    convert_units_to,
+    declare_units,
+    pint2cfunits,
+    str2pint,
+    to_agg_units,
+)
 
+from ..core.utils import DayOfYearStr
 from . import run_length as rl
 
-# __all__ = [
-#     "select_time",
-#     "select_resample_op",
-#     "doymax",
-#     "doymin",
-#     "default_freq",
-#     "threshold_count",
-#     "get_daily_events",
-#     "daily_downsampler",
-# ]
-
+__all__ = [
+    "aggregate_between_dates",
+    "compare",
+    "count_level_crossings",
+    "count_occurrences",
+    "daily_downsampler",
+    "day_lengths",
+    "default_freq",
+    "degree_days",
+    "diurnal_temperature_range",
+    "domain_count",
+    "doymax",
+    "doymin",
+    "get_daily_events",
+    "get_op",
+    "interday_diurnal_temperature_range",
+    "last_occurrence",
+    "select_resample_op",
+    "select_time",
+    "statistics",
+    "temperature_sum",
+    "threshold_count",
+    "thresholded_statistics",
+]
 
 binary_ops = {">": "gt", "<": "lt", ">=": "ge", "<=": "le", "==": "eq", "!=": "ne"}
 
@@ -88,7 +114,7 @@ def select_resample_op(da: xr.DataArray, op: str, freq: str = "YS", **indexer):
     return r.map(op)
 
 
-def doymax(da: xr.DataArray):
+def doymax(da: xr.DataArray) -> xr.DataArray:
     """Return the day of year of the maximum value."""
     i = da.argmax(dim="time")
     out = da.time.dt.dayofyear[i]
@@ -96,7 +122,7 @@ def doymax(da: xr.DataArray):
     return out
 
 
-def doymin(da: xr.DataArray):
+def doymin(da: xr.DataArray) -> xr.DataArray:
     """Return the day of year of the minimum value."""
     i = da.argmin(dim="time")
     out = da.time.dt.dayofyear[i]
@@ -104,7 +130,7 @@ def doymin(da: xr.DataArray):
     return out
 
 
-def default_freq(**indexer):
+def default_freq(**indexer) -> str:
     """Return the default frequency."""
     freq = "AS-JAN"
     if indexer:
@@ -128,7 +154,7 @@ def get_op(op: str):
         pass
     else:
         raise ValueError(f"Operation `{op}` not recognized.")
-    return xr.core.ops.get_op(op)
+    return xr.core.ops.get_op(op)  # noqa
 
 
 def compare(da: xr.DataArray, op: str, thresh: Union[float, int]) -> xr.DataArray:
@@ -191,8 +217,8 @@ def domain_count(da: xr.DataArray, low: float, high: float, freq: str) -> xr.Dat
     high : float
       Maximum threshold value.
     freq : str
-      Resampling frequency defining the periods
-      defined in http://pandas.pydata.org/pandas-docs/stable/timeseries.html#resampling.
+      Resampling frequency defining the periods defined in
+      https://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html#resampling.
 
     Returns
     -------
@@ -302,7 +328,7 @@ def daily_downsampler(da: xr.DataArray, freq: str = "YS") -> xr.DataArray:
 
 def count_level_crossings(
     low_data: xr.DataArray, high_data: xr.DataArray, threshold: str, freq: str
-):
+) -> xr.DataArray:
     """Calculate the number of times low_data is below threshold while high_data is above threshold.
 
     First, the threshold is transformed to the same standard_name and units as the input data,
@@ -318,6 +344,10 @@ def count_level_crossings(
       Quantity.
     freq: str
       Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray
     """
     # Convert units to low_data
     high_data = convert_units_to(high_data, low_data)
@@ -330,7 +360,9 @@ def count_level_crossings(
     return to_agg_units(out, low_data, "count", dim="time")
 
 
-def count_occurrences(data: xr.DataArray, threshold: str, condition: str, freq: str):
+def count_occurrences(
+    data: xr.DataArray, threshold: str, condition: str, freq: str
+) -> xr.DataArray:
     """Calculate the number of times some condition is met.
 
     First, the threshold is transformed to the same standard_name and units as the input data.
@@ -347,6 +379,10 @@ def count_occurrences(data: xr.DataArray, threshold: str, condition: str, freq:
       Operator.
     freq: str
       Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray
     """
     threshold = convert_units_to(threshold, data)
 
@@ -358,7 +394,7 @@ def count_occurrences(data: xr.DataArray, threshold: str, condition: str, freq:
 
 def diurnal_temperature_range(
     low_data: xr.DataArray, high_data: xr.DataArray, freq: str
-):
+) -> xr.DataArray:
     """Calculate the average diurnal temperature range.
 
     Parameters
@@ -369,6 +405,10 @@ def diurnal_temperature_range(
       Highest daily temperature (tasmax).
     freq: str
       Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray
     """
     high_data = convert_units_to(high_data, low_data)
 
@@ -380,7 +420,9 @@ def diurnal_temperature_range(
     return out
 
 
-def first_occurence(data: xr.DataArray, threshold: str, condition: str, freq: str):
+def first_occurrence(
+    data: xr.DataArray, threshold: str, condition: str, freq: str
+) -> xr.DataArray:
     """Calculate the first time some condition is met.
 
     First, the threshold is transformed to the same standard_name and units as the input data.
@@ -396,6 +438,10 @@ def first_occurence(data: xr.DataArray, threshold: str, condition: str, freq: st
       Operator
     freq : str
       Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray
     """
     threshold = convert_units_to(threshold, data)
 
@@ -411,7 +457,9 @@ def first_occurence(data: xr.DataArray, threshold: str, condition: str, freq: st
     return out
 
 
-def last_occurence(data: xr.DataArray, threshold: str, condition: str, freq: str):
+def last_occurrence(
+    data: xr.DataArray, threshold: str, condition: str, freq: str
+) -> xr.DataArray:
     """Calculate the last time some condition is met.
 
     First, the threshold is transformed to the same standard_name and units as the input data.
@@ -427,6 +475,10 @@ def last_occurence(data: xr.DataArray, threshold: str, condition: str, freq: str
       Operator
     freq : str
       Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray
     """
     threshold = convert_units_to(threshold, data)
 
@@ -444,7 +496,7 @@ def last_occurence(data: xr.DataArray, threshold: str, condition: str, freq: str
 
 def spell_length(
     data: xr.DataArray, threshold: str, condition: str, reducer: str, freq: str
-):
+) -> xr.DataArray:
     """Calculate statistics on lengths of spells.
 
     First, the threshold is transformed to the same standard_name and units as the input data.
@@ -462,6 +514,10 @@ def spell_length(
       Reducer.
     freq : str
       Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray
     """
     threshold = convert_units_to(threshold, data)
 
@@ -475,7 +531,7 @@ def spell_length(
     return to_agg_units(out, data, "count")
 
 
-def statistics(data: xr.DataArray, reducer: str, freq: str):
+def statistics(data: xr.DataArray, reducer: str, freq: str) -> xr.DataArray:
     """Calculate a simple statistic of the data.
 
     Parameters
@@ -485,6 +541,10 @@ def statistics(data: xr.DataArray, reducer: str, freq: str):
       Reducer.
     freq : str
       Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray
     """
     out = getattr(data.resample(time=freq), reducer)()
     out.attrs["units"] = data.attrs["units"]
@@ -493,7 +553,7 @@ def statistics(data: xr.DataArray, reducer: str, freq: str):
 
 def thresholded_statistics(
     data: xr.DataArray, threshold: str, condition: str, reducer: str, freq: str
-):
+) -> xr.DataArray:
     """Calculate a simple statistic of the data for which some condition is met.
 
     First, the threshold is transformed to the same standard_name and units as the input data.
@@ -511,6 +571,10 @@ def thresholded_statistics(
       Reducer.
     freq : str
       Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray
     """
     threshold = convert_units_to(threshold, data)
 
@@ -521,7 +585,9 @@ def thresholded_statistics(
     return out
 
 
-def temperature_sum(data: xr.DataArray, threshold: str, condition: str, freq: str):
+def temperature_sum(
+    data: xr.DataArray, threshold: str, condition: str, freq: str
+) -> xr.DataArray:
     """Calculate the temperature sum above/below a threshold.
 
     First, the threshold is transformed to the same standard_name and units as the input data.
@@ -538,6 +604,10 @@ def temperature_sum(data: xr.DataArray, threshold: str, condition: str, freq: st
       Operator
     freq : str
       Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray
     """
     threshold = convert_units_to(threshold, data)
 
@@ -551,7 +621,7 @@ def temperature_sum(data: xr.DataArray, threshold: str, condition: str, freq: st
 
 def interday_diurnal_temperature_range(
     low_data: xr.DataArray, high_data: xr.DataArray, freq: str
-):
+) -> xr.DataArray:
     """Calculate the average absolute day-to-day difference in diurnal temperature range.
 
     Parameters
@@ -562,6 +632,10 @@ def interday_diurnal_temperature_range(
       Highest daily temperature (tasmax).
     freq: str
       Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray
     """
     high_data = convert_units_to(high_data, low_data)
 
@@ -575,7 +649,7 @@ def interday_diurnal_temperature_range(
 
 def extreme_temperature_range(
     low_data: xr.DataArray, high_data: xr.DataArray, freq: str
-):
+) -> xr.DataArray:
     """Calculate the extreme temperature range as the maximum of daily maximum temperature minus the minimum of daily minimum temperature.
 
     Parameters
@@ -586,6 +660,10 @@ def extreme_temperature_range(
       Highest daily temperature (tasmax).
     freq: str
       Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray
     """
     high_data = convert_units_to(high_data, low_data)
 
@@ -594,3 +672,219 @@ def extreme_temperature_range(
     u = str2pint(low_data.units)
     out.attrs["units"] = pint2cfunits(u - u)
     return out
+
+
+def aggregate_between_dates(
+    data: xr.DataArray,
+    start: Union[xr.DataArray, DayOfYearStr],
+    end: Union[xr.DataArray, DayOfYearStr],
+    op: str = "sum",
+    freq: Optional[str] = None,
+):
+    """Aggregate the data over a period between start and end dates and apply the operator on the aggregated data.
+
+    Parameters
+    ----------
+    data : xr.DataArray
+      Data to aggregate between start and end dates.
+    start : xr.DataArray or DayOfYearStr
+      Start dates (as day-of-year) for the aggregation periods.
+    end : xr.DataArray or DayOfYearStr
+      End (as day-of-year) dates for the aggregation periods.
+    op : {'min', 'max', 'sum', 'mean', 'std'}
+      Operator.
+    freq : str
+      Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray, [dimensionless]
+      Aggregated data between the start and end dates. If the end date is before the start date, returns np.nan.
+      If there is no start and/or end date, returns np.nan.
+    """
+
+    def _get_days(_bound, _group, _base_time):
+        """Get bound in number of days since base_time. Bound can be a days_since array or a DayOfYearStr."""
+        if isinstance(_bound, str):
+            b_i = rl.index_of_date(_group.time, _bound, max_idxs=1)  # noqa
+            if not len(b_i):
+                return None
+            return (_group.time.isel(time=b_i[0]) - _group.time.isel(time=0)).dt.days
+        if _base_time in _bound.time:
+            return _bound.sel(time=_base_time)
+        return None
+
+    if freq is None:
+        frequencies = []
+        for i, bound in enumerate([start, end], start=1):
+            try:
+                frequencies.append(xr.infer_freq(bound.time))
+            except AttributeError:
+                frequencies.append(None)
+
+        good_freq = set(frequencies) - {None}
+
+        if len(good_freq) != 1:
+            raise ValueError(
+                f"Non-inferrable resampling frequency or inconsistent frequencies. Got start, end = {frequencies}."
+                " Please consider providing `freq` manually."
+            )
+        freq = good_freq.pop()
+
+    cal = get_calendar(data, dim="time")
+
+    if not isinstance(start, str):
+        start = convert_calendar(start, cal)
+        start.attrs["calendar"] = cal
+        start = doy_to_days_since(start)
+    if not isinstance(end, str):
+        end = convert_calendar(end, cal)
+        end.attrs["calendar"] = cal
+        end = doy_to_days_since(end)
+
+    out = list()
+    for base_time, indexes in data.resample(time=freq).groups.items():
+        # get group slice
+        group = data.isel(time=indexes)
+
+        start_d = _get_days(start, group, base_time)
+        end_d = _get_days(end, group, base_time)
+
+        # convert bounds for this group
+        if start_d is not None and end_d is not None:
+
+            days = (group.time - base_time).dt.days
+            days[days < 0] = np.nan
+
+            masked = group.where((days >= start_d) & (days <= end_d - 1))
+            res = getattr(masked, op)(dim="time", skipna=True)
+            res = xr.where(
+                ((start_d > end_d) | (start_d.isnull()) | (end_d.isnull())), np.nan, res
+            )
+            # Re-add the time dimension with the period's base time.
+            res = res.expand_dims(time=[base_time])
+            out.append(res)
+        else:
+            # Get an array with the good shape, put nans and add the new time.
+            res = (group.isel(time=0) * np.nan).expand_dims(time=[base_time])
+            out.append(res)
+            continue
+
+    out = xr.concat(out, dim="time")
+    return out
+
+
+@declare_units(tas="[temperature]")
+def degree_days(tas: xr.DataArray, thresh: str, condition: str) -> xr.DataArray:
+    """Calculate the degree days below/above the temperature threshold.
+
+    Parameters
+    ----------
+    tas : xr.DataArray
+      Mean daily temperature.
+    thresh : str
+      The temperature threshold.
+    condition : {"<", ">"}
+      Operator.
+
+    Returns
+    -------
+    xarray.DataArray
+    """
+    thresh = convert_units_to(thresh, tas)
+
+    if "<" in condition:
+        out = (thresh - tas).clip(0)
+    elif ">" in condition:
+        out = (tas - thresh).clip(0)
+    else:
+        raise NotImplementedError(f"Condition not supported: '{condition}'.")
+
+    out = to_agg_units(out, tas, op="delta_prod")
+    return out
+
+
+def day_lengths(
+    dates: xr.DataArray,
+    lat: xr.DataArray,
+    obliquity: float = -0.4091,
+    summer_solstice: DayOfYearStr = "06-21",
+    start_date: Optional[Union[xarray.DataArray, DayOfYearStr]] = None,
+    end_date: Optional[Union[xarray.DataArray, DayOfYearStr]] = None,
+    freq: str = "YS",
+) -> xr.DataArray:
+    r"""Day-lengths according to latitude, obliquity, and day of year.
+
+    Parameters
+    ----------
+    dates: xr.DataArray
+    lat: xarray.DataArray
+      Latitude coordinate.
+    obliquity: float
+      Obliquity of the elliptic (radians). Default: -0.4091.
+    summer_solstice: DayOfYearStr
+      Date of summer solstice in northern hemisphere. Used for approximating solar julian dates.
+    start_date: xarray.DataArray or DayOfYearStr, optional
+    end_date: xarray.DataArray or DayOfYearStr, optional
+    freq : str
+      Resampling frequency.
+
+    Returns
+    -------
+    xarray.DataArray
+      If start and end date provided, returns total sum of daylight-hour between dates at provided frequency.
+      If no start and end date provided, returns day-length in hours per individual day.
+
+    Notes
+    -----
+    Daylight-hours are dependent on latitude, :math:`lat`, the Julian day (solar day) from the summer solstice in the
+    Northern hemisphere, :math:`Jday`, and the axial tilt :math:`Axis`, therefore day-length at any latitude for a given
+    date on Earth, :math:`dayLength_{lat_{Jday}}`, for a given year in days, :math:`Year`, can be approximated as
+    follows:
+
+    .. math::
+        dayLength_{lat_{Jday}} = f({lat}, {Jday}) = \frac{\arccos(1-m_{lat_{Jday}})}{\pi} * 24
+
+    Where:
+
+    .. math::
+        m_{lat_{Jday}} = f({lat}, {Jday}) = 1 - \tan({Lat}) * \tan \left({Axis}*\cos\left[\frac{2*\pi*{Jday}}{||{Year}||} \right] \right)
+
+    The total sum of daylight hours for a given period between two days (:math:`{Jday} = 0` -> :math:`N`) within a solar
+    year then is:
+
+    .. math::
+        \sum({SeasonDayLength_{lat}}) = \sum_{Jday=1}^{N} dayLength_{lat_{Jday}}
+
+    References
+    ----------
+    Modified day-length equations for Huglin heliothermal index published in Hall, A., & Jones, G. V. (2010). Spatial
+    analysis of climate in winegrape-growing regions in Australia. Australian Journal of Grape and Wine Research, 16(3),
+    389‑404. https://doi.org/10.1111/j.1755-0238.2010.00100.x
+
+    Examples available from Glarner, 2006 (http://www.gandraxa.com/length_of_day.xml).
+    """
+    cal = get_calendar(dates)
+
+    year_length = dates.time.copy(
+        data=[days_in_year(x, calendar=cal) for x in dates.time.dt.year]
+    )
+
+    julian_date_from_solstice = dates.time.copy(
+        data=doy_to_days_since(
+            dates.time.dt.dayofyear, start=summer_solstice, calendar=cal
+        )
+    )
+
+    m_lat_dayofyear = 1 - np.tan(np.radians(lat)) * np.tan(
+        obliquity * (np.cos((2 * np.pi * julian_date_from_solstice) / year_length))
+    )
+
+    day_length_hours = (np.arccos(1 - m_lat_dayofyear) / np.pi) * 24
+
+    if start_date and end_date:
+        return aggregate_between_dates(
+            day_length_hours, start=start_date, end=end_date, op="sum", freq=freq
+        )
+    else:
+        return day_length_hours
