@@ -15,46 +15,87 @@ import numpy as np
 import xarray as xr
 from dask import array as dsk
 
-from xclim.core.utils import DateStr, DayOfYearStr
+from xclim.core.options import OPTIONS, RUN_LENGTH_UFUNC
+from xclim.core.utils import DateStr, DayOfYearStr, uses_dask
 
 npts_opt = 9000
+"""
+Arrays with less than this number of data points per slice will trigger
+the use of the ufunc version of run lengths algorithms.
+"""
 
 
-def get_npts(da: xr.DataArray) -> int:
-    """Return the number of gridpoints in a DataArray.
+def use_ufunc(
+    ufunc_1dim: Union[bool, str],
+    da: xr.DataArray,
+    dim: str = "time",
+    index: str = "first",
+) -> bool:
+    """Return whether the ufunc version of run length algorithms should be used with this DataArray or not.
+
+    If ufunc_1dim is 'from_context', the parameter is read from xclim's global (or context) options.
+    If it is 'auto', this returns False for dask-backed array and for arrays with more than :py:const:`npts_opt`
+    points per slice along `dim`.
 
     Parameters
     ----------
-    da : xarray.DataArray
-      N-dimensional input array
+    ufunc_1dim: {'from_context', 'auto', True, False}
+    da : xr.DataArray
+      Input array.
+    dim: str
+      The dimension along which to find runs.
+    index: {'first', 'last'}
+      If 'first', the run length is indexed with the first element in the run.
+      If 'last', with the last element in the run.
 
     Returns
     -------
-    int
-      Product of input DataArray coordinate sizes excluding the dimension 'time'
+    bool
+      If ufunc_1dim is "auto", returns True if the array is on dask or too large.
+      Otherwise, returns ufunc_1dim.
     """
-    return da.size // da.time.size
+
+    if ufunc_1dim == "from_context":
+        ufunc_1dim = OPTIONS[RUN_LENGTH_UFUNC]
+
+    if ufunc_1dim == "auto":
+        ufunc_1dim = not uses_dask(da) and (da.size // da[dim].size) < npts_opt
+
+    return index == "first" and ufunc_1dim
 
 
 def rle(
-    da: xr.DataArray, dim: str = "time", max_chunk: int = 1_000_000
+    da: xr.DataArray,
+    dim: str = "time",
+    max_chunk: int = 1_000_000,
+    index: str = "first",
 ) -> xr.DataArray:
     """Generate basic run length function.
 
     Parameters
     ----------
     da : xr.DataArray
+      Input array.
     dim : str
+      Dimension name.
     max_chunk : int
+      Maximum chunk size.
+    index: {'first', 'last'}
+      If 'first', the run length is indexed with the first element in the run.
+      If 'last', with the last element in the run.
 
     Returns
     -------
     xr.DataArray
-      Values are 0 where da is False (out of runs),
-      are N on the first day of a run, where N is the length of that run,
-      and are NaN on the other days of the runs.
+      Values are 0 where da is False (out of runs).
     """
     use_dask = isinstance(da.data, dsk.Array)
+
+    # Ensure boolean
+    da = da.astype(bool)
+    if index == "last":
+        da = da.reindex({dim: da[dim][::-1]})
+
     n = len(da[dim])
     # Need to chunk here to ensure the broadcasting is not made in memory
     i = xr.DataArray(np.arange(da[dim].size), dims=dim)
@@ -76,7 +117,7 @@ def rle(
     b = xr.concat([start1, b, end1], dim)
 
     # Ensure bfill operates on entire (unchunked) time dimension
-    # Determine appropraite chunk size for other dims - do not exceed 'max_chunk' total size per chunk (default 1000000)
+    # Determine appropriate chunk size for other dims - do not exceed 'max_chunk' total size per chunk (default 1000000)
     ndims = len(b.shape)
     if use_dask:
         chunk_dim = b[dim].size
@@ -96,33 +137,44 @@ def rle(
 
     # back fill nans with first position after
     z = b.bfill(dim=dim)
+
     # calculate lengths
     d = z.diff(dim=dim) - 1
     d = d.where(d >= 0)
     d = d.isel({dim: slice(None, -1)}).where(da, 0)
+    if index == "last":
+        d = d.reindex({dim: d[dim][::-1]})
     return d
 
 
 def rle_statistics(
     da: xr.DataArray,
     reducer: str = "max",
+    window: int = 1,
     dim: str = "time",
-    ufunc_1dim: Union[str, bool] = "auto",
+    ufunc_1dim: Union[str, bool] = "from_context",
+    index: str = "first",
 ) -> xr.DataArray:
     """Return the length of consecutive run of True values, according to a reducing operator.
 
     Parameters
     ----------
     da : xr.DataArray
-      N-dimensional array (boolean)
+      N-dimensional array (boolean).
     reducer: str
       Name of the reducing function.
+    window : int
+      Minimal length of consecutive runs to be included in the statistics.
     dim : str
       Dimension along which to calculate consecutive run; Default: 'time'.
     ufunc_1dim : Union[str, bool]
       Use the 1d 'ufunc' version of this function : default (auto) will attempt to select optimal
       usage based on number of data points.  Using 1D_ufunc=True is typically more efficient
       for DataArray with a small number of grid points.
+    index: {'first', 'last'}
+      If 'first', the run length is indexed with the first element in the run.
+      If 'last', with the last element in the run.
+
 
     Returns
     -------
@@ -130,22 +182,23 @@ def rle_statistics(
       Length of runs of True values along dimension, according to the reducing function (float)
       If there are no runs (but the data is valid), returns 0.
     """
-    if ufunc_1dim == "auto":
-        npts = get_npts(da)
-        ufunc_1dim = npts <= npts_opt
+    ufunc_1dim = use_ufunc(ufunc_1dim, da, dim=dim, index=index)
 
     if ufunc_1dim:
-        rl_stat = statistics_run_ufunc(da, reducer)
+        rl_stat = statistics_run_ufunc(da, reducer, window, dim)
     else:
-        d = rle(da, dim=dim)
-        rl_stat = getattr(d.where(d > 0), reducer)(dim=dim)
-        rl_stat = xr.where((d.isnull() | (d == 0)).all(dim=dim), 0, rl_stat)
+        d = rle(da, dim=dim, index=index)
+        rl_stat = getattr(d.where(d >= window), reducer)(dim=dim)
+        rl_stat = xr.where((d.isnull() | (d < window)).all(dim=dim), 0, rl_stat)
 
     return rl_stat
 
 
 def longest_run(
-    da: xr.DataArray, dim: str = "time", ufunc_1dim: Union[str, bool] = "auto"
+    da: xr.DataArray,
+    dim: str = "time",
+    ufunc_1dim: Union[str, bool] = "from_context",
+    index: str = "first",
 ) -> xr.DataArray:
     """Return the length of the longest consecutive run of True values.
 
@@ -159,13 +212,19 @@ def longest_run(
       Use the 1d 'ufunc' version of this function : default (auto) will attempt to select optimal
       usage based on number of data points.  Using 1D_ufunc=True is typically more efficient
       for DataArray with a small number of grid points.
+    index: {'first', 'last'}
+      If 'first', the run length is indexed with the first element in the run.
+      If 'last', with the last element in the run.
+
 
     Returns
     -------
     xr.DataArray
       Length of longest run of True values along dimension (int).
     """
-    return rle_statistics(da, reducer="max", dim=dim, ufunc_1dim=ufunc_1dim)
+    return rle_statistics(
+        da, reducer="max", dim=dim, ufunc_1dim=ufunc_1dim, index=index
+    )
 
 
 def windowed_run_events(
@@ -173,13 +232,14 @@ def windowed_run_events(
     window: int,
     dim: str = "time",
     ufunc_1dim: Union[str, bool] = "auto",
+    index: str = "first",
 ) -> xr.DataArray:
     """Return the number of runs of a minimum length.
 
     Parameters
     ----------
     da: xr.DataArray
-      Input N-dimensional DataArray (boolean)
+      Input N-dimensional DataArray (boolean).
     window : int
       Minimum run length.
     dim : str
@@ -188,20 +248,22 @@ def windowed_run_events(
       Use the 1d 'ufunc' version of this function : default (auto) will attempt to select optimal
       usage based on number of data points.  Using 1D_ufunc=True is typically more efficient
       for dataarray with a small number of gridpoints.
+    index: {'first', 'last'}
+      If 'first', the run length is indexed with the first element in the run.
+      If 'last', with the last element in the run.
+
 
     Returns
     -------
     xr.DataArray
       Number of distinct runs of a minimum length (int).
     """
-    if ufunc_1dim == "auto":
-        npts = get_npts(da)
-        ufunc_1dim = npts <= npts_opt
+    ufunc_1dim = use_ufunc(ufunc_1dim, da, dim=dim, index=index)
 
     if ufunc_1dim:
-        out = windowed_run_events_ufunc(da, window)
+        out = windowed_run_events_ufunc(da, window, dim)
     else:
-        d = rle(da, dim=dim)
+        d = rle(da, dim=dim, index=index)
         out = (d >= window).sum(dim=dim)
     return out
 
@@ -210,7 +272,8 @@ def windowed_run_count(
     da: xr.DataArray,
     window: int,
     dim: str = "time",
-    ufunc_1dim: Union[str, bool] = "auto",
+    ufunc_1dim: Union[str, bool] = "from_context",
+    index: str = "first",
 ) -> xr.DataArray:
     """Return the number of consecutive true values in array for runs at least as long as given duration.
 
@@ -226,20 +289,21 @@ def windowed_run_count(
       Use the 1d 'ufunc' version of this function : default (auto) will attempt to select optimal
       usage based on number of data points. Using 1D_ufunc=True is typically more efficient
       for dataarray with a small number of gridpoints.
+    index: {'first', 'last'}
+      If 'first', the run length is indexed with the first element in the run.
+      If 'last', with the last element in the run.
 
     Returns
     -------
     xr.DataArray
-      Total number of true values part of a consecutive runs of at least `window` long.
+      Total number of `True` values part of a consecutive runs of at least `window` long.
     """
-    if ufunc_1dim == "auto":
-        npts = get_npts(da)
-        ufunc_1dim = npts <= npts_opt
+    ufunc_1dim = use_ufunc(ufunc_1dim, da, dim=dim, index=index)
 
     if ufunc_1dim:
-        out = windowed_run_count_ufunc(da, window)
+        out = windowed_run_count_ufunc(da, window, dim)
     else:
-        d = rle(da, dim=dim)
+        d = rle(da, dim=dim, index=index)
         out = d.where(d >= window, 0).sum(dim=dim)
     return out
 
@@ -249,14 +313,14 @@ def first_run(
     window: int,
     dim: str = "time",
     coord: Optional[Union[str, bool]] = False,
-    ufunc_1dim: Union[str, bool] = "auto",
+    ufunc_1dim: Union[str, bool] = "from_context",
 ) -> xr.DataArray:
     """Return the index of the first item of the first run of at least a given length.
 
     Parameters
     ----------
     da : xr.DataArray
-      Input N-dimensional DataArray (boolean)
+      Input N-dimensional DataArray (boolean).
     window : int
       Minimum duration of consecutive run to accumulate values.
     dim : str
@@ -273,17 +337,12 @@ def first_run(
     Returns
     -------
     xr.DataArray
-      Index (or coordinate if `coord` is not False) of first item in first valid run. Returns np.nan if there are no valid run.
+      Index (or coordinate if `coord` is not False) of first item in first valid run.
+      Returns np.nan if there are no valid runs.
     """
-    if ufunc_1dim == "auto":
-        if isinstance(da.data, dsk.Array) and len(da.chunks[da.dims.index(dim)]) > 1:
-            ufunc_1dim = False
-        else:
-            npts = get_npts(da)
-            ufunc_1dim = npts <= npts_opt
+    ufunc_1dim = use_ufunc(ufunc_1dim, da, dim=dim)
 
     da = da.fillna(0)  # We expect a boolean array, but there could be NaNs nonetheless
-
     if ufunc_1dim:
         out = first_run_ufunc(x=da, window=window, dim=dim)
 
@@ -293,7 +352,7 @@ def first_run(
         ind = xr.broadcast(i, da)[0].transpose(*da.dims)
         if isinstance(da.data, dsk.Array):
             ind = ind.chunk(da.chunks)
-        wind_sum = da.rolling(time=window).sum(skipna=False)
+        wind_sum = da.rolling({dim: window}).sum(skipna=False)
         out = ind.where(wind_sum >= window).min(dim=dim) - (window - 1)
         # remove window - 1 as rolling result index is last element of the moving window
 
@@ -315,14 +374,14 @@ def last_run(
     window: int,
     dim: str = "time",
     coord: Optional[Union[str, bool]] = False,
-    ufunc_1dim: Union[str, bool] = "auto",
+    ufunc_1dim: Union[str, bool] = "from_context",
 ) -> xr.DataArray:
     """Return the index of the last item of the last run of at least a given length.
 
     Parameters
     ----------
     da : xr.DataArray
-      Input N-dimensional DataArray (boolean)
+      Input N-dimensional DataArray (boolean).
     window : int
       Minimum duration of consecutive run to accumulate values.
     dim : str
@@ -339,7 +398,8 @@ def last_run(
     Returns
     -------
     xr.DataArray
-      Index (or coordinate if `coord` is not False) of last item in last valid run. Returns np.nan if there are no valid run.
+      Index (or coordinate if `coord` is not False) of last item in last valid run.
+      Returns np.nan if there are no valid runs.
     """
     reversed_da = da.sortby(dim, ascending=False)
     out = first_run(
@@ -372,7 +432,7 @@ def run_bounds(
     xr.DataArray
       With ``dim`` reduced to "events" and "bounds". The events dim is as long as needed, padded with NaN or NaT.
     """
-    if isinstance(mask.data, dsk.Array):
+    if uses_dask(mask):
         raise NotImplementedError(
             "Dask arrays not supported as we can't know the final event number before computing."
         )
@@ -421,7 +481,7 @@ def keep_longest_run(da: xr.DataArray, dim: str = "time") -> xr.DataArray:
     Parameters
     ----------
     da : xr.DataArray
-      Boolean array
+      Boolean array.
     dim : str
       Dimension along which to check for the longest run.
 
@@ -458,7 +518,7 @@ def season(
     Parameters
     ----------
     da : xr.DataArray
-      Input N-dimensional DataArray (boolean)
+      Input N-dimensional DataArray (boolean).
     window : int
       Minimum duration of consecutive values to start and end the season.
     date: DayOfYearStr, optional
@@ -575,7 +635,7 @@ def season_length(
     Parameters
     ----------
     da : xr.DataArray
-      Input N-dimensional DataArray (boolean)
+      Input N-dimensional DataArray (boolean).
     window : int
       Minimum duration of consecutive values to start and end the season.
     date: DayOfYearStr, optional
@@ -616,7 +676,7 @@ def run_end_after_date(
     Parameters
     ----------
     da : xr.DataArray
-      Input N-dimensional DataArray (boolean)
+      Input N-dimensional DataArray (boolean).
     window : int
       Minimum duration of consecutive run to accumulate values.
     date : str
@@ -631,7 +691,8 @@ def run_end_after_date(
     Returns
     -------
     xr.DataArray
-      Index (or coordinate if `coord` is not False) of last item in last valid run. Returns np.nan if there are no valid run.
+      Index (or coordinate if `coord` is not False) of last item in last valid run.
+      Returns np.nan if there are no valid runs.
     """
     mid_idx = index_of_date(da[dim], date, max_idxs=1, default=0)
     if mid_idx.size == 0:  # The date is not within the group. Happens at boundaries.
@@ -668,7 +729,7 @@ def first_run_after_date(
     Parameters
     ----------
     da : xr.DataArray
-      Input N-dimensional DataArray (boolean)
+      Input N-dimensional DataArray (boolean).
     window : int
       Minimum duration of consecutive run to accumulate values.
     date : DayOfYearStr
@@ -683,14 +744,15 @@ def first_run_after_date(
     Returns
     -------
     xr.DataArray
-      Index (or coordinate if `coord` is not False) of first item in the first valid run. Returns np.nan if there are no valid run.
+      Index (or coordinate if `coord` is not False) of first item in the first valid run.
+      Returns np.nan if there are no valid runs.
     """
-    mid_idx = index_of_date(da.time, date, max_idxs=1, default=0)
+    mid_idx = index_of_date(da[dim], date, max_idxs=1, default=0)
     if mid_idx.size == 0:  # The date is not within the group. Happens at boundaries.
-        return xr.full_like(da.isel(time=0), np.nan, float).drop_vars("time")
+        return xr.full_like(da.isel({dim: 0}), np.nan, float).drop_vars(dim)
 
     return first_run(
-        da.where(da.time >= da.time[mid_idx][0]),
+        da.where(da[dim] >= da[dim][mid_idx][0]),
         window=window,
         dim=dim,
         coord=coord,
@@ -709,7 +771,7 @@ def last_run_before_date(
     Parameters
     ----------
     da : xr.DataArray
-      Input N-dimensional DataArray (boolean)
+      Input N-dimensional DataArray (boolean).
     window : int
       Minimum duration of consecutive run to accumulate values.
     date : DayOfYearStr
@@ -724,14 +786,15 @@ def last_run_before_date(
     Returns
     -------
     xr.DataArray
-      Index (or coordinate if `coord` is not False) of last item in last valid run. Returns np.nan if there are no valid run.
+      Index (or coordinate if `coord` is not False) of last item in last valid run.
+      Returns np.nan if there are no valid runs.
     """
-    mid_idx = index_of_date(da.time, date, default=-1)
+    mid_idx = index_of_date(da[dim], date, default=-1)
 
     if mid_idx.size == 0:  # The date is not within the group. Happens at boundaries.
-        return xr.full_like(da.isel(time=0), np.nan, float).drop_vars("time")
+        return xr.full_like(da.isel({dim: 0}), np.nan, float).drop_vars(dim)
 
-    run = da.where(da.time <= da.time[mid_idx][0])
+    run = da.where(da[dim] <= da[dim][mid_idx][0])
     return last_run(run, window=window, dim=dim, coord=coord)
 
 
@@ -748,11 +811,11 @@ def rle_1d(
     Returns
     -------
     values : np.array
-      The values taken by arr over each run
+      The values taken by arr over each run.
     run lengths : np.array
-      The length of each run
+      The length of each run.
     start position : np.array
-      The starting index of each run
+      The starting index of each run.
 
     Examples
     --------
@@ -783,14 +846,15 @@ def first_run_1d(arr: Sequence[Union[int, float]], window: int) -> int:
     Parameters
     ----------
     arr : Sequence[Union[int, float]]
-      Input array
+      Input array.
     window : int
       Minimum duration of consecutive run to accumulate values.
 
     Returns
     -------
     int
-      Index of first item in first valid run. Returns np.nan if there are no valid run.
+      Index of first item in first valid run.
+      Returns np.nan if there are no valid runs.
     """
     v, rl, pos = rle_1d(arr)
     ind = np.where(v * rl >= window, pos, np.inf).min()
@@ -800,13 +864,17 @@ def first_run_1d(arr: Sequence[Union[int, float]], window: int) -> int:
     return ind
 
 
-def statistics_run_1d(arr: Sequence[bool], reducer: str) -> int:
+def statistics_run_1d(arr: Sequence[bool], reducer: str, window: int = 1) -> int:
     """Return statistics on lengths of run of identical values.
 
     Parameters
     ----------
     arr : Sequence[bool]
       Input array (bool)
+    reducer : {'mean', 'sum', 'min', 'max', 'std'}
+      Reducing function name.
+    window: int
+      Minimal length of runs to be included in the statistics
 
     Returns
     -------
@@ -814,10 +882,10 @@ def statistics_run_1d(arr: Sequence[bool], reducer: str) -> int:
       Statistics on length of runs.
     """
     v, rl = rle_1d(arr)[:2]
-    if not np.any(v):
+    if not np.any(v) or np.all(v * rl < window):
         return 0
     func = getattr(np, f"nan{reducer}")
-    return func(np.where(v, rl, np.NaN))
+    return func(np.where(v * rl >= window, rl, np.NaN))
 
 
 def windowed_run_count_1d(arr: Sequence[bool], window: int) -> int:
@@ -826,7 +894,7 @@ def windowed_run_count_1d(arr: Sequence[bool], window: int) -> int:
     Parameters
     ----------
     arr : Sequence[bool]
-      Input array (bool)
+      Input array (bool).
     window : int
       Minimum duration of consecutive run to accumulate values.
 
@@ -839,44 +907,46 @@ def windowed_run_count_1d(arr: Sequence[bool], window: int) -> int:
     return np.where(v * rl >= window, rl, 0).sum()
 
 
-def windowed_run_events_1d(arr: Sequence[bool], window: int):
+def windowed_run_events_1d(arr: Sequence[bool], window: int) -> xr.DataArray:
     """Return the number of runs of a minimum length.
 
     Parameters
     ----------
     arr : Sequence[bool]
-      Input array (bool)
+      Input array (bool).
     window : int
       Minimum run length.
 
     Returns
     -------
-    func
+    xr.DataArray
       Number of distinct runs of a minimum length.
     """
     v, rl, pos = rle_1d(arr)
     return (v * rl >= window).sum()
 
 
-def windowed_run_count_ufunc(x: Sequence[bool], window: int) -> xr.DataArray:
+def windowed_run_count_ufunc(
+    x: Union[xr.DataArray, Sequence[bool]], window: int, dim: str
+) -> xr.DataArray:
     """Dask-parallel version of windowed_run_count_1d, ie: the number of consecutive true values in array for runs at least as long as given duration.
 
     Parameters
     ----------
     x : Sequence[bool]
-      Input array (bool)
+      Input array (bool).
     window : int
       Minimum duration of consecutive run to accumulate values.
 
     Returns
     -------
-    out : func
+    xr.DataArray
       A function operating along the time dimension of a dask-array.
     """
     return xr.apply_ufunc(
         windowed_run_count_1d,
         x,
-        input_core_dims=[["time"]],
+        input_core_dims=[[dim]],
         vectorize=True,
         dask="parallelized",
         output_dtypes=[int],
@@ -885,25 +955,27 @@ def windowed_run_count_ufunc(x: Sequence[bool], window: int) -> xr.DataArray:
     )
 
 
-def windowed_run_events_ufunc(x: Sequence[bool], window: int) -> xr.DataArray:
+def windowed_run_events_ufunc(
+    x: Union[xr.DataArray, Sequence[bool]], window: int, dim: str
+) -> xr.DataArray:
     """Dask-parallel version of windowed_run_events_1d, ie: the number of runs at least as long as given duration.
 
     Parameters
     ----------
     x : Sequence[bool]
-      Input array (bool)
+      Input array (bool).
     window : int
-      Minimum run length
+      Minimum run length.
 
     Returns
     -------
-    out : func
+    xr.DataArray
       A function operating along the time dimension of a dask-array.
     """
     return xr.apply_ufunc(
         windowed_run_events_1d,
         x,
-        input_core_dims=[["time"]],
+        input_core_dims=[[dim]],
         vectorize=True,
         dask="parallelized",
         output_dtypes=[int],
@@ -913,7 +985,10 @@ def windowed_run_events_ufunc(x: Sequence[bool], window: int) -> xr.DataArray:
 
 
 def statistics_run_ufunc(
-    x: Union[xr.DataArray, Sequence[bool]], reducer: str
+    x: Union[xr.DataArray, Sequence[bool]],
+    reducer: str,
+    window: int = 1,
+    dim: str = "time",
 ) -> xr.DataArray:
     """Dask-parallel version of statistics_run_1d, ie: the {reducer} number of consecutive true values in array.
 
@@ -921,19 +996,23 @@ def statistics_run_ufunc(
     ----------
     x : Sequence[bool]
       Input array (bool)
-    reducer: {'min', 'max', 'mean', 'sum'}
+    reducer: {'min', 'max', 'mean', 'sum', 'std'}
       Reducing function name.
+    window : int
+      Minimal length of runs.
+    dim : str
+      The dimension along which the run are found.
 
     Returns
     -------
-    out : func
+    xr.DataArray
       A function operating along the time dimension of a dask-array.
     """
     return xr.apply_ufunc(
         statistics_run_1d,
         x,
-        input_core_dims=[["time"]],
-        kwargs={"reducer": reducer},
+        input_core_dims=[[dim]],
+        kwargs={"reducer": reducer, "window": window},
         vectorize=True,
         dask="parallelized",
         output_dtypes=[float],
@@ -944,20 +1023,22 @@ def statistics_run_ufunc(
 def first_run_ufunc(
     x: Union[xr.DataArray, Sequence[bool]],
     window: int,
-    dim: str = "time",
+    dim: str,
 ) -> xr.DataArray:
     """Dask-parallel version of first_run_1d, ie: the first entry in array of consecutive true values.
 
     Parameters
     ----------
     x : Union[xr.DataArray, Sequence[bool]]
-      Input array (bool)
+      Input array (bool).
     window : int
-    dim: Optional[str]
+      Minimum run length.
+    dim : str
+      Dimension along which to index (default: "time").
 
     Returns
     -------
-    out : func
+    xr.DataArray
       A function operating along the time dimension of a dask-array.
     """
     ind = xr.apply_ufunc(
@@ -994,7 +1075,7 @@ def lazy_indexing(
     Returns
     -------
     xr.DataArray
-      Values of `da` at indices `index`
+      Values of `da` at indices `index`.
     """
     if da.ndim == 1:
         # Case where da is 1D and index is N-D
@@ -1076,7 +1157,7 @@ def index_of_date(
 
     Returns
     -------
-    ndarray
+    numpy.ndarray
       1D array of integers, indexes of `date` in `time`.
     """
     if date is None:
