@@ -1,8 +1,7 @@
 """SDBA utilities module."""
-from typing import Callable, List, Mapping, Optional, Union
+from typing import Callable, List, Mapping, Optional, Tuple, Union
 from warnings import warn
 
-import bottleneck as bn
 import numpy as np
 import xarray as xr
 from boltons.funcutils import wraps
@@ -19,69 +18,57 @@ ADDITIVE = "+"
 loffsets = {"MS": "14d", "M": "15d", "YS": "181d", "Y": "182d", "QS": "45d", "Q": "46d"}
 
 
-@parse_group
+def _ecdf_1d(x, value):
+    sx = np.r_[-np.inf, np.sort(x, axis=None)]
+    return np.searchsorted(sx, value, side="right") / np.sum(~np.isnan(sx))
+
+
+def map_cdf_1d(x, y, y_value):
+    """Return the value in `x` with the same CDF as `y_value` in `y`."""
+    q = _ecdf_1d(y, y_value)
+    _func = np.nanquantile
+    return _func(x, q=q)
+
+
 def map_cdf(
-    x: xr.DataArray,
-    y: xr.DataArray,
-    y_value: xr.DataArray,
+    ds: xr.Dataset,
     *,
-    group: Union[str, Grouper] = "time",
-    skipna: bool = False,
+    y_value: xr.DataArray,
+    dim,
 ):
     """Return the value in `x` with the same CDF as `y_value` in `y`.
 
+    This function is meant to be wrapped in a `Grouper.apply`.
+
     Parameters
     ----------
-    x : xr.DataArray
-      Values from which to pick
-    y : xr.DataArray
-      Reference values giving the ranking
+    ds : xr.Dataset
+      Variables: x, Values from which to pick,
+      y, Reference values giving the ranking
     y_value : float, array
       Value within the support of `y`.
     dim : str
       Dimension along which to compute quantile.
-    group: Union[str, Grouper]
-    skipna: bool
 
     Returns
     -------
     array
       Quantile of `x` with the same CDF as `y_value` in `y`.
     """
-
-    def _map_cdf_1d(x, y, y_value, skipna=False):
-        q = _ecdf_1d(y, y_value)
-        _func = np.nanquantile if skipna else np.quantile
-        return _func(x, q=q)
-
-    def _map_cdf_group(gr, y_value, dim=["time"], skipna=False):
-        return xr.apply_ufunc(
-            _map_cdf_1d,
-            gr.x,
-            gr.y,
-            input_core_dims=[dim] * 2,
-            output_core_dims=[["x"]],
-            vectorize=True,
-            keep_attrs=True,
-            kwargs={"y_value": y_value, "skipna": skipna},
-            dask="parallelized",
-            output_dtypes=[gr.x.dtype],
-        )
-
-    return group.apply(
-        _map_cdf_group,
-        {"x": x, "y": y},
-        y_value=np.atleast_1d(y_value),
-        skipna=skipna,
+    return xr.apply_ufunc(
+        map_cdf_1d,
+        ds.x,
+        ds.y,
+        input_core_dims=[dim] * 2,
+        output_core_dims=[["x"]],
+        vectorize=True,
+        keep_attrs=True,
+        kwargs={"y_value": np.atleast_1d(y_value)},
+        output_dtypes=[ds.x.dtype],
     )
 
 
-def _ecdf_1d(x, value):
-    sx = np.r_[-np.inf, np.sort(x)]
-    return np.searchsorted(sx, value, side="right") / np.sum(~np.isnan(sx))
-
-
-def ecdf(x: xr.DataArray, value: float, dim: str = "time"):
+def ecdf(x: xr.DataArray, value: float, dim: str = "time") -> xr.DataArray:
     """Return the empirical CDF of a sample at a given value.
 
     Parameters
@@ -90,10 +77,12 @@ def ecdf(x: xr.DataArray, value: float, dim: str = "time"):
       Sample.
     value : float
       The value within the support of `x` for which to compute the CDF value.
+    dim : str
+      Dimension name.
 
     Returns
     -------
-    array
+    xr.DataArray
       Empirical CDF.
     """
     return (x <= value).sum(dim) / x.notnull().sum(dim)
@@ -128,7 +117,7 @@ def ensure_longest_doy(func: Callable) -> Callable:
 
 
 @ensure_longest_doy
-def get_correction(x: xr.DataArray, y: xr.DataArray, kind: str):
+def get_correction(x: xr.DataArray, y: xr.DataArray, kind: str) -> xr.DataArray:
     """Return the additive or multiplicative correction/adjustment factors."""
     with xr.set_options(keep_attrs=True):
         if kind == ADDITIVE:
@@ -144,7 +133,9 @@ def get_correction(x: xr.DataArray, y: xr.DataArray, kind: str):
 
 
 @ensure_longest_doy
-def apply_correction(x: xr.DataArray, factor: xr.DataArray, kind: Optional[str] = None):
+def apply_correction(
+    x: xr.DataArray, factor: xr.DataArray, kind: Optional[str] = None
+) -> xr.DataArray:
     """Apply the additive or multiplicative correction/adjustment factors.
 
     If kind is not given, default to the one stored in the "kind" attribute of factor.
@@ -160,7 +151,7 @@ def apply_correction(x: xr.DataArray, factor: xr.DataArray, kind: Optional[str]
     return out
 
 
-def invert(x: xr.DataArray, kind: Optional[str] = None):
+def invert(x: xr.DataArray, kind: Optional[str] = None) -> xr.DataArray:
     """Invert a DataArray either additively (-x) or multiplicatively (1/x).
 
     If kind is not given, default to the one stored in the "kind" attribute of x.
@@ -182,7 +173,7 @@ def broadcast(
     group: Union[str, Grouper] = "time",
     interp: str = "nearest",
     sel: Optional[Mapping[str, xr.DataArray]] = None,
-):
+) -> xr.DataArray:
     """Broadcast a grouped array back to the same shape as a given array.
 
     Parameters
@@ -197,11 +188,15 @@ def broadcast(
       The interpolation method to use,
     sel : Mapping[str, xr.DataArray]
       Mapping of grouped coordinates to x coordinates (other than the grouping one).
+
+    Returns
+    -------
+    xr.DataArray
     """
     if sel is None:
         sel = {}
 
-    if group.prop is not None and group.prop not in sel:
+    if group.prop != "group" and group.prop not in sel:
         sel.update({group.prop: group.get_index(x, interp=interp)})
 
     if sel:
@@ -209,7 +204,12 @@ def broadcast(
         if interp == "nearest":  # Interpolate both the time group and the quantile.
             grouped = grouped.sel(sel, method="nearest")
         else:  # Find quantile for nearest time group and quantile.
-            if group.prop is not None:
+            # For `.interp` we need to explicitly pass the shared dims
+            # (see pydata/xarray#4463 and Ouranosinc/xclim#449,567)
+            sel.update(
+                {dim: x[dim] for dim in set(grouped.dims).intersection(set(x.dims))}
+            )
+            if group.prop != "group":
                 grouped = add_cyclic_bounds(grouped, group.prop, cyclic_coords=False)
 
             if interp == "cubic" and len(sel.keys()) > 1:
@@ -219,16 +219,18 @@ def broadcast(
                     " interpolation, not cubic. Using linear."
                 )
 
-            grouped = grouped.interp(sel, method=interp)
+            grouped = grouped.interp(sel, method=interp).astype(grouped.dtype)
 
         for var in sel.keys():
             if var in grouped.coords and var not in grouped.dims:
                 grouped = grouped.drop_vars(var)
 
+    if group.prop == "group" and "group" in grouped.dims:
+        grouped = grouped.squeeze("group", drop=True)
     return grouped
 
 
-def equally_spaced_nodes(n: int, eps: Union[float, None] = 1e-4):
+def equally_spaced_nodes(n: int, eps: Union[float, None] = 1e-4) -> np.array:
     """Return nodes with `n` equally spaced points within [0, 1] plus two end-points.
 
     Parameters
@@ -240,7 +242,7 @@ def equally_spaced_nodes(n: int, eps: Union[float, None] = 1e-4):
 
     Returns
     -------
-    array
+    np.array
       Nodes between 0 and 1.
 
     Notes
@@ -251,10 +253,12 @@ def equally_spaced_nodes(n: int, eps: Union[float, None] = 1e-4):
     q = np.linspace(dq, 1 - dq, n)
     if eps is None:
         return q
-    return sorted(np.append([eps, 1 - eps], q))
+    return np.insert(np.append(q, 1 - eps), 0, eps)
 
 
-def add_cyclic_bounds(da: xr.DataArray, att: str, cyclic_coords: bool = True):
+def add_cyclic_bounds(
+    da: xr.DataArray, att: str, cyclic_coords: bool = True
+) -> Union[xr.DataArray, xr.Dataset]:
     """Reindex an array to include the last slice at the beginning and the first at the end.
 
     This is done to allow interpolation near the end-points.
@@ -286,7 +290,12 @@ def add_cyclic_bounds(da: xr.DataArray, att: str, cyclic_coords: bool = True):
     return ensure_chunk_size(qmf, **{att: -1})
 
 
-def extrapolate_qm(qf: xr.DataArray, xq: xr.DataArray, method: str = "constant"):
+def extrapolate_qm(
+    qf: xr.DataArray,
+    xq: xr.DataArray,
+    method: str = "constant",
+    abs_bounds: Optional[tuple] = (-np.inf, np.inf),
+) -> Tuple[xr.DataArray, xr.DataArray]:
     """Extrapolate quantile adjustment factors beyond the computed quantiles.
 
     Parameters
@@ -297,17 +306,21 @@ def extrapolate_qm(qf: xr.DataArray, xq: xr.DataArray, method: str = "constant")
       Values at each `quantile`.
     method : {"constant"}
       Extrapolation method. See notes below.
+    abs_bounds : 2-tuple
+      The absolute bounds for the "constant*" methods. Defaults to (-inf, inf).
 
     Returns
     -------
-    xr.Dataset
-        Extrapolated adjustment factors and x-values.
+    xr.Dataset or xr.DataArray
+        Extrapolated adjustment factors.
+    xr.Dataset or xr.DataArray
+        Extrapolated x-values.
 
     Notes
     -----
-    nan
+    qf: xr.DataArray
       Estimating values above or below the computed values will return a NaN.
-    constant
+    xq: xr.DataArray
       The adjustment factor above and below the computed values are equal to the last and first values
       respectively.
     """
@@ -318,7 +331,7 @@ def extrapolate_qm(qf: xr.DataArray, xq: xr.DataArray, method: str = "constant")
 
     if method == "constant":
         q_l, q_r = [0], [1]
-        x_l, x_r = [-np.inf], [np.inf]
+        x_l, x_r = [abs_bounds[0]], [abs_bounds[1]]
         qf_l, qf_r = qf.isel(quantiles=0), qf.isel(quantiles=-1)
 
     elif (
@@ -342,7 +355,7 @@ def add_endpoints(
     left: List[Union[int, float, xr.DataArray, List[int], List[float]]],
     right: List[Union[int, float, xr.DataArray, List[int], List[float]]],
     dim: str = "quantiles",
-):
+) -> xr.DataArray:
     """Add left and right endpoints to a DataArray.
 
     Parameters
@@ -386,7 +399,7 @@ def interp_on_quantiles(
     Parameters
     ----------
     newx : xr.DataArray
-        The values at wich to evalute `yq`. If `group` has group information,
+        The values at which to evaluate `yq`. If `group` has group information,
         `new` should have a coordinate with the same name as the group name
          In that case, 2D interpolation is used.
     xq, yq : xr.DataArray
@@ -402,7 +415,7 @@ def interp_on_quantiles(
     dim = group.dim
     prop = group.prop
 
-    if prop is None:
+    if prop == "group":
         fill_value = "extrapolate" if method == "nearest" else np.nan
 
         def _interp_quantiles_1D(newx, oldx, oldy):
@@ -413,8 +426,8 @@ def interp_on_quantiles(
         return xr.apply_ufunc(
             _interp_quantiles_1D,
             newx,
-            xq,
-            yq,
+            xq.squeeze("group", drop=True),
+            yq.squeeze("group", drop=True),
             input_core_dims=[[dim], ["quantiles"], ["quantiles"]],
             output_core_dims=[[dim]],
             vectorize=True,
@@ -423,19 +436,19 @@ def interp_on_quantiles(
         )
     # else:
 
-    def _interp_quantiles_2D(newx, newg, oldx, oldy, oldg):
+    def _interp_quantiles_2D(_newx, _newg, _oldx, _oldy, _oldg):  # noqa
         if method != "nearest":
-            oldx = np.clip(oldx, newx.min() - 1, newx.max() + 1)
-        if np.all(np.isnan(newx)):
+            _oldx = np.clip(_oldx, _newx.min() - 1, _newx.max() + 1)
+        if np.all(np.isnan(_newx)):
             warn(
                 "All-NaN slice encountered in interp_on_quantiles",
                 category=RuntimeWarning,
             )
-            return newx
+            return _newx
         return griddata(
-            (oldx.flatten(), oldg.flatten()),
-            oldy.flatten(),
-            (newx, newg),
+            (_oldx.flatten(), _oldg.flatten()),
+            _oldy.flatten(),
+            (_newx, _newg),
             method=method,
         )
 
@@ -465,10 +478,12 @@ def interp_on_quantiles(
     )
 
 
-def rank(da, dim="time", pct=False):
-    """Ranks data.
+# TODO is this useless?
+def rank(da: xr.DataArray, dim: str = "time", pct: bool = False) -> xr.DataArray:
+    """Ranks data along a dimension.
 
-    Replicates `xr.DataArray.rank` but with support for dask-stored data. Xarray's docstring is below:
+    Replicates `xr.DataArray.rank` but as a function usable in a Grouper.apply().
+    Xarray's docstring is below:
 
     Equal values are assigned a rank that is the average of the ranks that
     would have been otherwise assigned to all of the values within that
@@ -480,36 +495,21 @@ def rank(da, dim="time", pct=False):
 
     Parameters
     ----------
-    dim : hashable
+    da: xr.DataArray
+    dim : str, hashable
         Dimension over which to compute rank.
     pct : bool, optional
         If True, compute percentage ranks, otherwise compute integer ranks.
 
     Returns
     -------
-    ranked : DataArray
+    DataArray
         DataArray with the same coordinates and dtype 'float64'.
     """
-
-    def _nanrank(data):
-        func = bn.nanrankdata if data.dtype.kind == "f" else bn.rankdata
-        ranked = func(data, axis=-1)
-        if pct:
-            count = np.sum(~np.isnan(data), axis=-1, keepdims=True)
-            ranked /= count
-        return ranked
-
-    return xr.apply_ufunc(
-        _nanrank,
-        da,
-        input_core_dims=[[dim]],
-        output_core_dims=[[dim]],
-        dask="parallelized",
-        output_dtypes=[da.dtype],
-    )
+    return da.rank(dim, pct=pct)
 
 
-def pc_matrix(arr: Union[np.ndarray, dsk.Array]):
+def pc_matrix(arr: Union[np.ndarray, dsk.Array]) -> Union[np.ndarray, dsk.Array]:
     """Construct a Principal Component matrix.
 
     This matrix can be used to transform points in arr to principal components
@@ -518,12 +518,12 @@ def pc_matrix(arr: Union[np.ndarray, dsk.Array]):
 
     Parameters
     ----------
-    arr : Union[numpy.ndarray, dask.array.Array]
+    arr : numpy.ndarray or dask.array.Array
       2D array (M, N) of the M coordinates of N points.
 
     Returns
     -------
-    A
+    numpy.ndarray or dask.array.Array
       MxM Array of the same type as arr.
     """
     # Get appropriate math module
@@ -544,7 +544,9 @@ def pc_matrix(arr: Union[np.ndarray, dsk.Array]):
     return eig_vec * mod.sqrt(eig_vals)
 
 
-def best_pc_orientation(A, Binv, val=1000):
+def best_pc_orientation(
+    A: np.ndarray, Binv: np.ndarray, val: float = 1000
+) -> np.ndarray:
     """Return best orientation vector for A.
 
     Eigenvectors returned by `pc_matrix` do not have a defined orientation.
@@ -556,9 +558,9 @@ def best_pc_orientation(A, Binv, val=1000):
 
     Parameters
     ----------
-    A : numpy.ndarray
+    A : np.ndarray
       MxM Matrix defining the final transformation.
-    Binv : numpy.ndarray
+    Binv : np.ndarray
       MxM Matrix defining the (inverse) first transformation.
     val : float
       The coordinate of the test point (same for all axes). It should be much
@@ -566,7 +568,7 @@ def best_pc_orientation(A, Binv, val=1000):
 
     Returns
     -------
-    orient :
+    np.ndarray
       Mx1 vector of orientation correction (1 or -1).
     """
     m = A.shape[0]
@@ -589,7 +591,9 @@ def best_pc_orientation(A, Binv, val=1000):
     return orient
 
 
-def get_clusters_1d(data: np.ndarray, u1: float, u2: float):
+def get_clusters_1d(
+    data: np.ndarray, u1: float, u2: float
+) -> Tuple[np.array, np.array, np.array, np.array]:
     """Get clusters of a 1D array.
 
     A cluster is defined as a sequence of values larger than u2 with at least one value larger than u1.
@@ -603,8 +607,12 @@ def get_clusters_1d(data: np.ndarray, u1: float, u2: float):
     u2 : float
       Cluster threshold, values above this can be part of a cluster.
 
-    Reference
-    ---------
+    Returns
+    -------
+    (np.array, np.array, np.array, np.array)
+
+    References
+    ----------
     `getcluster` of Extremes.jl (read on 2021-04-20) https://github.com/jojal5/Extremes.jl
     """
     # Boolean array, True where data is over u2
@@ -639,11 +647,22 @@ def get_clusters_1d(data: np.ndarray, u1: float, u2: float):
     )
 
 
-def get_clusters(data: xr.DataArray, u1, u2, dim: str = "time"):
+def get_clusters(data: xr.DataArray, u1, u2, dim: str = "time") -> xr.Dataset:
     """Get cluster count, maximum and position along a given dim.
 
     See `get_clusters_1d`. Used by `adjustment.ExtremeValues`.
 
+    Parameters
+    ----------
+    data: 1D ndarray
+      Values to get clusters from.
+    u1 : float
+      Extreme value threshold, at least one value in the cluster must exceed this.
+    u2 : float
+      Cluster threshold, values above this can be part of a cluster.
+    dim : str
+      Dimension name.
+
     Returns
     -------
     xr.Dataset
@@ -681,7 +700,6 @@ def get_clusters(data: xr.DataArray, u1, u2, dim: str = "time"):
         input_core_dims=[[dim], [], []],
         output_core_dims=[["cluster"], ["cluster"], ["cluster"], ["cluster"], []],
         kwargs={"N": N},
-        output_dtypes=[int, int, int, data.dtype, int],
         dask="parallelized",
         vectorize=True,
         dask_gufunc_kwargs={
@@ -707,3 +725,55 @@ def get_clusters(data: xr.DataArray, u1, u2, dim: str = "time"):
     )
 
     return ds
+
+
+def rand_rot_matrix(
+    crd: xr.DataArray, num: int = 1, new_dim: Optional[str] = None
+) -> xr.DataArray:
+    r"""Generate random rotation matrices.
+
+    Rotation matrices are members of the SO(n) group, where n is the matrix size (`crd.size`).
+    They can be characterized as orthogonal matrices with determinant 1. A square matrix :math:`R`
+    is a rotation matrix if and only if :math:`R^t = R^{−1}` and :math:`\mathrm{det} R = 1`.
+
+    Parameters
+    ----------
+    crd: xr.DataArray
+      1D coordinate DataArray along which the rotation occurs.
+      The output will be square with the same coordinate replicated,
+      the second renamed to `new_dim`.
+    num : int
+      If larger than 1 (default), the number of matrices to generate, stacked along a "matrices" dimension.
+    new_dim : str
+      Name of the new "prime" dimension, defaults to the same name as `crd` + "_prime".
+
+    Returns
+    -------
+    xr.DataArray
+      float, NxN if num = 1, numxNxN otherwise, where N is the length of crd.
+
+    References
+    ----------
+    .. [Mezzadri] Mezzadri, F. (2006). How to generate random matrices from the classical compact groups. arXiv preprint math-ph/0609050.
+    """
+    if num > 1:
+        return xr.concat([rand_rot_matrix(crd, num=1) for i in range(num)], "matrices")
+
+    N = crd.size
+    dim = crd.dims[0]
+    # Rename and rebuild second coordinate : "prime" axis.
+    if new_dim is None:
+        new_dim = dim + "_prime"
+    crd2 = xr.DataArray(crd.values, dims=new_dim, name=new_dim, attrs=crd.attrs)
+
+    # Random floats from the standardized normal distribution
+    Z = np.random.standard_normal((N, N))
+
+    # QR decomposition and manipulation from Mezzadri 2006
+    Q, R = np.linalg.qr(Z)
+    num = np.diag(R)
+    denum = np.abs(num)
+    lam = np.diag(num / denum)  # "lambda"
+    return xr.DataArray(
+        Q @ lam, dims=(dim, new_dim), coords={dim: crd, new_dim: crd2}
+    ).astype("float32")
