@@ -5,40 +5,50 @@ import pandas as pd
 import pytest
 import xarray as xr
 
+from xclim.core.options import set_options
 from xclim.indices import run_length as rl
 from xclim.testing import open_dataset
 
 K2C = 273.15
 
 
-class TestRLE:
-    def test_dataarray(self):
-        values = np.zeros(365)
-        time = pd.date_range(
-            "7/1/2000", periods=len(values), freq=pd.DateOffset(days=1)
-        )
-        values[1:11] = 1
-        da = xr.DataArray(values, coords={"time": time}, dims="time")
+@pytest.fixture(scope="module", params=[True, False], autouse=True)
+def ufunc(request):
+    with set_options(run_length_ufunc=request.param):
+        yield request.param
 
+
+@pytest.mark.parametrize("use_dask", [True, False])
+@pytest.mark.parametrize("index", ["first", "last"])
+def test_rle(ufunc, use_dask, index):
+    if use_dask and ufunc:
+        pytest.xfail("rle_1d is not implemented for dask arrays.")
+
+    values = np.zeros((10, 365, 4, 4))
+    time = pd.date_range("2000-01-01", periods=365, freq="D")
+    values[:, 1:11, ...] = 1
+    da = xr.DataArray(values, coords={"time": time}, dims=("a", "time", "b", "c"))
+
+    if ufunc:
+        da = da[0, :, 0, 0]
         v, l, p = rl.rle_1d(da != 0)
         np.testing.assert_array_equal(v, [False, True, False])
         np.testing.assert_array_equal(l, [1, 10, 354])
         np.testing.assert_array_equal(p, [0, 1, 11])
-
-    @pytest.mark.parametrize("use_dask", [True, False])
-    def test_dataarray_nd(self, use_dask):
-        values = np.zeros((10, 365, 4, 4))
-        time = pd.date_range("2000-01-01", periods=365, freq="D")
-        values[:, 1:11, ...] = 1
-        da = xr.DataArray(values, coords={"time": time}, dims=("a", "time", "b", "c"))
-
+    else:
         if use_dask:
             da = da.chunk({"a": 1, "b": 2})
+        import time
 
-        out = rl.rle(da != 0).mean(["a", "b", "c"])
-        expected = np.zeros(365)
-        expected[1] = 10
-        expected[2:11] = np.nan
+        out = rl.rle(da != 0, index=index).mean(["a", "b", "c"])
+        if index == "last":
+            expected = np.zeros(365)
+            expected[1:10] = np.nan
+            expected[10] = 10
+        else:
+            expected = np.zeros(365)
+            expected[1] = 10
+            expected[2:11] = np.nan
         np.testing.assert_array_equal(out, expected)
 
 
@@ -52,19 +62,10 @@ class TestStatisticsRun:
         )
         values[1:11] = 1
         da = xr.DataArray(values != 0, coords={"time": time}, dims="time")
-        lt = da.resample(time="M").map(rl.statistics_run_ufunc, reducer="max")
+        lt = da.resample(time="M").map(rl.rle_statistics, reducer="max")
         assert lt[0] == 10
         np.testing.assert_array_equal(lt[1:], 0)
 
-        # n-dim version versus ufunc
-        da3d = open_dataset(self.nc_pr).pr[:, 40:50, 50:68] != 0
-        lt_orig = da3d.resample(time="M").map(rl.statistics_run_ufunc, reducer="max")
-        # override 'auto' usage of ufunc for small number of gridpoints
-        lt_Ndim = da3d.resample(time="M").map(
-            rl.longest_run, dim="time", ufunc_1dim=False
-        )
-        np.testing.assert_array_equal(lt_orig, lt_Ndim)
-
     def test_start_at_0(self):
         values = np.zeros(365)
         time = pd.date_range(
@@ -72,21 +73,10 @@ class TestStatisticsRun:
         )
         values[0:10] = 1
         da = xr.DataArray(values != 0, coords={"time": time}, dims="time")
-        lt = da.resample(time="M").map(rl.statistics_run_ufunc, reducer="max")
+        lt = da.resample(time="M").map(rl.rle_statistics, reducer="max")
         assert lt[0] == 10
         np.testing.assert_array_equal(lt[1:], 0)
 
-        # n-dim version versus ufunc
-        da3d = open_dataset(self.nc_pr).pr[:, 40:50, 50:68] * 0
-        da3d[0:10] = da3d[0:10] + 1
-        da3d = da3d == 1
-        lt_orig = da3d.resample(time="M").map(rl.statistics_run_ufunc, reducer="max")
-        # override 'auto' usage of ufunc for small number of gridpoints
-        lt_Ndim = da3d.resample(time="M").map(
-            rl.longest_run, dim="time", ufunc_1dim=False
-        )  # override 'auto' for small
-        np.testing.assert_array_equal(lt_orig, lt_Ndim)
-
     def test_end_start_at_0(self):
         values = np.zeros(365)
         time = pd.date_range(
@@ -95,20 +85,10 @@ class TestStatisticsRun:
         values[-10:] = 1
         da = xr.DataArray(values != 0, coords={"time": time}, dims="time")
 
-        lt = da.resample(time="M").map(rl.statistics_run_ufunc, reducer="max")
+        lt = da.resample(time="M").map(rl.rle_statistics, reducer="max")
         assert lt[-1] == 10
         np.testing.assert_array_equal(lt[:-1], 0)
 
-        # n-dim version versus ufunc
-        da3d = open_dataset(self.nc_pr).pr[:, 40:50, 50:68] * 0
-        da3d[-10:] = da3d[-10:] + 1
-        da3d = da3d == 1
-        lt_orig = da3d.resample(time="M").map(rl.statistics_run_ufunc, reducer="max")
-        lt_Ndim = da3d.resample(time="M").map(
-            rl.longest_run, dim="time", ufunc_1dim=False
-        )
-        np.testing.assert_array_equal(lt_orig, lt_Ndim)
-
     def test_all_true(self):
         values = np.ones(365)
         time = pd.date_range(
@@ -116,18 +96,9 @@ class TestStatisticsRun:
         )
         da = xr.DataArray(values != 0, coords={"time": time}, dims="time")
 
-        lt = da.resample(time="M").map(rl.statistics_run_ufunc, reducer="max")
+        lt = da.resample(time="M").map(rl.rle_statistics, reducer="max")
         np.testing.assert_array_equal(lt, da.resample(time="M").count(dim="time"))
 
-        # n-dim version versus ufunc
-        da3d = open_dataset(self.nc_pr).pr[:, 40:50, 50:68] * 0 + 1
-        da3d = da3d == 1
-        lt_orig = da3d.resample(time="M").map(rl.statistics_run_ufunc, reducer="max")
-        lt_Ndim = da3d.resample(time="M").map(
-            rl.longest_run, dim="time", ufunc_1dim=False
-        )
-        np.testing.assert_array_equal(lt_orig, lt_Ndim)
-
     def test_almost_all_true(self):
         values = np.ones(365)
         values[35] = 0
@@ -136,23 +107,12 @@ class TestStatisticsRun:
         )
         da = xr.DataArray(values != 0, coords={"time": time}, dims="time")
 
-        lt = da.resample(time="M").map(rl.statistics_run_ufunc, reducer="max")
+        lt = da.resample(time="M").map(rl.rle_statistics, reducer="max")
         n = da.resample(time="M").count(dim="time")
         np.testing.assert_array_equal(lt[0], n[0])
         np.testing.assert_array_equal(lt[1], 26)
 
-        # n-dim version versus ufunc
-        da3d = open_dataset(self.nc_pr).pr[:, 40:50, 50:68] * 0 + 1
-        da3d[35] = da3d[35] + 1
-        da3d = da3d == 1
-        lt_orig = da3d.resample(time="M").map(rl.statistics_run_ufunc, reducer="max")
-        lt_Ndim = da3d.resample(time="M").map(
-            rl.longest_run, dim="time", ufunc_1dim=False
-        )
-        np.testing.assert_array_equal(lt_orig, lt_Ndim)
-
-    @pytest.mark.parametrize("ufunc_1dim", [True, False])
-    def test_other_stats(self, ufunc_1dim):
+    def test_other_stats(self):
         values = np.ones(365)
         values[35] = 0
         time = pd.date_range(
@@ -160,94 +120,68 @@ class TestStatisticsRun:
         )
         da = xr.DataArray(values != 0, coords={"time": time}, dims="time")
 
-        lt = da.resample(time="YS").map(
-            rl.rle_statistics, reducer="min", ufunc_1dim=ufunc_1dim
-        )
+        lt = da.resample(time="YS").map(rl.rle_statistics, reducer="min")
         assert lt == 35
 
-        lt = da.resample(time="YS").map(
-            rl.rle_statistics, reducer="mean", ufunc_1dim=ufunc_1dim
-        )
-        assert lt == 182
+        lt = da.resample(time="YS").map(rl.rle_statistics, reducer="mean", window=36)
+        assert lt == 329
 
-        lt = da.resample(time="YS").map(
-            rl.rle_statistics, reducer="std", ufunc_1dim=ufunc_1dim
-        )
+        lt = da.resample(time="YS").map(rl.rle_statistics, reducer="std")
         assert lt == 147
 
 
 class TestFirstRun:
     nc_pr = os.path.join("NRCANdaily", "nrcan_canada_daily_pr_1990.nc")
 
-    def test_run_1d(self):
-        a = np.zeros(100, bool)
+    def test_real_simple(self):
+        a = xr.DataArray(np.zeros(100, bool), dims=("x",))
         a[10:20] = 1
-        i = rl.first_run_1d(a, 5)
+        i = rl.first_run(a, 5, dim="x")
         assert 10 == i
 
     def test_real_data(self):
         # n-dim version versus ufunc
         da3d = open_dataset(self.nc_pr).pr[:, 40:50, 50:68] != 0
-        lt_orig = da3d.resample(time="M").map(rl.first_run_ufunc, window=5)
-        lt_Ndim = da3d.resample(time="M").map(
-            rl.first_run, window=5, dim="time", ufunc_1dim=False
-        )
-        np.testing.assert_array_equal(lt_orig, lt_Ndim)
+        da3d.resample(time="M").map(rl.first_run, window=5)
 
     @pytest.mark.parametrize(
         "coord,expected",
         [(False, 30), (True, np.datetime64("2000-01-31")), ("dayofyear", 31)],
     )
-    @pytest.mark.parametrize(
-        "use_dask,use_1dim", [(True, False), (False, False), (False, True)]
-    )
-    def test_simple(self, tas_series, coord, expected, use_dask, use_1dim):
+    @pytest.mark.parametrize("use_dask", [True, False])
+    def test_simple(self, tas_series, coord, expected, use_dask, ufunc):
+        # if use_dask and ufunc:
+        #     pytest.xfail("Ufunc run length algorithms not implemented for dask arrays.")
         t = np.zeros(60)
         t[30:40] = 2
         tas = tas_series(t, start="2000-01-01")
         runs = xr.concat((tas, tas), dim="dim0")
 
         if use_dask:
-            runs = runs.chunk({"time": 10, "dim0": 1})
+            runs = runs.chunk({"time": -1 if ufunc else 10, "dim0": 1})
 
-        out = rl.first_run(runs, window=1, dim="time", coord=coord, ufunc_1dim=use_1dim)
+        out = rl.first_run(runs, window=1, dim="time", coord=coord)
         np.testing.assert_array_equal(out.load(), expected)
 
 
 class TestWindowedRunEvents:
-    nc_pr = os.path.join("NRCANdaily", "nrcan_canada_daily_pr_1990.nc")
-
-    def test_simple(self):
-        a = np.zeros(50, bool)
+    @pytest.mark.parametrize("index", ["first", "last"])
+    def test_simple(self, index):
+        a = xr.DataArray(np.zeros(50, bool), dims=("x",))
         a[4:7] = True
         a[34:45] = True
-        assert rl.windowed_run_events_1d(a, 3) == 2
-
-        # n-dim version versus ufunc
-        da3d = open_dataset(self.nc_pr).pr[:, 40:50, 50:68] != 0
-        lt_orig = da3d.resample(time="M").map(rl.windowed_run_events_ufunc, window=4)
-        lt_Ndim = da3d.resample(time="M").map(
-            rl.windowed_run_events, window=4, dim="time", ufunc_1dim=False
-        )
-        np.testing.assert_array_equal(lt_orig, lt_Ndim)
+        assert rl.windowed_run_events(a, 3, dim="x", index=index) == 2
 
 
 class TestWindowedRunCount:
-    nc_pr = os.path.join("NRCANdaily", "nrcan_canada_daily_pr_1990.nc")
-
-    def test_simple(self):
-        a = np.zeros(50, bool)
+    @pytest.mark.parametrize("index", ["first", "last"])
+    def test_simple(self, index):
+        a = xr.DataArray(np.zeros(50, bool), dims=("time",))
         a[4:7] = True
         a[34:45] = True
-        assert rl.windowed_run_count_1d(a, 3) == len(a[4:7]) + len(a[34:45])
-
-        # n-dim version versus ufunc
-        da3d = open_dataset(self.nc_pr).pr[:, 40:50, 50:68] != 0
-        lt_orig = da3d.resample(time="M").map(rl.windowed_run_count_ufunc, window=4)
-        lt_Ndim = da3d.resample(time="M").map(
-            rl.windowed_run_count, window=4, dim="time", ufunc_1dim=False
-        )
-        np.testing.assert_array_equal(lt_orig, lt_Ndim)
+        assert rl.windowed_run_count(a, 3, dim="time", index=index) == len(
+            a[4:7]
+        ) + len(a[34:45])
 
 
 class TestLastRun:
@@ -255,19 +189,19 @@ class TestLastRun:
         "coord,expected",
         [(False, 39), (True, np.datetime64("2000-02-09")), ("dayofyear", 40)],
     )
-    @pytest.mark.parametrize(
-        "use_dask,use_1dim", [(True, False), (False, False), (False, True)]
-    )
-    def test_simple(self, tas_series, coord, expected, use_dask, use_1dim):
+    @pytest.mark.parametrize("use_dask", [True, False])
+    def test_simple(self, tas_series, coord, expected, use_dask, ufunc):
+        # if use_dask and ufunc:
+        #     pytest.xfail("Ufunc run length algorithms not implemented for dask arrays.")
         t = np.zeros(60)
         t[30:40] = 2
         tas = tas_series(t, start="2000-01-01")
         runs = xr.concat((tas, tas), dim="dim0")
 
         if use_dask:
-            runs = runs.chunk({"time": 10, "dim0": 1})
+            runs = runs.chunk({"time": -1 if ufunc else 10, "dim0": 1})
 
-        out = rl.last_run(runs, window=1, dim="time", coord=coord, ufunc_1dim=use_1dim)
+        out = rl.last_run(runs, window=1, dim="time", coord=coord, ufunc_1dim=ufunc)
         np.testing.assert_array_equal(out.load(), expected)
 
 
@@ -298,8 +232,8 @@ def test_run_bounds_data():
 
 
 def test_keep_longest_run_synthetic():
-    runs = xr.DataArray([0, 1, 1, 1, 0, 0, 1, 1, 1, 0], dims="x").astype(bool)
-    lrun = rl.keep_longest_run(runs, "x")
+    runs = xr.DataArray([0, 1, 1, 1, 0, 0, 1, 1, 1, 0], dims="time").astype(bool)
+    lrun = rl.keep_longest_run(runs, "time")
     np.testing.assert_array_equal(
         lrun, np.array([0, 1, 1, 1, 0, 0, 0, 0, 0, 0], dtype=bool)
     )
@@ -315,7 +249,8 @@ def test_keep_longest_run_data():
     )
 
     xr.testing.assert_equal(
-        rl.keep_longest_run(cond, "time").sum("time"), rl.longest_run(cond, "time")
+        rl.keep_longest_run(cond, "time").sum("time"),
+        rl.longest_run(cond, "time"),
     )
 
 
@@ -331,7 +266,9 @@ class TestRunsWithDates:
         ],
     )
     @pytest.mark.parametrize("use_dask", [True, False])
-    def test_season_length(self, tas_series, date, end, expected, use_dask):
+    def test_season_length(self, tas_series, date, end, expected, use_dask, ufunc):
+        # if use_dask and ufunc:
+        #     pytest.xfail("Ufunc run length algorithms not implemented for dask arrays.")
         t = np.zeros(360)
         t[140:end] = 1
         tas = tas_series(t, start="2000-01-01")
@@ -339,7 +276,7 @@ class TestRunsWithDates:
         runs = runs == 1
 
         if use_dask:
-            runs = runs.chunk({"time": 10, "dim0": 1})
+            runs = runs.chunk({"time": -1 if ufunc else 10, "dim0": 1})
 
         out = rl.season_length(
             runs,
@@ -359,7 +296,12 @@ class TestRunsWithDates:
         ],
     )
     @pytest.mark.parametrize("use_dask", [True, False])
-    def test_run_end_after_date(self, tas_series, coord, date, end, expected, use_dask):
+    def test_run_end_after_date(
+        self, tas_series, coord, date, end, expected, use_dask, ufunc
+    ):
+        # if use_dask and ufunc:
+        #     pytest.xfail("Ufunc run length algorithms not implemented for dask arrays.")
+
         t = np.zeros(360)
         t[140:end] = 1
         tas = tas_series(t, start="2000-01-01")
@@ -367,7 +309,7 @@ class TestRunsWithDates:
         runs = runs == 1
 
         if use_dask:
-            runs = runs.chunk({"time": 10, "dim0": 1})
+            runs = runs.chunk({"time": -1 if ufunc else 10, "dim0": 1})
 
         out = rl.run_end_after_date(runs, window=1, date=date, dim="time", coord=coord)
         np.testing.assert_array_equal(np.mean(out.load()), expected)
@@ -383,8 +325,10 @@ class TestRunsWithDates:
     )
     @pytest.mark.parametrize("use_dask", [True, False])
     def test_first_run_after_date(
-        self, tas_series, coord, date, beg, expected, use_dask
+        self, tas_series, coord, date, beg, expected, use_dask, ufunc
     ):
+        # if use_dask and ufunc:
+        #     pytest.xfail("Ufunc run length algorithms not implemented for dask arrays.")
         t = np.zeros(365)
         if beg:
             t[beg:] = 1
@@ -393,7 +337,7 @@ class TestRunsWithDates:
         runs = runs == 1
 
         if use_dask:
-            runs = runs.chunk({"time": 10, "dim0": 1})
+            runs = runs.chunk({"time": -1 if ufunc else 10, "dim0": 1})
 
         out = rl.first_run_after_date(
             runs, window=1, date=date, dim="time", coord=coord
@@ -411,8 +355,10 @@ class TestRunsWithDates:
     )
     @pytest.mark.parametrize("use_dask", [True, False])
     def test_last_run_before_date(
-        self, tas_series, coord, date, end, expected, use_dask
+        self, tas_series, coord, date, end, expected, use_dask, ufunc
     ):
+        # if use_dask and ufunc:
+        #     pytest.xfail("Ufunc run length algorithms not implemented for dask arrays.")
         t = np.zeros(360)
         t[140:end] = 1
         tas = tas_series(t, start="2000-01-01")
@@ -420,7 +366,7 @@ class TestRunsWithDates:
         runs = runs == 1
 
         if use_dask:
-            runs = runs.chunk({"time": 10, "dim0": 1})
+            runs = runs.chunk({"time": -1 if ufunc else 10, "dim0": 1})
 
         out = rl.last_run_before_date(
             runs, window=1, date=date, dim="time", coord=coord
@@ -432,7 +378,9 @@ class TestRunsWithDates:
         [rl.last_run_before_date, rl.run_end_after_date],
     )
     @pytest.mark.parametrize("use_dask", [True, False])
-    def test_run_with_dates_no_date(self, tas_series, use_dask, func):
+    def test_run_with_dates_no_date(self, tas_series, use_dask, func, ufunc):
+        # if use_dask and ufunc:
+        #     pytest.xfail("Ufunc run length algorithms not implemented for dask arrays.")
         t = np.ones(90)
         tas = tas_series(t, start="2000-01-01")
         runs = xr.concat((tas, tas), dim="dim0")
@@ -517,7 +465,9 @@ def test_lazy_indexing_special_cases(use_dask):
 
 
 @pytest.mark.parametrize("use_dask", [True, False])
-def test_season(use_dask, tas_series):
+def test_season(use_dask, tas_series, ufunc):
+    # if use_dask and ufunc:
+    #     pytest.xfail("Ufunc run length algorithms not implemented for dask arrays.")
     t = np.zeros(360)
     t[140:150] = 1
     tas = tas_series(t, start="2000-01-01")
@@ -525,7 +475,7 @@ def test_season(use_dask, tas_series):
     runs = runs >= 1
 
     if use_dask:
-        runs = runs.chunk({"time": 10, "dim0": 1})
+        runs = runs.chunk({"time": -1 if ufunc else 10, "dim0": 1})
 
     out = rl.season(runs, window=2)
     np.testing.assert_array_equal(out.start.load(), [140, 140])
