@@ -3,6 +3,7 @@ import pytest
 import xarray as xr
 from scipy.stats import genpareto, norm, uniform
 
+from xclim.core.options import set_options
 from xclim.core.units import convert_units_to
 from xclim.sdba.adjustment import (
     LOCI,
@@ -10,12 +11,18 @@ from xclim.sdba.adjustment import (
     DetrendedQuantileMapping,
     EmpiricalQuantileMapping,
     ExtremeValues,
+    NpdfTransform,
     PrincipalComponents,
     QuantileDeltaMapping,
     Scaling,
 )
-from xclim.sdba.base import Grouper
-from xclim.sdba.processing import jitter_under_thresh, uniform_noise_like
+from xclim.sdba.base import Grouper, stack_variables, unstack_variables
+from xclim.sdba.processing import (
+    jitter_under_thresh,
+    reordering,
+    standardize,
+    uniform_noise_like,
+)
 from xclim.sdba.utils import (
     ADDITIVE,
     MULTIPLICATIVE,
@@ -28,8 +35,8 @@ from xclim.testing import open_dataset
 from .utils import nancov
 
 
-@pytest.mark.parametrize("group,dec", (["time", 2], ["time.month", 1]))
 class TestLoci:
+    @pytest.mark.parametrize("group,dec", (["time", 2], ["time.month", 1]))
     def test_time_and_from_ds(self, series, group, dec, tmp_path):
         n = 10000
         u = np.random.rand(n)
@@ -43,16 +50,15 @@ class TestLoci:
         ref_fit = series(y, "pr").where(y > thresh, 0.1)
         ref = series(y, "pr")
 
-        loci = LOCI(group=group, thresh=thresh)
-        loci.train(ref_fit, hist)
+        loci = LOCI.train(ref_fit, hist, group=group, thresh=f"{thresh} kg m-2 s-1")
         np.testing.assert_array_almost_equal(loci.ds.hist_thresh, 1, dec)
         np.testing.assert_array_almost_equal(loci.ds.af, 2, dec)
 
         p = loci.adjust(sim)
         np.testing.assert_array_almost_equal(p, ref, dec)
 
-        assert "xclim_history" in p.attrs
-        assert "Bias-adjusted with LOCI(" in p.attrs["xclim_history"]
+        assert "history" in p.attrs
+        assert "Bias-adjusted with LOCI(" in p.attrs["history"]
 
         file = tmp_path / "test_loci.nc"
         loci.ds.to_netcdf(file)
@@ -65,6 +71,12 @@ class TestLoci:
         p2 = loci2.adjust(sim)
         np.testing.assert_array_equal(p, p2)
 
+    def test_reduce_dims(self, ref_hist_sim_tuto):
+        ref, hist, sim = ref_hist_sim_tuto()
+        hist = hist.expand_dims(member=[0, 1])
+        ref = ref.expand_dims(member=hist.member)
+        LOCI.train(ref, hist, group="time", thresh="283 K", add_dims=["member"])
+
 
 @pytest.mark.slow
 class TestScaling:
@@ -78,9 +90,10 @@ class TestScaling:
 
         hist = sim = series(x, name)
         ref = series(apply_correction(x, 2, kind), name)
+        if kind == ADDITIVE:
+            ref = convert_units_to(ref, "degC")
 
-        scaling = Scaling(group="time", kind=kind)
-        scaling.train(ref, hist)
+        scaling = Scaling.train(ref, hist, group="time", kind=kind)
         np.testing.assert_array_almost_equal(scaling.ds.af, 2)
 
         p = scaling.adjust(sim)
@@ -98,8 +111,7 @@ class TestScaling:
         ref = mon_series(apply_correction(x, 2, kind), name)
 
         # Test train
-        scaling = Scaling(group="time.month", kind=kind)
-        scaling.train(ref, hist)
+        scaling = Scaling.train(ref, hist, group="time.month", kind=kind)
         expected = apply_correction(mon_triangular, 2, kind)
         np.testing.assert_array_almost_equal(scaling.ds.af, expected)
 
@@ -107,6 +119,24 @@ class TestScaling:
         p = scaling.adjust(sim)
         np.testing.assert_array_almost_equal(p, ref)
 
+    def test_add_dim(self, series, mon_series):
+        n = 10000
+        u = np.random.rand(n, 4)
+
+        xd = uniform(loc=2, scale=1)
+        x = xd.ppf(u)
+
+        hist = sim = series(x, "tas")
+        ref = mon_series(apply_correction(x, 2, "+"), "tas")
+
+        group = Grouper("time.month", add_dims=["lon"])
+
+        scaling = Scaling.train(ref, hist, group=group, kind="+")
+        assert "lon" not in scaling.ds
+        p = scaling.adjust(sim)
+        assert "lon" in p.dims
+        np.testing.assert_array_almost_equal(p.transpose(*ref.dims), ref)
+
 
 @pytest.mark.slow
 class TestDQM:
@@ -133,12 +163,13 @@ class TestDQM:
         hist = sim = series(x, name)
         ref = series(y, name)
 
-        DQM = DetrendedQuantileMapping(
+        DQM = DetrendedQuantileMapping.train(
+            ref,
+            hist,
             kind=kind,
             group="time",
             nquantiles=50,
         )
-        DQM.train(ref, hist)
         p = DQM.adjust(sim, interp="linear")
 
         q = DQM.ds.quantiles
@@ -147,7 +178,9 @@ class TestDQM:
         expected = get_correction(ex, ey, kind)
 
         # Results are not so good at the endpoints
-        np.testing.assert_array_almost_equal(DQM.ds.af[2:-2], expected[2:-2], 1)
+        np.testing.assert_array_almost_equal(
+            DQM.ds.af[:, 2:-2], expected[np.newaxis, 2:-2], 1
+        )
 
         # Test predict
         # Accept discrepancies near extremes
@@ -203,25 +236,26 @@ class TestDQM:
         sim = series(apply_correction(x, trend, kind), name)
 
         if add_dims:
+            ref = ref.expand_dims(lat=[0, 1, 2]).chunk({"lat": 1})
             hist = hist.expand_dims(lat=[0, 1, 2]).chunk({"lat": 1})
             sim = sim.expand_dims(lat=[0, 1, 2]).chunk({"lat": 1})
             ref_t = ref_t.expand_dims(lat=[0, 1, 2])
 
-        DQM = DetrendedQuantileMapping(kind=kind, group="time.month", nquantiles=5)
-        DQM.train(ref, hist)
+        DQM = DetrendedQuantileMapping.train(
+            ref, hist, kind=kind, group="time.month", nquantiles=5
+        )
         mqm = DQM.ds.af.mean(dim="quantiles")
         p = DQM.adjust(sim)
 
         if add_dims:
             mqm = mqm.isel(lat=0)
         np.testing.assert_array_almost_equal(mqm, int(kind == MULTIPLICATIVE), 1)
-        np.testing.assert_allclose(p, ref_t, rtol=0.1, atol=0.5)
+        np.testing.assert_allclose(p.transpose(..., "time"), ref_t, rtol=0.1, atol=0.5)
 
     def test_cannon_and_from_ds(self, cannon_2015_rvs, tmp_path):
         ref, hist, sim = cannon_2015_rvs(15000)
 
-        DQM = DetrendedQuantileMapping(kind="*", group="time")
-        DQM.train(ref, hist)
+        DQM = DetrendedQuantileMapping.train(ref, hist, kind="*", group="time")
         p = DQM.adjust(sim)
 
         np.testing.assert_almost_equal(p.mean(), 41.6, 0)
@@ -262,25 +296,30 @@ class TestQDM:
         hist = sim = series(x, name)
         ref = series(y, name)
 
-        QDM = QuantileDeltaMapping(
+        QDM = QuantileDeltaMapping.train(
+            ref.astype("float32"),
+            hist.astype("float32"),
             kind=kind,
             group="time",
             nquantiles=10,
         )
-        QDM.train(ref, hist)
-        p = QDM.adjust(sim, interp="linear")
+        p = QDM.adjust(sim.astype("float32"), interp="linear")
 
         q = QDM.ds.coords["quantiles"]
-        expected = get_correction(xd.ppf(q), yd.ppf(q), kind)
+        expected = get_correction(xd.ppf(q), yd.ppf(q), kind)[np.newaxis, :]
 
         # Results are not so good at the endpoints
-        np.testing.assert_array_almost_equal(QDM.ds.af.T, expected, 1)
+        np.testing.assert_array_almost_equal(QDM.ds.af, expected, 1)
 
         # Test predict
         # Accept discrepancies near extremes
         middle = (u > 1e-2) * (u < 0.99)
         np.testing.assert_array_almost_equal(p[middle], ref[middle], 1)
 
+        # Test dtype control of map_blocks
+        assert QDM.ds.af.dtype == "float32"
+        assert p.dtype == "float32"
+
     @pytest.mark.parametrize("use_dask", [True, False])
     @pytest.mark.parametrize("kind,name", [(ADDITIVE, "tas"), (MULTIPLICATIVE, "pr")])
     @pytest.mark.parametrize("add_dims", [True, False])
@@ -306,20 +345,21 @@ class TestQDM:
         y = yd.ppf(u) + noise.ppf(u)
 
         # Test train
+        ref = mon_series(y, name)
         hist = sim = series(x, name)
         if use_dask:
             sim = sim.chunk({"time": -1})
         if add_dims:
+            ref = ref.expand_dims(site=[0, 1, 2, 3, 4])
             hist = hist.expand_dims(site=[0, 1, 2, 3, 4])
             sim = sim.expand_dims(site=[0, 1, 2, 3, 4])
             sel = {"site": 0}
         else:
             sel = {}
 
-        ref = mon_series(y, name)
-
-        QDM = QuantileDeltaMapping(kind=kind, group="time.month", nquantiles=40)
-        QDM.train(ref, hist)
+        QDM = QuantileDeltaMapping.train(
+            ref, hist, kind=kind, group="time.month", nquantiles=40
+        )
         p = QDM.adjust(sim, interp="linear" if kind == "+" else "nearest")
 
         q = QDM.ds.coords["quantiles"]
@@ -333,16 +373,22 @@ class TestQDM:
         )
 
         # Test predict
-        np.testing.assert_allclose(p.isel(**sel), ref, rtol=0.1, atol=0.2)
-        # np.testing.assert_array_almost_equal(p.isel(**sel), ref, 1)
+        np.testing.assert_allclose(p, ref.transpose(*p.dims), rtol=0.1, atol=0.2)
 
-    def test_cannon(self, cannon_2015_dist, cannon_2015_rvs):
+    def test_cannon_and_diagnostics(self, cannon_2015_dist, cannon_2015_rvs):
         ref, hist, sim = cannon_2015_rvs(15000, random=False)
 
         # Quantile mapping
-        QDM = QuantileDeltaMapping(kind="*", group="time", nquantiles=50)
-        QDM.train(ref, hist)
-        bc_sim = QDM.adjust(sim)
+        with set_options(sdba_extra_output=True):
+            QDM = QuantileDeltaMapping.train(
+                ref, hist, kind="*", group="time", nquantiles=50
+            )
+            scends = QDM.adjust(sim)
+
+        assert isinstance(scends, xr.Dataset)
+
+        sim_q_exp = sim.rank(dim="time", pct=True)
+        np.testing.assert_array_equal(sim_q_exp, scends.sim_q)
 
         # Theoretical results
         # ref, hist, sim = cannon_2015_dist
@@ -355,7 +401,7 @@ class TestQDM:
         # mean = np.trapz(pdf * pu, pu)
         # mom2 = np.trapz(pdf * pu ** 2, pu)
         # std = np.sqrt(mom2 - mean ** 2)
-
+        bc_sim = scends.scen
         np.testing.assert_almost_equal(bc_sim.mean(), 41.5, 1)
         np.testing.assert_almost_equal(bc_sim.std(), 16.7, 0)
 
@@ -383,19 +429,19 @@ class TestQM:
         # Test train
         hist = sim = series(x, name)
         ref = series(y, name)
-        QM = EmpiricalQuantileMapping(
+        QM = EmpiricalQuantileMapping.train(
+            ref,
+            hist,
             kind=kind,
             group="time",
             nquantiles=50,
         )
-        QM.train(ref, hist)
         p = QM.adjust(sim, interp="linear")
 
         q = QM.ds.coords["quantiles"]
-        expected = get_correction(xd.ppf(q), yd.ppf(q), kind)
-
+        expected = get_correction(xd.ppf(q), yd.ppf(q), kind)[np.newaxis, :]
         # Results are not so good at the endpoints
-        np.testing.assert_array_almost_equal(QM.ds.af[2:-2], expected[2:-2], 1)
+        np.testing.assert_array_almost_equal(QM.ds.af[:, 2:-2], expected[:, 2:-2], 1)
 
         # Test predict
         # Accept discrepancies near extremes
@@ -426,8 +472,9 @@ class TestQM:
         hist = sim = series(x, name)
         ref = mon_series(y, name)
 
-        QM = EmpiricalQuantileMapping(kind=kind, group="time.month", nquantiles=5)
-        QM.train(ref, hist)
+        QM = EmpiricalQuantileMapping.train(
+            ref, hist, kind=kind, group="time.month", nquantiles=5
+        )
         p = QM.adjust(sim)
         mqm = QM.ds.af.mean(dim="quantiles")
         expected = apply_correction(mon_triangular, 2, kind)
@@ -436,6 +483,39 @@ class TestQM:
         # Test predict
         np.testing.assert_array_almost_equal(p, ref, 2)
 
+    @pytest.mark.parametrize("use_dask", [True, False])
+    def test_add_dims(self, use_dask):
+        if use_dask:
+            chunks = {"location": -1}
+        else:
+            chunks = None
+        ref = (
+            open_dataset(
+                "sdba/ahccd_1950-2013.nc", chunks=chunks, drop_variables=["lat", "lon"]
+            )
+            .sel(time=slice("1981", "2010"))
+            .tasmax
+        )
+        ref = convert_units_to(ref, "K")
+        ref = ref.isel(location=1, drop=True).expand_dims(location=["Amos"])
+
+        dsim = open_dataset(
+            "sdba/CanESM2_1950-2100.nc", chunks=chunks, drop_variables=["lat", "lon"]
+        ).tasmax
+        hist = dsim.sel(time=slice("1981", "2010"))
+        sim = dsim.sel(time=slice("2041", "2070"))
+
+        # With add_dims, "does it run" test
+        group = Grouper("time.dayofyear", window=5, add_dims=["location"])
+        EQM = EmpiricalQuantileMapping.train(ref, hist, group=group)
+        EQM.adjust(sim).load()
+
+        # Without, sanity test.
+        group = Grouper("time.dayofyear", window=5)
+        EQM2 = EmpiricalQuantileMapping.train(ref, hist, group=group)
+        scen2 = EQM2.adjust(sim).load()
+        assert scen2.sel(location=["Kugluktuk", "Vancouver"]).isnull().all()
+
 
 class TestPrincipalComponents:
     @pytest.mark.parametrize(
@@ -456,13 +536,18 @@ class TestPrincipalComponents:
         sim_x = norm.rvs(loc=4, scale=2, size=(m, n))
         sim_y = sim_x + norm.rvs(loc=1, scale=1, size=(m, n))
 
-        ref = xr.DataArray([ref_x, ref_y], dims=("lat", "lon", "time"))
+        ref = xr.DataArray(
+            [ref_x, ref_y], dims=("lat", "lon", "time"), attrs={"units": "degC"}
+        )
         ref["time"] = xr.cftime_range("1990-01-01", periods=n, calendar="noleap")
-        sim = xr.DataArray([sim_x, sim_y], dims=("lat", "lon", "time"))
+        sim = xr.DataArray(
+            [sim_x, sim_y], dims=("lat", "lon", "time"), attrs={"units": "degC"}
+        )
         sim["time"] = ref["time"]
 
-        PCA = PrincipalComponents(group=group, crd_dims=crd_dims, pts_dims=pts_dims)
-        PCA.train(ref, sim)
+        PCA = PrincipalComponents.train(
+            ref, sim, group=group, crd_dims=crd_dims, pts_dims=pts_dims
+        )
         scen = PCA.adjust(sim)
 
         group = group if isinstance(group, Grouper) else Grouper("time")
@@ -504,8 +589,7 @@ class TestPrincipalComponents:
         ref = ds.air.isel(lat=21, lon=[40, 52]).drop_vars(["lon", "lat"])
         sim = ds.air.isel(lat=18, lon=[17, 35]).drop_vars(["lon", "lat"])
 
-        PCA = PrincipalComponents(group=group)
-        PCA.train(ref, sim)
+        PCA = PrincipalComponents.train(ref, sim, group=group)
         scen = PCA.adjust(sim)
 
         def dist(ref, sim):
@@ -519,16 +603,17 @@ class TestPrincipalComponents:
         assert (ref - scen).mean() < 5e-3
 
 
+@pytest.mark.xfail(reason="Inaccurate version.")
 class TestExtremeValues:
     @pytest.mark.parametrize(
-        "c_thresh,q_thresh,frac,power",
+        "c_thresh,q_thresh,frac,power,diags",
         [
-            ["1 mm/d", 0.95, 0.25, 1],
-            ["1 mm/d", 0.90, 1e-6, 1],
-            ["0.007 m/week", 0.95, 0.25, 2],
+            ["1 mm/d", 0.95, 0.25, 1, True],
+            ["1 mm/d", 0.90, 1e-6, 1, False],
+            ["0.007 m/week", 0.95, 0.25, 2, False],
         ],
     )
-    def test_simple(self, c_thresh, q_thresh, frac, power):
+    def test_simple(self, c_thresh, q_thresh, frac, power, diags):
         n = 45 * 365
 
         def gen_testdata(c, s):
@@ -550,12 +635,17 @@ class TestExtremeValues:
         hist = jitter_under_thresh(gen_testdata(-0.1, 2), 1e-3)
         sim = gen_testdata(-0.15, 2.5)
 
-        EQM = EmpiricalQuantileMapping(group="time.dayofyear", nquantiles=15, kind="*")
-        EQM.train(ref, hist)
+        EQM = EmpiricalQuantileMapping.train(
+            ref, hist, group="time.dayofyear", nquantiles=15, kind="*"
+        )
+
         scen = EQM.adjust(sim)
 
-        EX = ExtremeValues(c_thresh, q_thresh=q_thresh)
-        EX.train(ref, hist)
+        with set_options(sdba_extra_output=diags):
+            EX = ExtremeValues.train(ref, hist, c_thresh, q_thresh=q_thresh)
+
+        if diags:
+            assert "nclusters" in EX.ds
 
         qv = (ref.thresh + hist.thresh) / 2
         np.testing.assert_allclose(EX.ds.fit_params, [-0.1, qv, 2], atol=0.5, rtol=0.1)
@@ -572,7 +662,7 @@ class TestExtremeValues:
         # ONLY extreme values have been touched (but some might not have been modified)
         assert (((scen != scen2) | exval) == exval).all()
 
-    def test_dask_julia(self):
+    def test_dask_julia_diags(self):
 
         dsim = open_dataset("sdba/CanESM2_1950-2100.nc").chunk()
         dref = open_dataset("sdba/ahccd_1950-2013.nc").chunk()
@@ -583,19 +673,17 @@ class TestExtremeValues:
 
         quantiles = np.linspace(0.01, 0.99, num=50)
 
-        EQM = EmpiricalQuantileMapping(
-            group=Grouper("time.dayofyear", window=31), nquantiles=quantiles
-        )
-
         with xr.set_options(keep_attrs=True):
             ref = ref + uniform_noise_like(ref, low=1e-6, high=1e-3)
             hist = hist + uniform_noise_like(hist, low=1e-6, high=1e-3)
 
-        EQM.train(ref, hist)
+        EQM = EmpiricalQuantileMapping.train(
+            ref, hist, group=Grouper("time.dayofyear", window=31), nquantiles=quantiles
+        )
+
         scen = EQM.adjust(hist, interp="linear", extrapolation="constant")
 
-        EX = ExtremeValues(cluster_thresh="1 mm/day", q_thresh=0.97)
-        EX.train(ref, hist)
+        EX = ExtremeValues.train(ref, hist, cluster_thresh="1 mm/day", q_thresh=0.97)
         new_scen = EX.adjust(scen, hist, frac=0.000000001)
 
         new_scen.load()
@@ -611,6 +699,5 @@ class TestExtremeValues:
 
 def test_raise_on_multiple_chunks(tas_series):
     ref = tas_series(np.arange(730)).chunk({"time": 365})
-    Adj = BaseAdjustment(group=Grouper("time.month"))
     with pytest.raises(ValueError):
-        Adj.train(ref, ref)
+        EmpiricalQuantileMapping.train(ref, ref, group=Grouper("time.month"))
